/*
 * nghttp2 - HTTP/2.0 C Library
 *
 * Copyright (c) 2013 Tatsuhiro Tsujikawa
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include "nghttp2_hd_huffman.h"

const nghttp2_huff_sym req_huff_sym_table[] = {
  { 27, 0x7ffffbau },
  { 27, 0x7ffffbbu },
  { 27, 0x7ffffbcu },
  { 27, 0x7ffffbdu },
  { 27, 0x7ffffbeu },
  { 27, 0x7ffffbfu },
  { 27, 0x7ffffc0u },
  { 27, 0x7ffffc1u },
  { 27, 0x7ffffc2u },
  { 27, 0x7ffffc3u },
  { 27, 0x7ffffc4u },
  { 27, 0x7ffffc5u },
  { 27, 0x7ffffc6u },
  { 27, 0x7ffffc7u },
  { 27, 0x7ffffc8u },
  { 27, 0x7ffffc9u },
  { 27, 0x7ffffcau },
  { 27, 0x7ffffcbu },
  { 27, 0x7ffffccu },
  { 27, 0x7ffffcdu },
  { 27, 0x7ffffceu },
  { 27, 0x7ffffcfu },
  { 27, 0x7ffffd0u },
  { 27, 0x7ffffd1u },
  { 27, 0x7ffffd2u },
  { 27, 0x7ffffd3u },
  { 27, 0x7ffffd4u },
  { 27, 0x7ffffd5u },
  { 27, 0x7ffffd6u },
  { 27, 0x7ffffd7u },
  { 27, 0x7ffffd8u },
  { 27, 0x7ffffd9u },
  { 8, 0xe8u },
  { 12, 0xffcu },
  { 14, 0x3ffau },
  { 15, 0x7ffcu },
  { 15, 0x7ffdu },
  { 6, 0x24u },
  { 7, 0x6eu },
  { 15, 0x7ffeu },
  { 11, 0x7fau },
  { 11, 0x7fbu },
  { 10, 0x3fau },
  { 11, 0x7fcu },
  { 8, 0xe9u },
  { 6, 0x25u },
  { 5, 0x4u },
  { 4, 0x0u },
  { 5, 0x5u },
  { 5, 0x6u },
  { 5, 0x7u },
  { 6, 0x26u },
  { 6, 0x27u },
  { 6, 0x28u },
  { 6, 0x29u },
  { 6, 0x2au },
  { 6, 0x2bu },
  { 6, 0x2cu },
  { 9, 0x1ecu },
  { 8, 0xeau },
  { 18, 0x3fffeu },
  { 6, 0x2du },
  { 17, 0x1fffcu },
  { 9, 0x1edu },
  { 14, 0x3ffbu },
  { 7, 0x6fu },
  { 8, 0xebu },
  { 8, 0xecu },
  { 8, 0xedu },
  { 8, 0xeeu },
  { 7, 0x70u },
  { 9, 0x1eeu },
  { 9, 0x1efu },
  { 9, 0x1f0u },
  { 9, 0x1f1u },
  { 10, 0x3fbu },
  { 9, 0x1f2u },
  { 8, 0xefu },
  { 9, 0x1f3u },
  { 9, 0x1f4u },
  { 9, 0x1f5u },
  { 9, 0x1f6u },
  { 9, 0x1f7u },
  { 8, 0xf0u },
  { 8, 0xf1u },
  { 9, 0x1f8u },
  { 9, 0x1f9u },
  { 9, 0x1fau },
  { 9, 0x1fbu },
  { 9, 0x1fcu },
  { 10, 0x3fcu },
  { 14, 0x3ffcu },
  { 27, 0x7ffffdau },
  { 13, 0x1ffcu },
  { 14, 0x3ffdu },
  { 6, 0x2eu },
  { 19, 0x7fffeu },
  { 5, 0x8u },
  { 6, 0x2fu },
  { 5, 0x9u },
  { 6, 0x30u },
  { 4, 0x1u },
  { 6, 0x31u },
  { 6, 0x32u },
  { 6, 0x33u },
  { 5, 0xau },
  { 7, 0x71u },
  { 7, 0x72u },
  { 5, 0xbu },
  { 6, 0x34u },
  { 5, 0xcu },
  { 5, 0xdu },
  { 5, 0xeu },
  { 8, 0xf2u },
  { 5, 0xfu },
  { 5, 0x10u },
  { 5, 0x11u },
  { 6, 0x35u },
  { 7, 0x73u },
  { 6, 0x36u },
  { 8, 0xf3u },
  { 8, 0xf4u },
  { 8, 0xf5u },
  { 17, 0x1fffdu },
  { 11, 0x7fdu },
  { 17, 0x1fffeu },
  { 12, 0xffdu },
  { 27, 0x7ffffdbu },
  { 27, 0x7ffffdcu },
  { 27, 0x7ffffddu },
  { 27, 0x7ffffdeu },
  { 27, 0x7ffffdfu },
  { 27, 0x7ffffe0u },
  { 27, 0x7ffffe1u },
  { 27, 0x7ffffe2u },
  { 27, 0x7ffffe3u },
  { 27, 0x7ffffe4u },
  { 27, 0x7ffffe5u },
  { 27, 0x7ffffe6u },
  { 27, 0x7ffffe7u },
  { 27, 0x7ffffe8u },
  { 27, 0x7ffffe9u },
  { 27, 0x7ffffeau },
  { 27, 0x7ffffebu },
  { 27, 0x7ffffecu },
  { 27, 0x7ffffedu },
  { 27, 0x7ffffeeu },
  { 27, 0x7ffffefu },
  { 27, 0x7fffff0u },
  { 27, 0x7fffff1u },
  { 27, 0x7fffff2u },
  { 27, 0x7fffff3u },
  { 27, 0x7fffff4u },
  { 27, 0x7fffff5u },
  { 27, 0x7fffff6u },
  { 27, 0x7fffff7u },
  { 27, 0x7fffff8u },
  { 27, 0x7fffff9u },
  { 27, 0x7fffffau },
  { 27, 0x7fffffbu },
  { 27, 0x7fffffcu },
  { 27, 0x7fffffdu },
  { 27, 0x7fffffeu },
  { 27, 0x7ffffffu },
  { 26, 0x3ffff80u },
  { 26, 0x3ffff81u },
  { 26, 0x3ffff82u },
  { 26, 0x3ffff83u },
  { 26, 0x3ffff84u },
  { 26, 0x3ffff85u },
  { 26, 0x3ffff86u },
  { 26, 0x3ffff87u },
  { 26, 0x3ffff88u },
  { 26, 0x3ffff89u },
  { 26, 0x3ffff8au },
  { 26, 0x3ffff8bu },
  { 26, 0x3ffff8cu },
  { 26, 0x3ffff8du },
  { 26, 0x3ffff8eu },
  { 26, 0x3ffff8fu },
  { 26, 0x3ffff90u },
  { 26, 0x3ffff91u },
  { 26, 0x3ffff92u },
  { 26, 0x3ffff93u },
  { 26, 0x3ffff94u },
  { 26, 0x3ffff95u },
  { 26, 0x3ffff96u },
  { 26, 0x3ffff97u },
  { 26, 0x3ffff98u },
  { 26, 0x3ffff99u },
  { 26, 0x3ffff9au },
  { 26, 0x3ffff9bu },
  { 26, 0x3ffff9cu },
  { 26, 0x3ffff9du },
  { 26, 0x3ffff9eu },
  { 26, 0x3ffff9fu },
  { 26, 0x3ffffa0u },
  { 26, 0x3ffffa1u },
  { 26, 0x3ffffa2u },
  { 26, 0x3ffffa3u },
  { 26, 0x3ffffa4u },
  { 26, 0x3ffffa5u },
  { 26, 0x3ffffa6u },
  { 26, 0x3ffffa7u },
  { 26, 0x3ffffa8u },
  { 26, 0x3ffffa9u },
  { 26, 0x3ffffaau },
  { 26, 0x3ffffabu },
  { 26, 0x3ffffacu },
  { 26, 0x3ffffadu },
  { 26, 0x3ffffaeu },
  { 26, 0x3ffffafu },
  { 26, 0x3ffffb0u },
  { 26, 0x3ffffb1u },
  { 26, 0x3ffffb2u },
  { 26, 0x3ffffb3u },
  { 26, 0x3ffffb4u },
  { 26, 0x3ffffb5u },
  { 26, 0x3ffffb6u },
  { 26, 0x3ffffb7u },
  { 26, 0x3ffffb8u },
  { 26, 0x3ffffb9u },
  { 26, 0x3ffffbau },
  { 26, 0x3ffffbbu },
  { 26, 0x3ffffbcu },
  { 26, 0x3ffffbdu },
  { 26, 0x3ffffbeu },
  { 26, 0x3ffffbfu },
  { 26, 0x3ffffc0u },
  { 26, 0x3ffffc1u },
  { 26, 0x3ffffc2u },
  { 26, 0x3ffffc3u },
  { 26, 0x3ffffc4u },
  { 26, 0x3ffffc5u },
  { 26, 0x3ffffc6u },
  { 26, 0x3ffffc7u },
  { 26, 0x3ffffc8u },
  { 26, 0x3ffffc9u },
  { 26, 0x3ffffcau },
  { 26, 0x3ffffcbu },
  { 26, 0x3ffffccu },
  { 26, 0x3ffffcdu },
  { 26, 0x3ffffceu },
  { 26, 0x3ffffcfu },
  { 26, 0x3ffffd0u },
  { 26, 0x3ffffd1u },
  { 26, 0x3ffffd2u },
  { 26, 0x3ffffd3u },
  { 26, 0x3ffffd4u },
  { 26, 0x3ffffd5u },
  { 26, 0x3ffffd6u },
  { 26, 0x3ffffd7u },
  { 26, 0x3ffffd8u },
  { 26, 0x3ffffd9u },
  { 26, 0x3ffffdau },
  { 26, 0x3ffffdbu },
  { 26, 0x3ffffdcu }
};

const nghttp2_huff_sym res_huff_sym_table[] = {
  { 25, 0x1ffffbcu },
  { 25, 0x1ffffbdu },
  { 25, 0x1ffffbeu },
  { 25, 0x1ffffbfu },
  { 25, 0x1ffffc0u },
  { 25, 0x1ffffc1u },
  { 25, 0x1ffffc2u },
  { 25, 0x1ffffc3u },
  { 25, 0x1ffffc4u },
  { 25, 0x1ffffc5u },
  { 25, 0x1ffffc6u },
  { 25, 0x1ffffc7u },
  { 25, 0x1ffffc8u },
  { 25, 0x1ffffc9u },
  { 25, 0x1ffffcau },
  { 25, 0x1ffffcbu },
  { 25, 0x1ffffccu },
  { 25, 0x1ffffcdu },
  { 25, 0x1ffffceu },
  { 25, 0x1ffffcfu },
  { 25, 0x1ffffd0u },
  { 25, 0x1ffffd1u },
  { 25, 0x1ffffd2u },
  { 25, 0x1ffffd3u },
  { 25, 0x1ffffd4u },
  { 25, 0x1ffffd5u },
  { 25, 0x1ffffd6u },
  { 25, 0x1ffffd7u },
  { 25, 0x1ffffd8u },
  { 25, 0x1ffffd9u },
  { 25, 0x1ffffdau },
  { 25, 0x1ffffdbu },
  { 4, 0x0u },
  { 12, 0xffau },
  { 7, 0x6au },
  { 13, 0x1ffau },
  { 14, 0x3ffcu },
  { 9, 0x1ecu },
  { 10, 0x3f8u },
  { 13, 0x1ffbu },
  { 9, 0x1edu },
  { 9, 0x1eeu },
  { 12, 0xffbu },
  { 11, 0x7fau },
  { 6, 0x22u },
  { 6, 0x23u },
  { 6, 0x24u },
  { 7, 0x6bu },
  { 4, 0x1u },
  { 4, 0x2u },
  { 4, 0x3u },
  { 5, 0x8u },
  { 5, 0x9u },
  { 5, 0xau },
  { 6, 0x25u },
  { 6, 0x26u },
  { 5, 0xbu },
  { 5, 0xcu },
  { 5, 0xdu },
  { 9, 0x1efu },
  { 16, 0xfffau },
  { 7, 0x6cu },
  { 13, 0x1ffcu },
  { 12, 0xffcu },
  { 16, 0xfffbu },
  { 7, 0x6du },
  { 8, 0xeau },
  { 8, 0xebu },
  { 8, 0xecu },
  { 8, 0xedu },
  { 8, 0xeeu },
  { 6, 0x27u },
  { 9, 0x1f0u },
  { 8, 0xefu },
  { 8, 0xf0u },
  { 10, 0x3f9u },
  { 9, 0x1f1u },
  { 6, 0x28u },
  { 8, 0xf1u },
  { 8, 0xf2u },
  { 9, 0x1f2u },
  { 10, 0x3fau },
  { 9, 0x1f3u },
  { 6, 0x29u },
  { 5, 0xeu },
  { 9, 0x1f4u },
  { 9, 0x1f5u },
  { 8, 0xf3u },
  { 10, 0x3fbu },
  { 9, 0x1f6u },
  { 10, 0x3fcu },
  { 11, 0x7fbu },
  { 13, 0x1ffdu },
  { 11, 0x7fcu },
  { 15, 0x7ffcu },
  { 9, 0x1f7u },
  { 17, 0x1fffeu },
  { 5, 0xfu },
  { 7, 0x6eu },
  { 6, 0x2au },
  { 6, 0x2bu },
  { 5, 0x10u },
  { 7, 0x6fu },
  { 7, 0x70u },
  { 7, 0x71u },
  { 6, 0x2cu },
  { 9, 0x1f8u },
  { 9, 0x1f9u },
  { 7, 0x72u },
  { 6, 0x2du },
  { 6, 0x2eu },
  { 6, 0x2fu },
  { 6, 0x30u },
  { 9, 0x1fau },
  { 6, 0x31u },
  { 6, 0x32u },
  { 6, 0x33u },
  { 6, 0x34u },
  { 7, 0x73u },
  { 8, 0xf4u },
  { 7, 0x74u },
  { 8, 0xf5u },
  { 9, 0x1fbu },
  { 16, 0xfffcu },
  { 14, 0x3ffdu },
  { 16, 0xfffdu },
  { 16, 0xfffeu },
  { 25, 0x1ffffdcu },
  { 25, 0x1ffffddu },
  { 25, 0x1ffffdeu },
  { 25, 0x1ffffdfu },
  { 25, 0x1ffffe0u },
  { 25, 0x1ffffe1u },
  { 25, 0x1ffffe2u },
  { 25, 0x1ffffe3u },
  { 25, 0x1ffffe4u },
  { 25, 0x1ffffe5u },
  { 25, 0x1ffffe6u },
  { 25, 0x1ffffe7u },
  { 25, 0x1ffffe8u },
  { 25, 0x1ffffe9u },
  { 25, 0x1ffffeau },
  { 25, 0x1ffffebu },
  { 25, 0x1ffffecu },
  { 25, 0x1ffffedu },
  { 25, 0x1ffffeeu },
  { 25, 0x1ffffefu },
  { 25, 0x1fffff0u },
  { 25, 0x1fffff1u },
  { 25, 0x1fffff2u },
  { 25, 0x1fffff3u },
  { 25, 0x1fffff4u },
  { 25, 0x1fffff5u },
  { 25, 0x1fffff6u },
  { 25, 0x1fffff7u },
  { 25, 0x1fffff8u },
  { 25, 0x1fffff9u },
  { 25, 0x1fffffau },
  { 25, 0x1fffffbu },
  { 25, 0x1fffffcu },
  { 25, 0x1fffffdu },
  { 25, 0x1fffffeu },
  { 25, 0x1ffffffu },
  { 24, 0xffff80u },
  { 24, 0xffff81u },
  { 24, 0xffff82u },
  { 24, 0xffff83u },
  { 24, 0xffff84u },
  { 24, 0xffff85u },
  { 24, 0xffff86u },
  { 24, 0xffff87u },
  { 24, 0xffff88u },
  { 24, 0xffff89u },
  { 24, 0xffff8au },
  { 24, 0xffff8bu },
  { 24, 0xffff8cu },
  { 24, 0xffff8du },
  { 24, 0xffff8eu },
  { 24, 0xffff8fu },
  { 24, 0xffff90u },
  { 24, 0xffff91u },
  { 24, 0xffff92u },
  { 24, 0xffff93u },
  { 24, 0xffff94u },
  { 24, 0xffff95u },
  { 24, 0xffff96u },
  { 24, 0xffff97u },
  { 24, 0xffff98u },
  { 24, 0xffff99u },
  { 24, 0xffff9au },
  { 24, 0xffff9bu },
  { 24, 0xffff9cu },
  { 24, 0xffff9du },
  { 24, 0xffff9eu },
  { 24, 0xffff9fu },
  { 24, 0xffffa0u },
  { 24, 0xffffa1u },
  { 24, 0xffffa2u },
  { 24, 0xffffa3u },
  { 24, 0xffffa4u },
  { 24, 0xffffa5u },
  { 24, 0xffffa6u },
  { 24, 0xffffa7u },
  { 24, 0xffffa8u },
  { 24, 0xffffa9u },
  { 24, 0xffffaau },
  { 24, 0xffffabu },
  { 24, 0xffffacu },
  { 24, 0xffffadu },
  { 24, 0xffffaeu },
  { 24, 0xffffafu },
  { 24, 0xffffb0u },
  { 24, 0xffffb1u },
  { 24, 0xffffb2u },
  { 24, 0xffffb3u },
  { 24, 0xffffb4u },
  { 24, 0xffffb5u },
  { 24, 0xffffb6u },
  { 24, 0xffffb7u },
  { 24, 0xffffb8u },
  { 24, 0xffffb9u },
  { 24, 0xffffbau },
  { 24, 0xffffbbu },
  { 24, 0xffffbcu },
  { 24, 0xffffbdu },
  { 24, 0xffffbeu },
  { 24, 0xffffbfu },
  { 24, 0xffffc0u },
  { 24, 0xffffc1u },
  { 24, 0xffffc2u },
  { 24, 0xffffc3u },
  { 24, 0xffffc4u },
  { 24, 0xffffc5u },
  { 24, 0xffffc6u },
  { 24, 0xffffc7u },
  { 24, 0xffffc8u },
  { 24, 0xffffc9u },
  { 24, 0xffffcau },
  { 24, 0xffffcbu },
  { 24, 0xffffccu },
  { 24, 0xffffcdu },
  { 24, 0xffffceu },
  { 24, 0xffffcfu },
  { 24, 0xffffd0u },
  { 24, 0xffffd1u },
  { 24, 0xffffd2u },
  { 24, 0xffffd3u },
  { 24, 0xffffd4u },
  { 24, 0xffffd5u },
  { 24, 0xffffd6u },
  { 24, 0xffffd7u },
  { 24, 0xffffd8u },
  { 24, 0xffffd9u },
  { 24, 0xffffdau },
  { 24, 0xffffdbu },
  { 24, 0xffffdcu },
  { 24, 0xffffddu }
};

const nghttp2_huff_decode req_huff_decode_table[][16] = {
/* 0 */
{
  {0, 0x03, 47},
  {0, 0x03, 101},
  {5, 0x00, 0},
  {6, 0x00, 0},
  {9, 0x00, 0},
  {10, 0x00, 0},
  {12, 0x00, 0},
  {13, 0x00, 0},
  {17, 0x00, 0},
  {18, 0x00, 0},
  {22, 0x00, 0},
  {25, 0x00, 0},
  {30, 0x00, 0},
  {33, 0x00, 0},
  {38, 0x00, 0},
  {49, 0x01, 0},
},
/* 1 */
{
  {1, 0x02, 47},
  {14, 0x03, 47},
  {1, 0x02, 101},
  {14, 0x03, 101},
  {0, 0x03, 46},
  {0, 0x03, 48},
  {0, 0x03, 49},
  {0, 0x03, 50},
  {0, 0x03, 97},
  {0, 0x03, 99},
  {0, 0x03, 105},
  {0, 0x03, 108},
  {0, 0x03, 110},
  {0, 0x03, 111},
  {0, 0x03, 112},
  {0, 0x03, 114},
},
/* 2 */
{
  {2, 0x02, 47},
  {7, 0x02, 47},
  {15, 0x02, 47},
  {28, 0x03, 47},
  {2, 0x02, 101},
  {7, 0x02, 101},
  {15, 0x02, 101},
  {28, 0x03, 101},
  {1, 0x02, 46},
  {14, 0x03, 46},
  {1, 0x02, 48},
  {14, 0x03, 48},
  {1, 0x02, 49},
  {14, 0x03, 49},
  {1, 0x02, 50},
  {14, 0x03, 50},
},
/* 3 */
{
  {3, 0x02, 47},
  {4, 0x02, 47},
  {8, 0x02, 47},
  {11, 0x02, 47},
  {16, 0x02, 47},
  {21, 0x02, 47},
  {29, 0x02, 47},
  {37, 0x03, 47},
  {3, 0x02, 101},
  {4, 0x02, 101},
  {8, 0x02, 101},
  {11, 0x02, 101},
  {16, 0x02, 101},
  {21, 0x02, 101},
  {29, 0x02, 101},
  {37, 0x03, 101},
},
/* 4 */
{
  {2, 0x02, 46},
  {7, 0x02, 46},
  {15, 0x02, 46},
  {28, 0x03, 46},
  {2, 0x02, 48},
  {7, 0x02, 48},
  {15, 0x02, 48},
  {28, 0x03, 48},
  {2, 0x02, 49},
  {7, 0x02, 49},
  {15, 0x02, 49},
  {28, 0x03, 49},
  {2, 0x02, 50},
  {7, 0x02, 50},
  {15, 0x02, 50},
  {28, 0x03, 50},
},
/* 5 */
{
  {3, 0x02, 46},
  {4, 0x02, 46},
  {8, 0x02, 46},
  {11, 0x02, 46},
  {16, 0x02, 46},
  {21, 0x02, 46},
  {29, 0x02, 46},
  {37, 0x03, 46},
  {3, 0x02, 48},
  {4, 0x02, 48},
  {8, 0x02, 48},
  {11, 0x02, 48},
  {16, 0x02, 48},
  {21, 0x02, 48},
  {29, 0x02, 48},
  {37, 0x03, 48},
},
/* 6 */
{
  {3, 0x02, 49},
  {4, 0x02, 49},
  {8, 0x02, 49},
  {11, 0x02, 49},
  {16, 0x02, 49},
  {21, 0x02, 49},
  {29, 0x02, 49},
  {37, 0x03, 49},
  {3, 0x02, 50},
  {4, 0x02, 50},
  {8, 0x02, 50},
  {11, 0x02, 50},
  {16, 0x02, 50},
  {21, 0x02, 50},
  {29, 0x02, 50},
  {37, 0x03, 50},
},
/* 7 */
{
  {1, 0x02, 97},
  {14, 0x03, 97},
  {1, 0x02, 99},
  {14, 0x03, 99},
  {1, 0x02, 105},
  {14, 0x03, 105},
  {1, 0x02, 108},
  {14, 0x03, 108},
  {1, 0x02, 110},
  {14, 0x03, 110},
  {1, 0x02, 111},
  {14, 0x03, 111},
  {1, 0x02, 112},
  {14, 0x03, 112},
  {1, 0x02, 114},
  {14, 0x03, 114},
},
/* 8 */
{
  {2, 0x02, 97},
  {7, 0x02, 97},
  {15, 0x02, 97},
  {28, 0x03, 97},
  {2, 0x02, 99},
  {7, 0x02, 99},
  {15, 0x02, 99},
  {28, 0x03, 99},
  {2, 0x02, 105},
  {7, 0x02, 105},
  {15, 0x02, 105},
  {28, 0x03, 105},
  {2, 0x02, 108},
  {7, 0x02, 108},
  {15, 0x02, 108},
  {28, 0x03, 108},
},
/* 9 */
{
  {3, 0x02, 97},
  {4, 0x02, 97},
  {8, 0x02, 97},
  {11, 0x02, 97},
  {16, 0x02, 97},
  {21, 0x02, 97},
  {29, 0x02, 97},
  {37, 0x03, 97},
  {3, 0x02, 99},
  {4, 0x02, 99},
  {8, 0x02, 99},
  {11, 0x02, 99},
  {16, 0x02, 99},
  {21, 0x02, 99},
  {29, 0x02, 99},
  {37, 0x03, 99},
},
/* 10 */
{
  {3, 0x02, 105},
  {4, 0x02, 105},
  {8, 0x02, 105},
  {11, 0x02, 105},
  {16, 0x02, 105},
  {21, 0x02, 105},
  {29, 0x02, 105},
  {37, 0x03, 105},
  {3, 0x02, 108},
  {4, 0x02, 108},
  {8, 0x02, 108},
  {11, 0x02, 108},
  {16, 0x02, 108},
  {21, 0x02, 108},
  {29, 0x02, 108},
  {37, 0x03, 108},
},
/* 11 */
{
  {2, 0x02, 110},
  {7, 0x02, 110},
  {15, 0x02, 110},
  {28, 0x03, 110},
  {2, 0x02, 111},
  {7, 0x02, 111},
  {15, 0x02, 111},
  {28, 0x03, 111},
  {2, 0x02, 112},
  {7, 0x02, 112},
  {15, 0x02, 112},
  {28, 0x03, 112},
  {2, 0x02, 114},
  {7, 0x02, 114},
  {15, 0x02, 114},
  {28, 0x03, 114},
},
/* 12 */
{
  {3, 0x02, 110},
  {4, 0x02, 110},
  {8, 0x02, 110},
  {11, 0x02, 110},
  {16, 0x02, 110},
  {21, 0x02, 110},
  {29, 0x02, 110},
  {37, 0x03, 110},
  {3, 0x02, 111},
  {4, 0x02, 111},
  {8, 0x02, 111},
  {11, 0x02, 111},
  {16, 0x02, 111},
  {21, 0x02, 111},
  {29, 0x02, 111},
  {37, 0x03, 111},
},
/* 13 */
{
  {3, 0x02, 112},
  {4, 0x02, 112},
  {8, 0x02, 112},
  {11, 0x02, 112},
  {16, 0x02, 112},
  {21, 0x02, 112},
  {29, 0x02, 112},
  {37, 0x03, 112},
  {3, 0x02, 114},
  {4, 0x02, 114},
  {8, 0x02, 114},
  {11, 0x02, 114},
  {16, 0x02, 114},
  {21, 0x02, 114},
  {29, 0x02, 114},
  {37, 0x03, 114},
},
/* 14 */
{
  {0, 0x03, 115},
  {0, 0x03, 116},
  {19, 0x00, 0},
  {20, 0x00, 0},
  {23, 0x00, 0},
  {24, 0x00, 0},
  {26, 0x00, 0},
  {27, 0x00, 0},
  {31, 0x00, 0},
  {32, 0x00, 0},
  {34, 0x00, 0},
  {35, 0x00, 0},
  {39, 0x00, 0},
  {42, 0x00, 0},
  {50, 0x00, 0},
  {59, 0x01, 0},
},
/* 15 */
{
  {1, 0x02, 115},
  {14, 0x03, 115},
  {1, 0x02, 116},
  {14, 0x03, 116},
  {0, 0x03, 37},
  {0, 0x03, 45},
  {0, 0x03, 51},
  {0, 0x03, 52},
  {0, 0x03, 53},
  {0, 0x03, 54},
  {0, 0x03, 55},
  {0, 0x03, 56},
  {0, 0x03, 57},
  {0, 0x03, 61},
  {0, 0x03, 95},
  {0, 0x03, 98},
},
/* 16 */
{
  {2, 0x02, 115},
  {7, 0x02, 115},
  {15, 0x02, 115},
  {28, 0x03, 115},
  {2, 0x02, 116},
  {7, 0x02, 116},
  {15, 0x02, 116},
  {28, 0x03, 116},
  {1, 0x02, 37},
  {14, 0x03, 37},
  {1, 0x02, 45},
  {14, 0x03, 45},
  {1, 0x02, 51},
  {14, 0x03, 51},
  {1, 0x02, 52},
  {14, 0x03, 52},
},
/* 17 */
{
  {3, 0x02, 115},
  {4, 0x02, 115},
  {8, 0x02, 115},
  {11, 0x02, 115},
  {16, 0x02, 115},
  {21, 0x02, 115},
  {29, 0x02, 115},
  {37, 0x03, 115},
  {3, 0x02, 116},
  {4, 0x02, 116},
  {8, 0x02, 116},
  {11, 0x02, 116},
  {16, 0x02, 116},
  {21, 0x02, 116},
  {29, 0x02, 116},
  {37, 0x03, 116},
},
/* 18 */
{
  {2, 0x02, 37},
  {7, 0x02, 37},
  {15, 0x02, 37},
  {28, 0x03, 37},
  {2, 0x02, 45},
  {7, 0x02, 45},
  {15, 0x02, 45},
  {28, 0x03, 45},
  {2, 0x02, 51},
  {7, 0x02, 51},
  {15, 0x02, 51},
  {28, 0x03, 51},
  {2, 0x02, 52},
  {7, 0x02, 52},
  {15, 0x02, 52},
  {28, 0x03, 52},
},
/* 19 */
{
  {3, 0x02, 37},
  {4, 0x02, 37},
  {8, 0x02, 37},
  {11, 0x02, 37},
  {16, 0x02, 37},
  {21, 0x02, 37},
  {29, 0x02, 37},
  {37, 0x03, 37},
  {3, 0x02, 45},
  {4, 0x02, 45},
  {8, 0x02, 45},
  {11, 0x02, 45},
  {16, 0x02, 45},
  {21, 0x02, 45},
  {29, 0x02, 45},
  {37, 0x03, 45},
},
/* 20 */
{
  {3, 0x02, 51},
  {4, 0x02, 51},
  {8, 0x02, 51},
  {11, 0x02, 51},
  {16, 0x02, 51},
  {21, 0x02, 51},
  {29, 0x02, 51},
  {37, 0x03, 51},
  {3, 0x02, 52},
  {4, 0x02, 52},
  {8, 0x02, 52},
  {11, 0x02, 52},
  {16, 0x02, 52},
  {21, 0x02, 52},
  {29, 0x02, 52},
  {37, 0x03, 52},
},
/* 21 */
{
  {1, 0x02, 53},
  {14, 0x03, 53},
  {1, 0x02, 54},
  {14, 0x03, 54},
  {1, 0x02, 55},
  {14, 0x03, 55},
  {1, 0x02, 56},
  {14, 0x03, 56},
  {1, 0x02, 57},
  {14, 0x03, 57},
  {1, 0x02, 61},
  {14, 0x03, 61},
  {1, 0x02, 95},
  {14, 0x03, 95},
  {1, 0x02, 98},
  {14, 0x03, 98},
},
/* 22 */
{
  {2, 0x02, 53},
  {7, 0x02, 53},
  {15, 0x02, 53},
  {28, 0x03, 53},
  {2, 0x02, 54},
  {7, 0x02, 54},
  {15, 0x02, 54},
  {28, 0x03, 54},
  {2, 0x02, 55},
  {7, 0x02, 55},
  {15, 0x02, 55},
  {28, 0x03, 55},
  {2, 0x02, 56},
  {7, 0x02, 56},
  {15, 0x02, 56},
  {28, 0x03, 56},
},
/* 23 */
{
  {3, 0x02, 53},
  {4, 0x02, 53},
  {8, 0x02, 53},
  {11, 0x02, 53},
  {16, 0x02, 53},
  {21, 0x02, 53},
  {29, 0x02, 53},
  {37, 0x03, 53},
  {3, 0x02, 54},
  {4, 0x02, 54},
  {8, 0x02, 54},
  {11, 0x02, 54},
  {16, 0x02, 54},
  {21, 0x02, 54},
  {29, 0x02, 54},
  {37, 0x03, 54},
},
/* 24 */
{
  {3, 0x02, 55},
  {4, 0x02, 55},
  {8, 0x02, 55},
  {11, 0x02, 55},
  {16, 0x02, 55},
  {21, 0x02, 55},
  {29, 0x02, 55},
  {37, 0x03, 55},
  {3, 0x02, 56},
  {4, 0x02, 56},
  {8, 0x02, 56},
  {11, 0x02, 56},
  {16, 0x02, 56},
  {21, 0x02, 56},
  {29, 0x02, 56},
  {37, 0x03, 56},
},
/* 25 */
{
  {2, 0x02, 57},
  {7, 0x02, 57},
  {15, 0x02, 57},
  {28, 0x03, 57},
  {2, 0x02, 61},
  {7, 0x02, 61},
  {15, 0x02, 61},
  {28, 0x03, 61},
  {2, 0x02, 95},
  {7, 0x02, 95},
  {15, 0x02, 95},
  {28, 0x03, 95},
  {2, 0x02, 98},
  {7, 0x02, 98},
  {15, 0x02, 98},
  {28, 0x03, 98},
},
/* 26 */
{
  {3, 0x02, 57},
  {4, 0x02, 57},
  {8, 0x02, 57},
  {11, 0x02, 57},
  {16, 0x02, 57},
  {21, 0x02, 57},
  {29, 0x02, 57},
  {37, 0x03, 57},
  {3, 0x02, 61},
  {4, 0x02, 61},
  {8, 0x02, 61},
  {11, 0x02, 61},
  {16, 0x02, 61},
  {21, 0x02, 61},
  {29, 0x02, 61},
  {37, 0x03, 61},
},
/* 27 */
{
  {3, 0x02, 95},
  {4, 0x02, 95},
  {8, 0x02, 95},
  {11, 0x02, 95},
  {16, 0x02, 95},
  {21, 0x02, 95},
  {29, 0x02, 95},
  {37, 0x03, 95},
  {3, 0x02, 98},
  {4, 0x02, 98},
  {8, 0x02, 98},
  {11, 0x02, 98},
  {16, 0x02, 98},
  {21, 0x02, 98},
  {29, 0x02, 98},
  {37, 0x03, 98},
},
/* 28 */
{
  {0, 0x03, 100},
  {0, 0x03, 102},
  {0, 0x03, 103},
  {0, 0x03, 104},
  {0, 0x03, 109},
  {0, 0x03, 117},
  {0, 0x03, 119},
  {36, 0x00, 0},
  {40, 0x00, 0},
  {41, 0x00, 0},
  {43, 0x00, 0},
  {46, 0x00, 0},
  {51, 0x00, 0},
  {54, 0x00, 0},
  {60, 0x00, 0},
  {67, 0x01, 0},
},
/* 29 */
{
  {1, 0x02, 100},
  {14, 0x03, 100},
  {1, 0x02, 102},
  {14, 0x03, 102},
  {1, 0x02, 103},
  {14, 0x03, 103},
  {1, 0x02, 104},
  {14, 0x03, 104},
  {1, 0x02, 109},
  {14, 0x03, 109},
  {1, 0x02, 117},
  {14, 0x03, 117},
  {1, 0x02, 119},
  {14, 0x03, 119},
  {0, 0x03, 38},
  {0, 0x03, 65},
},
/* 30 */
{
  {2, 0x02, 100},
  {7, 0x02, 100},
  {15, 0x02, 100},
  {28, 0x03, 100},
  {2, 0x02, 102},
  {7, 0x02, 102},
  {15, 0x02, 102},
  {28, 0x03, 102},
  {2, 0x02, 103},
  {7, 0x02, 103},
  {15, 0x02, 103},
  {28, 0x03, 103},
  {2, 0x02, 104},
  {7, 0x02, 104},
  {15, 0x02, 104},
  {28, 0x03, 104},
},
/* 31 */
{
  {3, 0x02, 100},
  {4, 0x02, 100},
  {8, 0x02, 100},
  {11, 0x02, 100},
  {16, 0x02, 100},
  {21, 0x02, 100},
  {29, 0x02, 100},
  {37, 0x03, 100},
  {3, 0x02, 102},
  {4, 0x02, 102},
  {8, 0x02, 102},
  {11, 0x02, 102},
  {16, 0x02, 102},
  {21, 0x02, 102},
  {29, 0x02, 102},
  {37, 0x03, 102},
},
/* 32 */
{
  {3, 0x02, 103},
  {4, 0x02, 103},
  {8, 0x02, 103},
  {11, 0x02, 103},
  {16, 0x02, 103},
  {21, 0x02, 103},
  {29, 0x02, 103},
  {37, 0x03, 103},
  {3, 0x02, 104},
  {4, 0x02, 104},
  {8, 0x02, 104},
  {11, 0x02, 104},
  {16, 0x02, 104},
  {21, 0x02, 104},
  {29, 0x02, 104},
  {37, 0x03, 104},
},
/* 33 */
{
  {2, 0x02, 109},
  {7, 0x02, 109},
  {15, 0x02, 109},
  {28, 0x03, 109},
  {2, 0x02, 117},
  {7, 0x02, 117},
  {15, 0x02, 117},
  {28, 0x03, 117},
  {2, 0x02, 119},
  {7, 0x02, 119},
  {15, 0x02, 119},
  {28, 0x03, 119},
  {1, 0x02, 38},
  {14, 0x03, 38},
  {1, 0x02, 65},
  {14, 0x03, 65},
},
/* 34 */
{
  {3, 0x02, 109},
  {4, 0x02, 109},
  {8, 0x02, 109},
  {11, 0x02, 109},
  {16, 0x02, 109},
  {21, 0x02, 109},
  {29, 0x02, 109},
  {37, 0x03, 109},
  {3, 0x02, 117},
  {4, 0x02, 117},
  {8, 0x02, 117},
  {11, 0x02, 117},
  {16, 0x02, 117},
  {21, 0x02, 117},
  {29, 0x02, 117},
  {37, 0x03, 117},
},
/* 35 */
{
  {3, 0x02, 119},
  {4, 0x02, 119},
  {8, 0x02, 119},
  {11, 0x02, 119},
  {16, 0x02, 119},
  {21, 0x02, 119},
  {29, 0x02, 119},
  {37, 0x03, 119},
  {2, 0x02, 38},
  {7, 0x02, 38},
  {15, 0x02, 38},
  {28, 0x03, 38},
  {2, 0x02, 65},
  {7, 0x02, 65},
  {15, 0x02, 65},
  {28, 0x03, 65},
},
/* 36 */
{
  {3, 0x02, 38},
  {4, 0x02, 38},
  {8, 0x02, 38},
  {11, 0x02, 38},
  {16, 0x02, 38},
  {21, 0x02, 38},
  {29, 0x02, 38},
  {37, 0x03, 38},
  {3, 0x02, 65},
  {4, 0x02, 65},
  {8, 0x02, 65},
  {11, 0x02, 65},
  {16, 0x02, 65},
  {21, 0x02, 65},
  {29, 0x02, 65},
  {37, 0x03, 65},
},
/* 37 */
{
  {0, 0x03, 70},
  {0, 0x03, 106},
  {0, 0x03, 107},
  {0, 0x03, 118},
  {44, 0x00, 0},
  {45, 0x00, 0},
  {47, 0x00, 0},
  {48, 0x00, 0},
  {52, 0x00, 0},
  {53, 0x00, 0},
  {55, 0x00, 0},
  {56, 0x00, 0},
  {61, 0x00, 0},
  {64, 0x00, 0},
  {68, 0x00, 0},
  {71, 0x01, 0},
},
/* 38 */
{
  {1, 0x02, 70},
  {14, 0x03, 70},
  {1, 0x02, 106},
  {14, 0x03, 106},
  {1, 0x02, 107},
  {14, 0x03, 107},
  {1, 0x02, 118},
  {14, 0x03, 118},
  {0, 0x03, 32},
  {0, 0x03, 44},
  {0, 0x03, 59},
  {0, 0x03, 66},
  {0, 0x03, 67},
  {0, 0x03, 68},
  {0, 0x03, 69},
  {0, 0x03, 77},
},
/* 39 */
{
  {2, 0x02, 70},
  {7, 0x02, 70},
  {15, 0x02, 70},
  {28, 0x03, 70},
  {2, 0x02, 106},
  {7, 0x02, 106},
  {15, 0x02, 106},
  {28, 0x03, 106},
  {2, 0x02, 107},
  {7, 0x02, 107},
  {15, 0x02, 107},
  {28, 0x03, 107},
  {2, 0x02, 118},
  {7, 0x02, 118},
  {15, 0x02, 118},
  {28, 0x03, 118},
},
/* 40 */
{
  {3, 0x02, 70},
  {4, 0x02, 70},
  {8, 0x02, 70},
  {11, 0x02, 70},
  {16, 0x02, 70},
  {21, 0x02, 70},
  {29, 0x02, 70},
  {37, 0x03, 70},
  {3, 0x02, 106},
  {4, 0x02, 106},
  {8, 0x02, 106},
  {11, 0x02, 106},
  {16, 0x02, 106},
  {21, 0x02, 106},
  {29, 0x02, 106},
  {37, 0x03, 106},
},
/* 41 */
{
  {3, 0x02, 107},
  {4, 0x02, 107},
  {8, 0x02, 107},
  {11, 0x02, 107},
  {16, 0x02, 107},
  {21, 0x02, 107},
  {29, 0x02, 107},
  {37, 0x03, 107},
  {3, 0x02, 118},
  {4, 0x02, 118},
  {8, 0x02, 118},
  {11, 0x02, 118},
  {16, 0x02, 118},
  {21, 0x02, 118},
  {29, 0x02, 118},
  {37, 0x03, 118},
},
/* 42 */
{
  {1, 0x02, 32},
  {14, 0x03, 32},
  {1, 0x02, 44},
  {14, 0x03, 44},
  {1, 0x02, 59},
  {14, 0x03, 59},
  {1, 0x02, 66},
  {14, 0x03, 66},
  {1, 0x02, 67},
  {14, 0x03, 67},
  {1, 0x02, 68},
  {14, 0x03, 68},
  {1, 0x02, 69},
  {14, 0x03, 69},
  {1, 0x02, 77},
  {14, 0x03, 77},
},
/* 43 */
{
  {2, 0x02, 32},
  {7, 0x02, 32},
  {15, 0x02, 32},
  {28, 0x03, 32},
  {2, 0x02, 44},
  {7, 0x02, 44},
  {15, 0x02, 44},
  {28, 0x03, 44},
  {2, 0x02, 59},
  {7, 0x02, 59},
  {15, 0x02, 59},
  {28, 0x03, 59},
  {2, 0x02, 66},
  {7, 0x02, 66},
  {15, 0x02, 66},
  {28, 0x03, 66},
},
/* 44 */
{
  {3, 0x02, 32},
  {4, 0x02, 32},
  {8, 0x02, 32},
  {11, 0x02, 32},
  {16, 0x02, 32},
  {21, 0x02, 32},
  {29, 0x02, 32},
  {37, 0x03, 32},
  {3, 0x02, 44},
  {4, 0x02, 44},
  {8, 0x02, 44},
  {11, 0x02, 44},
  {16, 0x02, 44},
  {21, 0x02, 44},
  {29, 0x02, 44},
  {37, 0x03, 44},
},
/* 45 */
{
  {3, 0x02, 59},
  {4, 0x02, 59},
  {8, 0x02, 59},
  {11, 0x02, 59},
  {16, 0x02, 59},
  {21, 0x02, 59},
  {29, 0x02, 59},
  {37, 0x03, 59},
  {3, 0x02, 66},
  {4, 0x02, 66},
  {8, 0x02, 66},
  {11, 0x02, 66},
  {16, 0x02, 66},
  {21, 0x02, 66},
  {29, 0x02, 66},
  {37, 0x03, 66},
},
/* 46 */
{
  {2, 0x02, 67},
  {7, 0x02, 67},
  {15, 0x02, 67},
  {28, 0x03, 67},
  {2, 0x02, 68},
  {7, 0x02, 68},
  {15, 0x02, 68},
  {28, 0x03, 68},
  {2, 0x02, 69},
  {7, 0x02, 69},
  {15, 0x02, 69},
  {28, 0x03, 69},
  {2, 0x02, 77},
  {7, 0x02, 77},
  {15, 0x02, 77},
  {28, 0x03, 77},
},
/* 47 */
{
  {3, 0x02, 67},
  {4, 0x02, 67},
  {8, 0x02, 67},
  {11, 0x02, 67},
  {16, 0x02, 67},
  {21, 0x02, 67},
  {29, 0x02, 67},
  {37, 0x03, 67},
  {3, 0x02, 68},
  {4, 0x02, 68},
  {8, 0x02, 68},
  {11, 0x02, 68},
  {16, 0x02, 68},
  {21, 0x02, 68},
  {29, 0x02, 68},
  {37, 0x03, 68},
},
/* 48 */
{
  {3, 0x02, 69},
  {4, 0x02, 69},
  {8, 0x02, 69},
  {11, 0x02, 69},
  {16, 0x02, 69},
  {21, 0x02, 69},
  {29, 0x02, 69},
  {37, 0x03, 69},
  {3, 0x02, 77},
  {4, 0x02, 77},
  {8, 0x02, 77},
  {11, 0x02, 77},
  {16, 0x02, 77},
  {21, 0x02, 77},
  {29, 0x02, 77},
  {37, 0x03, 77},
},
/* 49 */
{
  {0, 0x03, 83},
  {0, 0x03, 84},
  {0, 0x03, 113},
  {0, 0x03, 120},
  {0, 0x03, 121},
  {0, 0x03, 122},
  {57, 0x00, 0},
  {58, 0x00, 0},
  {62, 0x00, 0},
  {63, 0x00, 0},
  {65, 0x00, 0},
  {66, 0x00, 0},
  {69, 0x00, 0},
  {70, 0x00, 0},
  {72, 0x00, 0},
  {74, 0x00, 0},
},
/* 50 */
{
  {1, 0x02, 83},
  {14, 0x03, 83},
  {1, 0x02, 84},
  {14, 0x03, 84},
  {1, 0x02, 113},
  {14, 0x03, 113},
  {1, 0x02, 120},
  {14, 0x03, 120},
  {1, 0x02, 121},
  {14, 0x03, 121},
  {1, 0x02, 122},
  {14, 0x03, 122},
  {0, 0x03, 58},
  {0, 0x03, 63},
  {0, 0x03, 71},
  {0, 0x03, 72},
},
/* 51 */
{
  {2, 0x02, 83},
  {7, 0x02, 83},
  {15, 0x02, 83},
  {28, 0x03, 83},
  {2, 0x02, 84},
  {7, 0x02, 84},
  {15, 0x02, 84},
  {28, 0x03, 84},
  {2, 0x02, 113},
  {7, 0x02, 113},
  {15, 0x02, 113},
  {28, 0x03, 113},
  {2, 0x02, 120},
  {7, 0x02, 120},
  {15, 0x02, 120},
  {28, 0x03, 120},
},
/* 52 */
{
  {3, 0x02, 83},
  {4, 0x02, 83},
  {8, 0x02, 83},
  {11, 0x02, 83},
  {16, 0x02, 83},
  {21, 0x02, 83},
  {29, 0x02, 83},
  {37, 0x03, 83},
  {3, 0x02, 84},
  {4, 0x02, 84},
  {8, 0x02, 84},
  {11, 0x02, 84},
  {16, 0x02, 84},
  {21, 0x02, 84},
  {29, 0x02, 84},
  {37, 0x03, 84},
},
/* 53 */
{
  {3, 0x02, 113},
  {4, 0x02, 113},
  {8, 0x02, 113},
  {11, 0x02, 113},
  {16, 0x02, 113},
  {21, 0x02, 113},
  {29, 0x02, 113},
  {37, 0x03, 113},
  {3, 0x02, 120},
  {4, 0x02, 120},
  {8, 0x02, 120},
  {11, 0x02, 120},
  {16, 0x02, 120},
  {21, 0x02, 120},
  {29, 0x02, 120},
  {37, 0x03, 120},
},
/* 54 */
{
  {2, 0x02, 121},
  {7, 0x02, 121},
  {15, 0x02, 121},
  {28, 0x03, 121},
  {2, 0x02, 122},
  {7, 0x02, 122},
  {15, 0x02, 122},
  {28, 0x03, 122},
  {1, 0x02, 58},
  {14, 0x03, 58},
  {1, 0x02, 63},
  {14, 0x03, 63},
  {1, 0x02, 71},
  {14, 0x03, 71},
  {1, 0x02, 72},
  {14, 0x03, 72},
},
/* 55 */
{
  {3, 0x02, 121},
  {4, 0x02, 121},
  {8, 0x02, 121},
  {11, 0x02, 121},
  {16, 0x02, 121},
  {21, 0x02, 121},
  {29, 0x02, 121},
  {37, 0x03, 121},
  {3, 0x02, 122},
  {4, 0x02, 122},
  {8, 0x02, 122},
  {11, 0x02, 122},
  {16, 0x02, 122},
  {21, 0x02, 122},
  {29, 0x02, 122},
  {37, 0x03, 122},
},
/* 56 */
{
  {2, 0x02, 58},
  {7, 0x02, 58},
  {15, 0x02, 58},
  {28, 0x03, 58},
  {2, 0x02, 63},
  {7, 0x02, 63},
  {15, 0x02, 63},
  {28, 0x03, 63},
  {2, 0x02, 71},
  {7, 0x02, 71},
  {15, 0x02, 71},
  {28, 0x03, 71},
  {2, 0x02, 72},
  {7, 0x02, 72},
  {15, 0x02, 72},
  {28, 0x03, 72},
},
/* 57 */
{
  {3, 0x02, 58},
  {4, 0x02, 58},
  {8, 0x02, 58},
  {11, 0x02, 58},
  {16, 0x02, 58},
  {21, 0x02, 58},
  {29, 0x02, 58},
  {37, 0x03, 58},
  {3, 0x02, 63},
  {4, 0x02, 63},
  {8, 0x02, 63},
  {11, 0x02, 63},
  {16, 0x02, 63},
  {21, 0x02, 63},
  {29, 0x02, 63},
  {37, 0x03, 63},
},
/* 58 */
{
  {3, 0x02, 71},
  {4, 0x02, 71},
  {8, 0x02, 71},
  {11, 0x02, 71},
  {16, 0x02, 71},
  {21, 0x02, 71},
  {29, 0x02, 71},
  {37, 0x03, 71},
  {3, 0x02, 72},
  {4, 0x02, 72},
  {8, 0x02, 72},
  {11, 0x02, 72},
  {16, 0x02, 72},
  {21, 0x02, 72},
  {29, 0x02, 72},
  {37, 0x03, 72},
},
/* 59 */
{
  {0, 0x03, 73},
  {0, 0x03, 74},
  {0, 0x03, 76},
  {0, 0x03, 78},
  {0, 0x03, 79},
  {0, 0x03, 80},
  {0, 0x03, 81},
  {0, 0x03, 82},
  {0, 0x03, 85},
  {0, 0x03, 86},
  {0, 0x03, 87},
  {0, 0x03, 88},
  {0, 0x03, 89},
  {73, 0x00, 0},
  {75, 0x00, 0},
  {77, 0x00, 0},
},
/* 60 */
{
  {1, 0x02, 73},
  {14, 0x03, 73},
  {1, 0x02, 74},
  {14, 0x03, 74},
  {1, 0x02, 76},
  {14, 0x03, 76},
  {1, 0x02, 78},
  {14, 0x03, 78},
  {1, 0x02, 79},
  {14, 0x03, 79},
  {1, 0x02, 80},
  {14, 0x03, 80},
  {1, 0x02, 81},
  {14, 0x03, 81},
  {1, 0x02, 82},
  {14, 0x03, 82},
},
/* 61 */
{
  {2, 0x02, 73},
  {7, 0x02, 73},
  {15, 0x02, 73},
  {28, 0x03, 73},
  {2, 0x02, 74},
  {7, 0x02, 74},
  {15, 0x02, 74},
  {28, 0x03, 74},
  {2, 0x02, 76},
  {7, 0x02, 76},
  {15, 0x02, 76},
  {28, 0x03, 76},
  {2, 0x02, 78},
  {7, 0x02, 78},
  {15, 0x02, 78},
  {28, 0x03, 78},
},
/* 62 */
{
  {3, 0x02, 73},
  {4, 0x02, 73},
  {8, 0x02, 73},
  {11, 0x02, 73},
  {16, 0x02, 73},
  {21, 0x02, 73},
  {29, 0x02, 73},
  {37, 0x03, 73},
  {3, 0x02, 74},
  {4, 0x02, 74},
  {8, 0x02, 74},
  {11, 0x02, 74},
  {16, 0x02, 74},
  {21, 0x02, 74},
  {29, 0x02, 74},
  {37, 0x03, 74},
},
/* 63 */
{
  {3, 0x02, 76},
  {4, 0x02, 76},
  {8, 0x02, 76},
  {11, 0x02, 76},
  {16, 0x02, 76},
  {21, 0x02, 76},
  {29, 0x02, 76},
  {37, 0x03, 76},
  {3, 0x02, 78},
  {4, 0x02, 78},
  {8, 0x02, 78},
  {11, 0x02, 78},
  {16, 0x02, 78},
  {21, 0x02, 78},
  {29, 0x02, 78},
  {37, 0x03, 78},
},
/* 64 */
{
  {2, 0x02, 79},
  {7, 0x02, 79},
  {15, 0x02, 79},
  {28, 0x03, 79},
  {2, 0x02, 80},
  {7, 0x02, 80},
  {15, 0x02, 80},
  {28, 0x03, 80},
  {2, 0x02, 81},
  {7, 0x02, 81},
  {15, 0x02, 81},
  {28, 0x03, 81},
  {2, 0x02, 82},
  {7, 0x02, 82},
  {15, 0x02, 82},
  {28, 0x03, 82},
},
/* 65 */
{
  {3, 0x02, 79},
  {4, 0x02, 79},
  {8, 0x02, 79},
  {11, 0x02, 79},
  {16, 0x02, 79},
  {21, 0x02, 79},
  {29, 0x02, 79},
  {37, 0x03, 79},
  {3, 0x02, 80},
  {4, 0x02, 80},
  {8, 0x02, 80},
  {11, 0x02, 80},
  {16, 0x02, 80},
  {21, 0x02, 80},
  {29, 0x02, 80},
  {37, 0x03, 80},
},
/* 66 */
{
  {3, 0x02, 81},
  {4, 0x02, 81},
  {8, 0x02, 81},
  {11, 0x02, 81},
  {16, 0x02, 81},
  {21, 0x02, 81},
  {29, 0x02, 81},
  {37, 0x03, 81},
  {3, 0x02, 82},
  {4, 0x02, 82},
  {8, 0x02, 82},
  {11, 0x02, 82},
  {16, 0x02, 82},
  {21, 0x02, 82},
  {29, 0x02, 82},
  {37, 0x03, 82},
},
/* 67 */
{
  {1, 0x02, 85},
  {14, 0x03, 85},
  {1, 0x02, 86},
  {14, 0x03, 86},
  {1, 0x02, 87},
  {14, 0x03, 87},
  {1, 0x02, 88},
  {14, 0x03, 88},
  {1, 0x02, 89},
  {14, 0x03, 89},
  {0, 0x03, 42},
  {0, 0x03, 75},
  {0, 0x03, 90},
  {76, 0x00, 0},
  {78, 0x00, 0},
  {79, 0x00, 0},
},
/* 68 */
{
  {2, 0x02, 85},
  {7, 0x02, 85},
  {15, 0x02, 85},
  {28, 0x03, 85},
  {2, 0x02, 86},
  {7, 0x02, 86},
  {15, 0x02, 86},
  {28, 0x03, 86},
  {2, 0x02, 87},
  {7, 0x02, 87},
  {15, 0x02, 87},
  {28, 0x03, 87},
  {2, 0x02, 88},
  {7, 0x02, 88},
  {15, 0x02, 88},
  {28, 0x03, 88},
},
/* 69 */
{
  {3, 0x02, 85},
  {4, 0x02, 85},
  {8, 0x02, 85},
  {11, 0x02, 85},
  {16, 0x02, 85},
  {21, 0x02, 85},
  {29, 0x02, 85},
  {37, 0x03, 85},
  {3, 0x02, 86},
  {4, 0x02, 86},
  {8, 0x02, 86},
  {11, 0x02, 86},
  {16, 0x02, 86},
  {21, 0x02, 86},
  {29, 0x02, 86},
  {37, 0x03, 86},
},
/* 70 */
{
  {3, 0x02, 87},
  {4, 0x02, 87},
  {8, 0x02, 87},
  {11, 0x02, 87},
  {16, 0x02, 87},
  {21, 0x02, 87},
  {29, 0x02, 87},
  {37, 0x03, 87},
  {3, 0x02, 88},
  {4, 0x02, 88},
  {8, 0x02, 88},
  {11, 0x02, 88},
  {16, 0x02, 88},
  {21, 0x02, 88},
  {29, 0x02, 88},
  {37, 0x03, 88},
},
/* 71 */
{
  {2, 0x02, 89},
  {7, 0x02, 89},
  {15, 0x02, 89},
  {28, 0x03, 89},
  {1, 0x02, 42},
  {14, 0x03, 42},
  {1, 0x02, 75},
  {14, 0x03, 75},
  {1, 0x02, 90},
  {14, 0x03, 90},
  {0, 0x03, 40},
  {0, 0x03, 41},
  {0, 0x03, 43},
  {0, 0x03, 124},
  {80, 0x00, 0},
  {81, 0x00, 0},
},
/* 72 */
{
  {3, 0x02, 89},
  {4, 0x02, 89},
  {8, 0x02, 89},
  {11, 0x02, 89},
  {16, 0x02, 89},
  {21, 0x02, 89},
  {29, 0x02, 89},
  {37, 0x03, 89},
  {2, 0x02, 42},
  {7, 0x02, 42},
  {15, 0x02, 42},
  {28, 0x03, 42},
  {2, 0x02, 75},
  {7, 0x02, 75},
  {15, 0x02, 75},
  {28, 0x03, 75},
},
/* 73 */
{
  {3, 0x02, 42},
  {4, 0x02, 42},
  {8, 0x02, 42},
  {11, 0x02, 42},
  {16, 0x02, 42},
  {21, 0x02, 42},
  {29, 0x02, 42},
  {37, 0x03, 42},
  {3, 0x02, 75},
  {4, 0x02, 75},
  {8, 0x02, 75},
  {11, 0x02, 75},
  {16, 0x02, 75},
  {21, 0x02, 75},
  {29, 0x02, 75},
  {37, 0x03, 75},
},
/* 74 */
{
  {2, 0x02, 90},
  {7, 0x02, 90},
  {15, 0x02, 90},
  {28, 0x03, 90},
  {1, 0x02, 40},
  {14, 0x03, 40},
  {1, 0x02, 41},
  {14, 0x03, 41},
  {1, 0x02, 43},
  {14, 0x03, 43},
  {1, 0x02, 124},
  {14, 0x03, 124},
  {0, 0x03, 33},
  {0, 0x03, 126},
  {82, 0x00, 0},
  {84, 0x00, 0},
},
/* 75 */
{
  {3, 0x02, 90},
  {4, 0x02, 90},
  {8, 0x02, 90},
  {11, 0x02, 90},
  {16, 0x02, 90},
  {21, 0x02, 90},
  {29, 0x02, 90},
  {37, 0x03, 90},
  {2, 0x02, 40},
  {7, 0x02, 40},
  {15, 0x02, 40},
  {28, 0x03, 40},
  {2, 0x02, 41},
  {7, 0x02, 41},
  {15, 0x02, 41},
  {28, 0x03, 41},
},
/* 76 */
{
  {3, 0x02, 40},
  {4, 0x02, 40},
  {8, 0x02, 40},
  {11, 0x02, 40},
  {16, 0x02, 40},
  {21, 0x02, 40},
  {29, 0x02, 40},
  {37, 0x03, 40},
  {3, 0x02, 41},
  {4, 0x02, 41},
  {8, 0x02, 41},
  {11, 0x02, 41},
  {16, 0x02, 41},
  {21, 0x02, 41},
  {29, 0x02, 41},
  {37, 0x03, 41},
},
/* 77 */
{
  {2, 0x02, 43},
  {7, 0x02, 43},
  {15, 0x02, 43},
  {28, 0x03, 43},
  {2, 0x02, 124},
  {7, 0x02, 124},
  {15, 0x02, 124},
  {28, 0x03, 124},
  {1, 0x02, 33},
  {14, 0x03, 33},
  {1, 0x02, 126},
  {14, 0x03, 126},
  {0, 0x03, 93},
  {83, 0x00, 0},
  {85, 0x00, 0},
  {86, 0x00, 0},
},
/* 78 */
{
  {3, 0x02, 43},
  {4, 0x02, 43},
  {8, 0x02, 43},
  {11, 0x02, 43},
  {16, 0x02, 43},
  {21, 0x02, 43},
  {29, 0x02, 43},
  {37, 0x03, 43},
  {3, 0x02, 124},
  {4, 0x02, 124},
  {8, 0x02, 124},
  {11, 0x02, 124},
  {16, 0x02, 124},
  {21, 0x02, 124},
  {29, 0x02, 124},
  {37, 0x03, 124},
},
/* 79 */
{
  {2, 0x02, 33},
  {7, 0x02, 33},
  {15, 0x02, 33},
  {28, 0x03, 33},
  {2, 0x02, 126},
  {7, 0x02, 126},
  {15, 0x02, 126},
  {28, 0x03, 126},
  {1, 0x02, 93},
  {14, 0x03, 93},
  {0, 0x03, 34},
  {0, 0x03, 64},
  {0, 0x03, 91},
  {0, 0x03, 94},
  {87, 0x00, 0},
  {88, 0x00, 0},
},
/* 80 */
{
  {3, 0x02, 33},
  {4, 0x02, 33},
  {8, 0x02, 33},
  {11, 0x02, 33},
  {16, 0x02, 33},
  {21, 0x02, 33},
  {29, 0x02, 33},
  {37, 0x03, 33},
  {3, 0x02, 126},
  {4, 0x02, 126},
  {8, 0x02, 126},
  {11, 0x02, 126},
  {16, 0x02, 126},
  {21, 0x02, 126},
  {29, 0x02, 126},
  {37, 0x03, 126},
},
/* 81 */
{
  {2, 0x02, 93},
  {7, 0x02, 93},
  {15, 0x02, 93},
  {28, 0x03, 93},
  {1, 0x02, 34},
  {14, 0x03, 34},
  {1, 0x02, 64},
  {14, 0x03, 64},
  {1, 0x02, 91},
  {14, 0x03, 91},
  {1, 0x02, 94},
  {14, 0x03, 94},
  {0, 0x03, 35},
  {0, 0x03, 36},
  {0, 0x03, 39},
  {89, 0x00, 0},
},
/* 82 */
{
  {3, 0x02, 93},
  {4, 0x02, 93},
  {8, 0x02, 93},
  {11, 0x02, 93},
  {16, 0x02, 93},
  {21, 0x02, 93},
  {29, 0x02, 93},
  {37, 0x03, 93},
  {2, 0x02, 34},
  {7, 0x02, 34},
  {15, 0x02, 34},
  {28, 0x03, 34},
  {2, 0x02, 64},
  {7, 0x02, 64},
  {15, 0x02, 64},
  {28, 0x03, 64},
},
/* 83 */
{
  {3, 0x02, 34},
  {4, 0x02, 34},
  {8, 0x02, 34},
  {11, 0x02, 34},
  {16, 0x02, 34},
  {21, 0x02, 34},
  {29, 0x02, 34},
  {37, 0x03, 34},
  {3, 0x02, 64},
  {4, 0x02, 64},
  {8, 0x02, 64},
  {11, 0x02, 64},
  {16, 0x02, 64},
  {21, 0x02, 64},
  {29, 0x02, 64},
  {37, 0x03, 64},
},
/* 84 */
{
  {2, 0x02, 91},
  {7, 0x02, 91},
  {15, 0x02, 91},
  {28, 0x03, 91},
  {2, 0x02, 94},
  {7, 0x02, 94},
  {15, 0x02, 94},
  {28, 0x03, 94},
  {1, 0x02, 35},
  {14, 0x03, 35},
  {1, 0x02, 36},
  {14, 0x03, 36},
  {1, 0x02, 39},
  {14, 0x03, 39},
  {90, 0x00, 0},
  {91, 0x00, 0},
},
/* 85 */
{
  {3, 0x02, 91},
  {4, 0x02, 91},
  {8, 0x02, 91},
  {11, 0x02, 91},
  {16, 0x02, 91},
  {21, 0x02, 91},
  {29, 0x02, 91},
  {37, 0x03, 91},
  {3, 0x02, 94},
  {4, 0x02, 94},
  {8, 0x02, 94},
  {11, 0x02, 94},
  {16, 0x02, 94},
  {21, 0x02, 94},
  {29, 0x02, 94},
  {37, 0x03, 94},
},
/* 86 */
{
  {2, 0x02, 35},
  {7, 0x02, 35},
  {15, 0x02, 35},
  {28, 0x03, 35},
  {2, 0x02, 36},
  {7, 0x02, 36},
  {15, 0x02, 36},
  {28, 0x03, 36},
  {2, 0x02, 39},
  {7, 0x02, 39},
  {15, 0x02, 39},
  {28, 0x03, 39},
  {0, 0x03, 62},
  {0, 0x03, 123},
  {0, 0x03, 125},
  {92, 0x00, 0},
},
/* 87 */
{
  {3, 0x02, 35},
  {4, 0x02, 35},
  {8, 0x02, 35},
  {11, 0x02, 35},
  {16, 0x02, 35},
  {21, 0x02, 35},
  {29, 0x02, 35},
  {37, 0x03, 35},
  {3, 0x02, 36},
  {4, 0x02, 36},
  {8, 0x02, 36},
  {11, 0x02, 36},
  {16, 0x02, 36},
  {21, 0x02, 36},
  {29, 0x02, 36},
  {37, 0x03, 36},
},
/* 88 */
{
  {3, 0x02, 39},
  {4, 0x02, 39},
  {8, 0x02, 39},
  {11, 0x02, 39},
  {16, 0x02, 39},
  {21, 0x02, 39},
  {29, 0x02, 39},
  {37, 0x03, 39},
  {1, 0x02, 62},
  {14, 0x03, 62},
  {1, 0x02, 123},
  {14, 0x03, 123},
  {1, 0x02, 125},
  {14, 0x03, 125},
  {0, 0x03, 60},
  {93, 0x00, 0},
},
/* 89 */
{
  {2, 0x02, 62},
  {7, 0x02, 62},
  {15, 0x02, 62},
  {28, 0x03, 62},
  {2, 0x02, 123},
  {7, 0x02, 123},
  {15, 0x02, 123},
  {28, 0x03, 123},
  {2, 0x02, 125},
  {7, 0x02, 125},
  {15, 0x02, 125},
  {28, 0x03, 125},
  {1, 0x02, 60},
  {14, 0x03, 60},
  {0, 0x03, 96},
  {94, 0x00, 0},
},
/* 90 */
{
  {3, 0x02, 62},
  {4, 0x02, 62},
  {8, 0x02, 62},
  {11, 0x02, 62},
  {16, 0x02, 62},
  {21, 0x02, 62},
  {29, 0x02, 62},
  {37, 0x03, 62},
  {3, 0x02, 123},
  {4, 0x02, 123},
  {8, 0x02, 123},
  {11, 0x02, 123},
  {16, 0x02, 123},
  {21, 0x02, 123},
  {29, 0x02, 123},
  {37, 0x03, 123},
},
/* 91 */
{
  {3, 0x02, 125},
  {4, 0x02, 125},
  {8, 0x02, 125},
  {11, 0x02, 125},
  {16, 0x02, 125},
  {21, 0x02, 125},
  {29, 0x02, 125},
  {37, 0x03, 125},
  {2, 0x02, 60},
  {7, 0x02, 60},
  {15, 0x02, 60},
  {28, 0x03, 60},
  {1, 0x02, 96},
  {14, 0x03, 96},
  {95, 0x00, 0},
  {158, 0x00, 0},
},
/* 92 */
{
  {3, 0x02, 60},
  {4, 0x02, 60},
  {8, 0x02, 60},
  {11, 0x02, 60},
  {16, 0x02, 60},
  {21, 0x02, 60},
  {29, 0x02, 60},
  {37, 0x03, 60},
  {2, 0x02, 96},
  {7, 0x02, 96},
  {15, 0x02, 96},
  {28, 0x03, 96},
  {96, 0x00, 0},
  {127, 0x00, 0},
  {159, 0x00, 0},
  {193, 0x00, 0},
},
/* 93 */
{
  {3, 0x02, 96},
  {4, 0x02, 96},
  {8, 0x02, 96},
  {11, 0x02, 96},
  {16, 0x02, 96},
  {21, 0x02, 96},
  {29, 0x02, 96},
  {37, 0x03, 96},
  {97, 0x00, 0},
  {112, 0x00, 0},
  {128, 0x00, 0},
  {143, 0x00, 0},
  {160, 0x00, 0},
  {175, 0x00, 0},
  {194, 0x00, 0},
  {225, 0x00, 0},
},
/* 94 */
{
  {98, 0x00, 0},
  {105, 0x00, 0},
  {113, 0x00, 0},
  {120, 0x00, 0},
  {129, 0x00, 0},
  {136, 0x00, 0},
  {144, 0x00, 0},
  {151, 0x00, 0},
  {161, 0x00, 0},
  {168, 0x00, 0},
  {176, 0x00, 0},
  {183, 0x00, 0},
  {195, 0x00, 0},
  {210, 0x00, 0},
  {226, 0x00, 0},
  {241, 0x00, 0},
},
/* 95 */
{
  {99, 0x00, 0},
  {102, 0x00, 0},
  {106, 0x00, 0},
  {109, 0x00, 0},
  {114, 0x00, 0},
  {117, 0x00, 0},
  {121, 0x00, 0},
  {124, 0x00, 0},
  {130, 0x00, 0},
  {133, 0x00, 0},
  {137, 0x00, 0},
  {140, 0x00, 0},
  {145, 0x00, 0},
  {148, 0x00, 0},
  {152, 0x00, 0},
  {155, 0x00, 0},
},
/* 96 */
{
  {100, 0x00, 0},
  {101, 0x00, 0},
  {103, 0x00, 0},
  {104, 0x00, 0},
  {107, 0x00, 0},
  {108, 0x00, 0},
  {110, 0x00, 0},
  {111, 0x00, 0},
  {115, 0x00, 0},
  {116, 0x00, 0},
  {118, 0x00, 0},
  {119, 0x00, 0},
  {122, 0x00, 0},
  {123, 0x00, 0},
  {125, 0x00, 0},
  {126, 0x00, 0},
},
/* 97 */
{
  {0, 0x03, 164},
  {0, 0x03, 165},
  {0, 0x03, 166},
  {0, 0x03, 167},
  {0, 0x03, 168},
  {0, 0x03, 169},
  {0, 0x03, 170},
  {0, 0x03, 171},
  {0, 0x03, 172},
  {0, 0x03, 173},
  {0, 0x03, 174},
  {0, 0x03, 175},
  {0, 0x03, 176},
  {0, 0x03, 177},
  {0, 0x03, 178},
  {0, 0x03, 179},
},
/* 98 */
{
  {1, 0x02, 164},
  {14, 0x03, 164},
  {1, 0x02, 165},
  {14, 0x03, 165},
  {1, 0x02, 166},
  {14, 0x03, 166},
  {1, 0x02, 167},
  {14, 0x03, 167},
  {1, 0x02, 168},
  {14, 0x03, 168},
  {1, 0x02, 169},
  {14, 0x03, 169},
  {1, 0x02, 170},
  {14, 0x03, 170},
  {1, 0x02, 171},
  {14, 0x03, 171},
},
/* 99 */
{
  {2, 0x02, 164},
  {7, 0x02, 164},
  {15, 0x02, 164},
  {28, 0x03, 164},
  {2, 0x02, 165},
  {7, 0x02, 165},
  {15, 0x02, 165},
  {28, 0x03, 165},
  {2, 0x02, 166},
  {7, 0x02, 166},
  {15, 0x02, 166},
  {28, 0x03, 166},
  {2, 0x02, 167},
  {7, 0x02, 167},
  {15, 0x02, 167},
  {28, 0x03, 167},
},
/* 100 */
{
  {3, 0x02, 164},
  {4, 0x02, 164},
  {8, 0x02, 164},
  {11, 0x02, 164},
  {16, 0x02, 164},
  {21, 0x02, 164},
  {29, 0x02, 164},
  {37, 0x03, 164},
  {3, 0x02, 165},
  {4, 0x02, 165},
  {8, 0x02, 165},
  {11, 0x02, 165},
  {16, 0x02, 165},
  {21, 0x02, 165},
  {29, 0x02, 165},
  {37, 0x03, 165},
},
/* 101 */
{
  {3, 0x02, 166},
  {4, 0x02, 166},
  {8, 0x02, 166},
  {11, 0x02, 166},
  {16, 0x02, 166},
  {21, 0x02, 166},
  {29, 0x02, 166},
  {37, 0x03, 166},
  {3, 0x02, 167},
  {4, 0x02, 167},
  {8, 0x02, 167},
  {11, 0x02, 167},
  {16, 0x02, 167},
  {21, 0x02, 167},
  {29, 0x02, 167},
  {37, 0x03, 167},
},
/* 102 */
{
  {2, 0x02, 168},
  {7, 0x02, 168},
  {15, 0x02, 168},
  {28, 0x03, 168},
  {2, 0x02, 169},
  {7, 0x02, 169},
  {15, 0x02, 169},
  {28, 0x03, 169},
  {2, 0x02, 170},
  {7, 0x02, 170},
  {15, 0x02, 170},
  {28, 0x03, 170},
  {2, 0x02, 171},
  {7, 0x02, 171},
  {15, 0x02, 171},
  {28, 0x03, 171},
},
/* 103 */
{
  {3, 0x02, 168},
  {4, 0x02, 168},
  {8, 0x02, 168},
  {11, 0x02, 168},
  {16, 0x02, 168},
  {21, 0x02, 168},
  {29, 0x02, 168},
  {37, 0x03, 168},
  {3, 0x02, 169},
  {4, 0x02, 169},
  {8, 0x02, 169},
  {11, 0x02, 169},
  {16, 0x02, 169},
  {21, 0x02, 169},
  {29, 0x02, 169},
  {37, 0x03, 169},
},
/* 104 */
{
  {3, 0x02, 170},
  {4, 0x02, 170},
  {8, 0x02, 170},
  {11, 0x02, 170},
  {16, 0x02, 170},
  {21, 0x02, 170},
  {29, 0x02, 170},
  {37, 0x03, 170},
  {3, 0x02, 171},
  {4, 0x02, 171},
  {8, 0x02, 171},
  {11, 0x02, 171},
  {16, 0x02, 171},
  {21, 0x02, 171},
  {29, 0x02, 171},
  {37, 0x03, 171},
},
/* 105 */
{
  {1, 0x02, 172},
  {14, 0x03, 172},
  {1, 0x02, 173},
  {14, 0x03, 173},
  {1, 0x02, 174},
  {14, 0x03, 174},
  {1, 0x02, 175},
  {14, 0x03, 175},
  {1, 0x02, 176},
  {14, 0x03, 176},
  {1, 0x02, 177},
  {14, 0x03, 177},
  {1, 0x02, 178},
  {14, 0x03, 178},
  {1, 0x02, 179},
  {14, 0x03, 179},
},
/* 106 */
{
  {2, 0x02, 172},
  {7, 0x02, 172},
  {15, 0x02, 172},
  {28, 0x03, 172},
  {2, 0x02, 173},
  {7, 0x02, 173},
  {15, 0x02, 173},
  {28, 0x03, 173},
  {2, 0x02, 174},
  {7, 0x02, 174},
  {15, 0x02, 174},
  {28, 0x03, 174},
  {2, 0x02, 175},
  {7, 0x02, 175},
  {15, 0x02, 175},
  {28, 0x03, 175},
},
/* 107 */
{
  {3, 0x02, 172},
  {4, 0x02, 172},
  {8, 0x02, 172},
  {11, 0x02, 172},
  {16, 0x02, 172},
  {21, 0x02, 172},
  {29, 0x02, 172},
  {37, 0x03, 172},
  {3, 0x02, 173},
  {4, 0x02, 173},
  {8, 0x02, 173},
  {11, 0x02, 173},
  {16, 0x02, 173},
  {21, 0x02, 173},
  {29, 0x02, 173},
  {37, 0x03, 173},
},
/* 108 */
{
  {3, 0x02, 174},
  {4, 0x02, 174},
  {8, 0x02, 174},
  {11, 0x02, 174},
  {16, 0x02, 174},
  {21, 0x02, 174},
  {29, 0x02, 174},
  {37, 0x03, 174},
  {3, 0x02, 175},
  {4, 0x02, 175},
  {8, 0x02, 175},
  {11, 0x02, 175},
  {16, 0x02, 175},
  {21, 0x02, 175},
  {29, 0x02, 175},
  {37, 0x03, 175},
},
/* 109 */
{
  {2, 0x02, 176},
  {7, 0x02, 176},
  {15, 0x02, 176},
  {28, 0x03, 176},
  {2, 0x02, 177},
  {7, 0x02, 177},
  {15, 0x02, 177},
  {28, 0x03, 177},
  {2, 0x02, 178},
  {7, 0x02, 178},
  {15, 0x02, 178},
  {28, 0x03, 178},
  {2, 0x02, 179},
  {7, 0x02, 179},
  {15, 0x02, 179},
  {28, 0x03, 179},
},
/* 110 */
{
  {3, 0x02, 176},
  {4, 0x02, 176},
  {8, 0x02, 176},
  {11, 0x02, 176},
  {16, 0x02, 176},
  {21, 0x02, 176},
  {29, 0x02, 176},
  {37, 0x03, 176},
  {3, 0x02, 177},
  {4, 0x02, 177},
  {8, 0x02, 177},
  {11, 0x02, 177},
  {16, 0x02, 177},
  {21, 0x02, 177},
  {29, 0x02, 177},
  {37, 0x03, 177},
},
/* 111 */
{
  {3, 0x02, 178},
  {4, 0x02, 178},
  {8, 0x02, 178},
  {11, 0x02, 178},
  {16, 0x02, 178},
  {21, 0x02, 178},
  {29, 0x02, 178},
  {37, 0x03, 178},
  {3, 0x02, 179},
  {4, 0x02, 179},
  {8, 0x02, 179},
  {11, 0x02, 179},
  {16, 0x02, 179},
  {21, 0x02, 179},
  {29, 0x02, 179},
  {37, 0x03, 179},
},
/* 112 */
{
  {0, 0x03, 180},
  {0, 0x03, 181},
  {0, 0x03, 182},
  {0, 0x03, 183},
  {0, 0x03, 184},
  {0, 0x03, 185},
  {0, 0x03, 186},
  {0, 0x03, 187},
  {0, 0x03, 188},
  {0, 0x03, 189},
  {0, 0x03, 190},
  {0, 0x03, 191},
  {0, 0x03, 192},
  {0, 0x03, 193},
  {0, 0x03, 194},
  {0, 0x03, 195},
},
/* 113 */
{
  {1, 0x02, 180},
  {14, 0x03, 180},
  {1, 0x02, 181},
  {14, 0x03, 181},
  {1, 0x02, 182},
  {14, 0x03, 182},
  {1, 0x02, 183},
  {14, 0x03, 183},
  {1, 0x02, 184},
  {14, 0x03, 184},
  {1, 0x02, 185},
  {14, 0x03, 185},
  {1, 0x02, 186},
  {14, 0x03, 186},
  {1, 0x02, 187},
  {14, 0x03, 187},
},
/* 114 */
{
  {2, 0x02, 180},
  {7, 0x02, 180},
  {15, 0x02, 180},
  {28, 0x03, 180},
  {2, 0x02, 181},
  {7, 0x02, 181},
  {15, 0x02, 181},
  {28, 0x03, 181},
  {2, 0x02, 182},
  {7, 0x02, 182},
  {15, 0x02, 182},
  {28, 0x03, 182},
  {2, 0x02, 183},
  {7, 0x02, 183},
  {15, 0x02, 183},
  {28, 0x03, 183},
},
/* 115 */
{
  {3, 0x02, 180},
  {4, 0x02, 180},
  {8, 0x02, 180},
  {11, 0x02, 180},
  {16, 0x02, 180},
  {21, 0x02, 180},
  {29, 0x02, 180},
  {37, 0x03, 180},
  {3, 0x02, 181},
  {4, 0x02, 181},
  {8, 0x02, 181},
  {11, 0x02, 181},
  {16, 0x02, 181},
  {21, 0x02, 181},
  {29, 0x02, 181},
  {37, 0x03, 181},
},
/* 116 */
{
  {3, 0x02, 182},
  {4, 0x02, 182},
  {8, 0x02, 182},
  {11, 0x02, 182},
  {16, 0x02, 182},
  {21, 0x02, 182},
  {29, 0x02, 182},
  {37, 0x03, 182},
  {3, 0x02, 183},
  {4, 0x02, 183},
  {8, 0x02, 183},
  {11, 0x02, 183},
  {16, 0x02, 183},
  {21, 0x02, 183},
  {29, 0x02, 183},
  {37, 0x03, 183},
},
/* 117 */
{
  {2, 0x02, 184},
  {7, 0x02, 184},
  {15, 0x02, 184},
  {28, 0x03, 184},
  {2, 0x02, 185},
  {7, 0x02, 185},
  {15, 0x02, 185},
  {28, 0x03, 185},
  {2, 0x02, 186},
  {7, 0x02, 186},
  {15, 0x02, 186},
  {28, 0x03, 186},
  {2, 0x02, 187},
  {7, 0x02, 187},
  {15, 0x02, 187},
  {28, 0x03, 187},
},
/* 118 */
{
  {3, 0x02, 184},
  {4, 0x02, 184},
  {8, 0x02, 184},
  {11, 0x02, 184},
  {16, 0x02, 184},
  {21, 0x02, 184},
  {29, 0x02, 184},
  {37, 0x03, 184},
  {3, 0x02, 185},
  {4, 0x02, 185},
  {8, 0x02, 185},
  {11, 0x02, 185},
  {16, 0x02, 185},
  {21, 0x02, 185},
  {29, 0x02, 185},
  {37, 0x03, 185},
},
/* 119 */
{
  {3, 0x02, 186},
  {4, 0x02, 186},
  {8, 0x02, 186},
  {11, 0x02, 186},
  {16, 0x02, 186},
  {21, 0x02, 186},
  {29, 0x02, 186},
  {37, 0x03, 186},
  {3, 0x02, 187},
  {4, 0x02, 187},
  {8, 0x02, 187},
  {11, 0x02, 187},
  {16, 0x02, 187},
  {21, 0x02, 187},
  {29, 0x02, 187},
  {37, 0x03, 187},
},
/* 120 */
{
  {1, 0x02, 188},
  {14, 0x03, 188},
  {1, 0x02, 189},
  {14, 0x03, 189},
  {1, 0x02, 190},
  {14, 0x03, 190},
  {1, 0x02, 191},
  {14, 0x03, 191},
  {1, 0x02, 192},
  {14, 0x03, 192},
  {1, 0x02, 193},
  {14, 0x03, 193},
  {1, 0x02, 194},
  {14, 0x03, 194},
  {1, 0x02, 195},
  {14, 0x03, 195},
},
/* 121 */
{
  {2, 0x02, 188},
  {7, 0x02, 188},
  {15, 0x02, 188},
  {28, 0x03, 188},
  {2, 0x02, 189},
  {7, 0x02, 189},
  {15, 0x02, 189},
  {28, 0x03, 189},
  {2, 0x02, 190},
  {7, 0x02, 190},
  {15, 0x02, 190},
  {28, 0x03, 190},
  {2, 0x02, 191},
  {7, 0x02, 191},
  {15, 0x02, 191},
  {28, 0x03, 191},
},
/* 122 */
{
  {3, 0x02, 188},
  {4, 0x02, 188},
  {8, 0x02, 188},
  {11, 0x02, 188},
  {16, 0x02, 188},
  {21, 0x02, 188},
  {29, 0x02, 188},
  {37, 0x03, 188},
  {3, 0x02, 189},
  {4, 0x02, 189},
  {8, 0x02, 189},
  {11, 0x02, 189},
  {16, 0x02, 189},
  {21, 0x02, 189},
  {29, 0x02, 189},
  {37, 0x03, 189},
},
/* 123 */
{
  {3, 0x02, 190},
  {4, 0x02, 190},
  {8, 0x02, 190},
  {11, 0x02, 190},
  {16, 0x02, 190},
  {21, 0x02, 190},
  {29, 0x02, 190},
  {37, 0x03, 190},
  {3, 0x02, 191},
  {4, 0x02, 191},
  {8, 0x02, 191},
  {11, 0x02, 191},
  {16, 0x02, 191},
  {21, 0x02, 191},
  {29, 0x02, 191},
  {37, 0x03, 191},
},
/* 124 */
{
  {2, 0x02, 192},
  {7, 0x02, 192},
  {15, 0x02, 192},
  {28, 0x03, 192},
  {2, 0x02, 193},
  {7, 0x02, 193},
  {15, 0x02, 193},
  {28, 0x03, 193},
  {2, 0x02, 194},
  {7, 0x02, 194},
  {15, 0x02, 194},
  {28, 0x03, 194},
  {2, 0x02, 195},
  {7, 0x02, 195},
  {15, 0x02, 195},
  {28, 0x03, 195},
},
/* 125 */
{
  {3, 0x02, 192},
  {4, 0x02, 192},
  {8, 0x02, 192},
  {11, 0x02, 192},
  {16, 0x02, 192},
  {21, 0x02, 192},
  {29, 0x02, 192},
  {37, 0x03, 192},
  {3, 0x02, 193},
  {4, 0x02, 193},
  {8, 0x02, 193},
  {11, 0x02, 193},
  {16, 0x02, 193},
  {21, 0x02, 193},
  {29, 0x02, 193},
  {37, 0x03, 193},
},
/* 126 */
{
  {3, 0x02, 194},
  {4, 0x02, 194},
  {8, 0x02, 194},
  {11, 0x02, 194},
  {16, 0x02, 194},
  {21, 0x02, 194},
  {29, 0x02, 194},
  {37, 0x03, 194},
  {3, 0x02, 195},
  {4, 0x02, 195},
  {8, 0x02, 195},
  {11, 0x02, 195},
  {16, 0x02, 195},
  {21, 0x02, 195},
  {29, 0x02, 195},
  {37, 0x03, 195},
},
/* 127 */
{
  {131, 0x00, 0},
  {132, 0x00, 0},
  {134, 0x00, 0},
  {135, 0x00, 0},
  {138, 0x00, 0},
  {139, 0x00, 0},
  {141, 0x00, 0},
  {142, 0x00, 0},
  {146, 0x00, 0},
  {147, 0x00, 0},
  {149, 0x00, 0},
  {150, 0x00, 0},
  {153, 0x00, 0},
  {154, 0x00, 0},
  {156, 0x00, 0},
  {157, 0x00, 0},
},
/* 128 */
{
  {0, 0x03, 196},
  {0, 0x03, 197},
  {0, 0x03, 198},
  {0, 0x03, 199},
  {0, 0x03, 200},
  {0, 0x03, 201},
  {0, 0x03, 202},
  {0, 0x03, 203},
  {0, 0x03, 204},
  {0, 0x03, 205},
  {0, 0x03, 206},
  {0, 0x03, 207},
  {0, 0x03, 208},
  {0, 0x03, 209},
  {0, 0x03, 210},
  {0, 0x03, 211},
},
/* 129 */
{
  {1, 0x02, 196},
  {14, 0x03, 196},
  {1, 0x02, 197},
  {14, 0x03, 197},
  {1, 0x02, 198},
  {14, 0x03, 198},
  {1, 0x02, 199},
  {14, 0x03, 199},
  {1, 0x02, 200},
  {14, 0x03, 200},
  {1, 0x02, 201},
  {14, 0x03, 201},
  {1, 0x02, 202},
  {14, 0x03, 202},
  {1, 0x02, 203},
  {14, 0x03, 203},
},
/* 130 */
{
  {2, 0x02, 196},
  {7, 0x02, 196},
  {15, 0x02, 196},
  {28, 0x03, 196},
  {2, 0x02, 197},
  {7, 0x02, 197},
  {15, 0x02, 197},
  {28, 0x03, 197},
  {2, 0x02, 198},
  {7, 0x02, 198},
  {15, 0x02, 198},
  {28, 0x03, 198},
  {2, 0x02, 199},
  {7, 0x02, 199},
  {15, 0x02, 199},
  {28, 0x03, 199},
},
/* 131 */
{
  {3, 0x02, 196},
  {4, 0x02, 196},
  {8, 0x02, 196},
  {11, 0x02, 196},
  {16, 0x02, 196},
  {21, 0x02, 196},
  {29, 0x02, 196},
  {37, 0x03, 196},
  {3, 0x02, 197},
  {4, 0x02, 197},
  {8, 0x02, 197},
  {11, 0x02, 197},
  {16, 0x02, 197},
  {21, 0x02, 197},
  {29, 0x02, 197},
  {37, 0x03, 197},
},
/* 132 */
{
  {3, 0x02, 198},
  {4, 0x02, 198},
  {8, 0x02, 198},
  {11, 0x02, 198},
  {16, 0x02, 198},
  {21, 0x02, 198},
  {29, 0x02, 198},
  {37, 0x03, 198},
  {3, 0x02, 199},
  {4, 0x02, 199},
  {8, 0x02, 199},
  {11, 0x02, 199},
  {16, 0x02, 199},
  {21, 0x02, 199},
  {29, 0x02, 199},
  {37, 0x03, 199},
},
/* 133 */
{
  {2, 0x02, 200},
  {7, 0x02, 200},
  {15, 0x02, 200},
  {28, 0x03, 200},
  {2, 0x02, 201},
  {7, 0x02, 201},
  {15, 0x02, 201},
  {28, 0x03, 201},
  {2, 0x02, 202},
  {7, 0x02, 202},
  {15, 0x02, 202},
  {28, 0x03, 202},
  {2, 0x02, 203},
  {7, 0x02, 203},
  {15, 0x02, 203},
  {28, 0x03, 203},
},
/* 134 */
{
  {3, 0x02, 200},
  {4, 0x02, 200},
  {8, 0x02, 200},
  {11, 0x02, 200},
  {16, 0x02, 200},
  {21, 0x02, 200},
  {29, 0x02, 200},
  {37, 0x03, 200},
  {3, 0x02, 201},
  {4, 0x02, 201},
  {8, 0x02, 201},
  {11, 0x02, 201},
  {16, 0x02, 201},
  {21, 0x02, 201},
  {29, 0x02, 201},
  {37, 0x03, 201},
},
/* 135 */
{
  {3, 0x02, 202},
  {4, 0x02, 202},
  {8, 0x02, 202},
  {11, 0x02, 202},
  {16, 0x02, 202},
  {21, 0x02, 202},
  {29, 0x02, 202},
  {37, 0x03, 202},
  {3, 0x02, 203},
  {4, 0x02, 203},
  {8, 0x02, 203},
  {11, 0x02, 203},
  {16, 0x02, 203},
  {21, 0x02, 203},
  {29, 0x02, 203},
  {37, 0x03, 203},
},
/* 136 */
{
  {1, 0x02, 204},
  {14, 0x03, 204},
  {1, 0x02, 205},
  {14, 0x03, 205},
  {1, 0x02, 206},
  {14, 0x03, 206},
  {1, 0x02, 207},
  {14, 0x03, 207},
  {1, 0x02, 208},
  {14, 0x03, 208},
  {1, 0x02, 209},
  {14, 0x03, 209},
  {1, 0x02, 210},
  {14, 0x03, 210},
  {1, 0x02, 211},
  {14, 0x03, 211},
},
/* 137 */
{
  {2, 0x02, 204},
  {7, 0x02, 204},
  {15, 0x02, 204},
  {28, 0x03, 204},
  {2, 0x02, 205},
  {7, 0x02, 205},
  {15, 0x02, 205},
  {28, 0x03, 205},
  {2, 0x02, 206},
  {7, 0x02, 206},
  {15, 0x02, 206},
  {28, 0x03, 206},
  {2, 0x02, 207},
  {7, 0x02, 207},
  {15, 0x02, 207},
  {28, 0x03, 207},
},
/* 138 */
{
  {3, 0x02, 204},
  {4, 0x02, 204},
  {8, 0x02, 204},
  {11, 0x02, 204},
  {16, 0x02, 204},
  {21, 0x02, 204},
  {29, 0x02, 204},
  {37, 0x03, 204},
  {3, 0x02, 205},
  {4, 0x02, 205},
  {8, 0x02, 205},
  {11, 0x02, 205},
  {16, 0x02, 205},
  {21, 0x02, 205},
  {29, 0x02, 205},
  {37, 0x03, 205},
},
/* 139 */
{
  {3, 0x02, 206},
  {4, 0x02, 206},
  {8, 0x02, 206},
  {11, 0x02, 206},
  {16, 0x02, 206},
  {21, 0x02, 206},
  {29, 0x02, 206},
  {37, 0x03, 206},
  {3, 0x02, 207},
  {4, 0x02, 207},
  {8, 0x02, 207},
  {11, 0x02, 207},
  {16, 0x02, 207},
  {21, 0x02, 207},
  {29, 0x02, 207},
  {37, 0x03, 207},
},
/* 140 */
{
  {2, 0x02, 208},
  {7, 0x02, 208},
  {15, 0x02, 208},
  {28, 0x03, 208},
  {2, 0x02, 209},
  {7, 0x02, 209},
  {15, 0x02, 209},
  {28, 0x03, 209},
  {2, 0x02, 210},
  {7, 0x02, 210},
  {15, 0x02, 210},
  {28, 0x03, 210},
  {2, 0x02, 211},
  {7, 0x02, 211},
  {15, 0x02, 211},
  {28, 0x03, 211},
},
/* 141 */
{
  {3, 0x02, 208},
  {4, 0x02, 208},
  {8, 0x02, 208},
  {11, 0x02, 208},
  {16, 0x02, 208},
  {21, 0x02, 208},
  {29, 0x02, 208},
  {37, 0x03, 208},
  {3, 0x02, 209},
  {4, 0x02, 209},
  {8, 0x02, 209},
  {11, 0x02, 209},
  {16, 0x02, 209},
  {21, 0x02, 209},
  {29, 0x02, 209},
  {37, 0x03, 209},
},
/* 142 */
{
  {3, 0x02, 210},
  {4, 0x02, 210},
  {8, 0x02, 210},
  {11, 0x02, 210},
  {16, 0x02, 210},
  {21, 0x02, 210},
  {29, 0x02, 210},
  {37, 0x03, 210},
  {3, 0x02, 211},
  {4, 0x02, 211},
  {8, 0x02, 211},
  {11, 0x02, 211},
  {16, 0x02, 211},
  {21, 0x02, 211},
  {29, 0x02, 211},
  {37, 0x03, 211},
},
/* 143 */
{
  {0, 0x03, 212},
  {0, 0x03, 213},
  {0, 0x03, 214},
  {0, 0x03, 215},
  {0, 0x03, 216},
  {0, 0x03, 217},
  {0, 0x03, 218},
  {0, 0x03, 219},
  {0, 0x03, 220},
  {0, 0x03, 221},
  {0, 0x03, 222},
  {0, 0x03, 223},
  {0, 0x03, 224},
  {0, 0x03, 225},
  {0, 0x03, 226},
  {0, 0x03, 227},
},
/* 144 */
{
  {1, 0x02, 212},
  {14, 0x03, 212},
  {1, 0x02, 213},
  {14, 0x03, 213},
  {1, 0x02, 214},
  {14, 0x03, 214},
  {1, 0x02, 215},
  {14, 0x03, 215},
  {1, 0x02, 216},
  {14, 0x03, 216},
  {1, 0x02, 217},
  {14, 0x03, 217},
  {1, 0x02, 218},
  {14, 0x03, 218},
  {1, 0x02, 219},
  {14, 0x03, 219},
},
/* 145 */
{
  {2, 0x02, 212},
  {7, 0x02, 212},
  {15, 0x02, 212},
  {28, 0x03, 212},
  {2, 0x02, 213},
  {7, 0x02, 213},
  {15, 0x02, 213},
  {28, 0x03, 213},
  {2, 0x02, 214},
  {7, 0x02, 214},
  {15, 0x02, 214},
  {28, 0x03, 214},
  {2, 0x02, 215},
  {7, 0x02, 215},
  {15, 0x02, 215},
  {28, 0x03, 215},
},
/* 146 */
{
  {3, 0x02, 212},
  {4, 0x02, 212},
  {8, 0x02, 212},
  {11, 0x02, 212},
  {16, 0x02, 212},
  {21, 0x02, 212},
  {29, 0x02, 212},
  {37, 0x03, 212},
  {3, 0x02, 213},
  {4, 0x02, 213},
  {8, 0x02, 213},
  {11, 0x02, 213},
  {16, 0x02, 213},
  {21, 0x02, 213},
  {29, 0x02, 213},
  {37, 0x03, 213},
},
/* 147 */
{
  {3, 0x02, 214},
  {4, 0x02, 214},
  {8, 0x02, 214},
  {11, 0x02, 214},
  {16, 0x02, 214},
  {21, 0x02, 214},
  {29, 0x02, 214},
  {37, 0x03, 214},
  {3, 0x02, 215},
  {4, 0x02, 215},
  {8, 0x02, 215},
  {11, 0x02, 215},
  {16, 0x02, 215},
  {21, 0x02, 215},
  {29, 0x02, 215},
  {37, 0x03, 215},
},
/* 148 */
{
  {2, 0x02, 216},
  {7, 0x02, 216},
  {15, 0x02, 216},
  {28, 0x03, 216},
  {2, 0x02, 217},
  {7, 0x02, 217},
  {15, 0x02, 217},
  {28, 0x03, 217},
  {2, 0x02, 218},
  {7, 0x02, 218},
  {15, 0x02, 218},
  {28, 0x03, 218},
  {2, 0x02, 219},
  {7, 0x02, 219},
  {15, 0x02, 219},
  {28, 0x03, 219},
},
/* 149 */
{
  {3, 0x02, 216},
  {4, 0x02, 216},
  {8, 0x02, 216},
  {11, 0x02, 216},
  {16, 0x02, 216},
  {21, 0x02, 216},
  {29, 0x02, 216},
  {37, 0x03, 216},
  {3, 0x02, 217},
  {4, 0x02, 217},
  {8, 0x02, 217},
  {11, 0x02, 217},
  {16, 0x02, 217},
  {21, 0x02, 217},
  {29, 0x02, 217},
  {37, 0x03, 217},
},
/* 150 */
{
  {3, 0x02, 218},
  {4, 0x02, 218},
  {8, 0x02, 218},
  {11, 0x02, 218},
  {16, 0x02, 218},
  {21, 0x02, 218},
  {29, 0x02, 218},
  {37, 0x03, 218},
  {3, 0x02, 219},
  {4, 0x02, 219},
  {8, 0x02, 219},
  {11, 0x02, 219},
  {16, 0x02, 219},
  {21, 0x02, 219},
  {29, 0x02, 219},
  {37, 0x03, 219},
},
/* 151 */
{
  {1, 0x02, 220},
  {14, 0x03, 220},
  {1, 0x02, 221},
  {14, 0x03, 221},
  {1, 0x02, 222},
  {14, 0x03, 222},
  {1, 0x02, 223},
  {14, 0x03, 223},
  {1, 0x02, 224},
  {14, 0x03, 224},
  {1, 0x02, 225},
  {14, 0x03, 225},
  {1, 0x02, 226},
  {14, 0x03, 226},
  {1, 0x02, 227},
  {14, 0x03, 227},
},
/* 152 */
{
  {2, 0x02, 220},
  {7, 0x02, 220},
  {15, 0x02, 220},
  {28, 0x03, 220},
  {2, 0x02, 221},
  {7, 0x02, 221},
  {15, 0x02, 221},
  {28, 0x03, 221},
  {2, 0x02, 222},
  {7, 0x02, 222},
  {15, 0x02, 222},
  {28, 0x03, 222},
  {2, 0x02, 223},
  {7, 0x02, 223},
  {15, 0x02, 223},
  {28, 0x03, 223},
},
/* 153 */
{
  {3, 0x02, 220},
  {4, 0x02, 220},
  {8, 0x02, 220},
  {11, 0x02, 220},
  {16, 0x02, 220},
  {21, 0x02, 220},
  {29, 0x02, 220},
  {37, 0x03, 220},
  {3, 0x02, 221},
  {4, 0x02, 221},
  {8, 0x02, 221},
  {11, 0x02, 221},
  {16, 0x02, 221},
  {21, 0x02, 221},
  {29, 0x02, 221},
  {37, 0x03, 221},
},
/* 154 */
{
  {3, 0x02, 222},
  {4, 0x02, 222},
  {8, 0x02, 222},
  {11, 0x02, 222},
  {16, 0x02, 222},
  {21, 0x02, 222},
  {29, 0x02, 222},
  {37, 0x03, 222},
  {3, 0x02, 223},
  {4, 0x02, 223},
  {8, 0x02, 223},
  {11, 0x02, 223},
  {16, 0x02, 223},
  {21, 0x02, 223},
  {29, 0x02, 223},
  {37, 0x03, 223},
},
/* 155 */
{
  {2, 0x02, 224},
  {7, 0x02, 224},
  {15, 0x02, 224},
  {28, 0x03, 224},
  {2, 0x02, 225},
  {7, 0x02, 225},
  {15, 0x02, 225},
  {28, 0x03, 225},
  {2, 0x02, 226},
  {7, 0x02, 226},
  {15, 0x02, 226},
  {28, 0x03, 226},
  {2, 0x02, 227},
  {7, 0x02, 227},
  {15, 0x02, 227},
  {28, 0x03, 227},
},
/* 156 */
{
  {3, 0x02, 224},
  {4, 0x02, 224},
  {8, 0x02, 224},
  {11, 0x02, 224},
  {16, 0x02, 224},
  {21, 0x02, 224},
  {29, 0x02, 224},
  {37, 0x03, 224},
  {3, 0x02, 225},
  {4, 0x02, 225},
  {8, 0x02, 225},
  {11, 0x02, 225},
  {16, 0x02, 225},
  {21, 0x02, 225},
  {29, 0x02, 225},
  {37, 0x03, 225},
},
/* 157 */
{
  {3, 0x02, 226},
  {4, 0x02, 226},
  {8, 0x02, 226},
  {11, 0x02, 226},
  {16, 0x02, 226},
  {21, 0x02, 226},
  {29, 0x02, 226},
  {37, 0x03, 226},
  {3, 0x02, 227},
  {4, 0x02, 227},
  {8, 0x02, 227},
  {11, 0x02, 227},
  {16, 0x02, 227},
  {21, 0x02, 227},
  {29, 0x02, 227},
  {37, 0x03, 227},
},
/* 158 */
{
  {162, 0x00, 0},
  {165, 0x00, 0},
  {169, 0x00, 0},
  {172, 0x00, 0},
  {177, 0x00, 0},
  {180, 0x00, 0},
  {184, 0x00, 0},
  {187, 0x00, 0},
  {196, 0x00, 0},
  {203, 0x00, 0},
  {211, 0x00, 0},
  {218, 0x00, 0},
  {227, 0x00, 0},
  {234, 0x00, 0},
  {242, 0x00, 0},
  {249, 0x00, 0},
},
/* 159 */
{
  {163, 0x00, 0},
  {164, 0x00, 0},
  {166, 0x00, 0},
  {167, 0x00, 0},
  {170, 0x00, 0},
  {171, 0x00, 0},
  {173, 0x00, 0},
  {174, 0x00, 0},
  {178, 0x00, 0},
  {179, 0x00, 0},
  {181, 0x00, 0},
  {182, 0x00, 0},
  {185, 0x00, 0},
  {186, 0x00, 0},
  {188, 0x00, 0},
  {190, 0x00, 0},
},
/* 160 */
{
  {0, 0x03, 228},
  {0, 0x03, 229},
  {0, 0x03, 230},
  {0, 0x03, 231},
  {0, 0x03, 232},
  {0, 0x03, 233},
  {0, 0x03, 234},
  {0, 0x03, 235},
  {0, 0x03, 236},
  {0, 0x03, 237},
  {0, 0x03, 238},
  {0, 0x03, 239},
  {0, 0x03, 240},
  {0, 0x03, 241},
  {0, 0x03, 242},
  {0, 0x03, 243},
},
/* 161 */
{
  {1, 0x02, 228},
  {14, 0x03, 228},
  {1, 0x02, 229},
  {14, 0x03, 229},
  {1, 0x02, 230},
  {14, 0x03, 230},
  {1, 0x02, 231},
  {14, 0x03, 231},
  {1, 0x02, 232},
  {14, 0x03, 232},
  {1, 0x02, 233},
  {14, 0x03, 233},
  {1, 0x02, 234},
  {14, 0x03, 234},
  {1, 0x02, 235},
  {14, 0x03, 235},
},
/* 162 */
{
  {2, 0x02, 228},
  {7, 0x02, 228},
  {15, 0x02, 228},
  {28, 0x03, 228},
  {2, 0x02, 229},
  {7, 0x02, 229},
  {15, 0x02, 229},
  {28, 0x03, 229},
  {2, 0x02, 230},
  {7, 0x02, 230},
  {15, 0x02, 230},
  {28, 0x03, 230},
  {2, 0x02, 231},
  {7, 0x02, 231},
  {15, 0x02, 231},
  {28, 0x03, 231},
},
/* 163 */
{
  {3, 0x02, 228},
  {4, 0x02, 228},
  {8, 0x02, 228},
  {11, 0x02, 228},
  {16, 0x02, 228},
  {21, 0x02, 228},
  {29, 0x02, 228},
  {37, 0x03, 228},
  {3, 0x02, 229},
  {4, 0x02, 229},
  {8, 0x02, 229},
  {11, 0x02, 229},
  {16, 0x02, 229},
  {21, 0x02, 229},
  {29, 0x02, 229},
  {37, 0x03, 229},
},
/* 164 */
{
  {3, 0x02, 230},
  {4, 0x02, 230},
  {8, 0x02, 230},
  {11, 0x02, 230},
  {16, 0x02, 230},
  {21, 0x02, 230},
  {29, 0x02, 230},
  {37, 0x03, 230},
  {3, 0x02, 231},
  {4, 0x02, 231},
  {8, 0x02, 231},
  {11, 0x02, 231},
  {16, 0x02, 231},
  {21, 0x02, 231},
  {29, 0x02, 231},
  {37, 0x03, 231},
},
/* 165 */
{
  {2, 0x02, 232},
  {7, 0x02, 232},
  {15, 0x02, 232},
  {28, 0x03, 232},
  {2, 0x02, 233},
  {7, 0x02, 233},
  {15, 0x02, 233},
  {28, 0x03, 233},
  {2, 0x02, 234},
  {7, 0x02, 234},
  {15, 0x02, 234},
  {28, 0x03, 234},
  {2, 0x02, 235},
  {7, 0x02, 235},
  {15, 0x02, 235},
  {28, 0x03, 235},
},
/* 166 */
{
  {3, 0x02, 232},
  {4, 0x02, 232},
  {8, 0x02, 232},
  {11, 0x02, 232},
  {16, 0x02, 232},
  {21, 0x02, 232},
  {29, 0x02, 232},
  {37, 0x03, 232},
  {3, 0x02, 233},
  {4, 0x02, 233},
  {8, 0x02, 233},
  {11, 0x02, 233},
  {16, 0x02, 233},
  {21, 0x02, 233},
  {29, 0x02, 233},
  {37, 0x03, 233},
},
/* 167 */
{
  {3, 0x02, 234},
  {4, 0x02, 234},
  {8, 0x02, 234},
  {11, 0x02, 234},
  {16, 0x02, 234},
  {21, 0x02, 234},
  {29, 0x02, 234},
  {37, 0x03, 234},
  {3, 0x02, 235},
  {4, 0x02, 235},
  {8, 0x02, 235},
  {11, 0x02, 235},
  {16, 0x02, 235},
  {21, 0x02, 235},
  {29, 0x02, 235},
  {37, 0x03, 235},
},
/* 168 */
{
  {1, 0x02, 236},
  {14, 0x03, 236},
  {1, 0x02, 237},
  {14, 0x03, 237},
  {1, 0x02, 238},
  {14, 0x03, 238},
  {1, 0x02, 239},
  {14, 0x03, 239},
  {1, 0x02, 240},
  {14, 0x03, 240},
  {1, 0x02, 241},
  {14, 0x03, 241},
  {1, 0x02, 242},
  {14, 0x03, 242},
  {1, 0x02, 243},
  {14, 0x03, 243},
},
/* 169 */
{
  {2, 0x02, 236},
  {7, 0x02, 236},
  {15, 0x02, 236},
  {28, 0x03, 236},
  {2, 0x02, 237},
  {7, 0x02, 237},
  {15, 0x02, 237},
  {28, 0x03, 237},
  {2, 0x02, 238},
  {7, 0x02, 238},
  {15, 0x02, 238},
  {28, 0x03, 238},
  {2, 0x02, 239},
  {7, 0x02, 239},
  {15, 0x02, 239},
  {28, 0x03, 239},
},
/* 170 */
{
  {3, 0x02, 236},
  {4, 0x02, 236},
  {8, 0x02, 236},
  {11, 0x02, 236},
  {16, 0x02, 236},
  {21, 0x02, 236},
  {29, 0x02, 236},
  {37, 0x03, 236},
  {3, 0x02, 237},
  {4, 0x02, 237},
  {8, 0x02, 237},
  {11, 0x02, 237},
  {16, 0x02, 237},
  {21, 0x02, 237},
  {29, 0x02, 237},
  {37, 0x03, 237},
},
/* 171 */
{
  {3, 0x02, 238},
  {4, 0x02, 238},
  {8, 0x02, 238},
  {11, 0x02, 238},
  {16, 0x02, 238},
  {21, 0x02, 238},
  {29, 0x02, 238},
  {37, 0x03, 238},
  {3, 0x02, 239},
  {4, 0x02, 239},
  {8, 0x02, 239},
  {11, 0x02, 239},
  {16, 0x02, 239},
  {21, 0x02, 239},
  {29, 0x02, 239},
  {37, 0x03, 239},
},
/* 172 */
{
  {2, 0x02, 240},
  {7, 0x02, 240},
  {15, 0x02, 240},
  {28, 0x03, 240},
  {2, 0x02, 241},
  {7, 0x02, 241},
  {15, 0x02, 241},
  {28, 0x03, 241},
  {2, 0x02, 242},
  {7, 0x02, 242},
  {15, 0x02, 242},
  {28, 0x03, 242},
  {2, 0x02, 243},
  {7, 0x02, 243},
  {15, 0x02, 243},
  {28, 0x03, 243},
},
/* 173 */
{
  {3, 0x02, 240},
  {4, 0x02, 240},
  {8, 0x02, 240},
  {11, 0x02, 240},
  {16, 0x02, 240},
  {21, 0x02, 240},
  {29, 0x02, 240},
  {37, 0x03, 240},
  {3, 0x02, 241},
  {4, 0x02, 241},
  {8, 0x02, 241},
  {11, 0x02, 241},
  {16, 0x02, 241},
  {21, 0x02, 241},
  {29, 0x02, 241},
  {37, 0x03, 241},
},
/* 174 */
{
  {3, 0x02, 242},
  {4, 0x02, 242},
  {8, 0x02, 242},
  {11, 0x02, 242},
  {16, 0x02, 242},
  {21, 0x02, 242},
  {29, 0x02, 242},
  {37, 0x03, 242},
  {3, 0x02, 243},
  {4, 0x02, 243},
  {8, 0x02, 243},
  {11, 0x02, 243},
  {16, 0x02, 243},
  {21, 0x02, 243},
  {29, 0x02, 243},
  {37, 0x03, 243},
},
/* 175 */
{
  {0, 0x03, 244},
  {0, 0x03, 245},
  {0, 0x03, 246},
  {0, 0x03, 247},
  {0, 0x03, 248},
  {0, 0x03, 249},
  {0, 0x03, 250},
  {0, 0x03, 251},
  {0, 0x03, 252},
  {0, 0x03, 253},
  {0, 0x03, 254},
  {0, 0x03, 255},
  {-1, 0x00, 0},
  {189, 0x00, 0},
  {191, 0x00, 0},
  {192, 0x00, 0},
},
/* 176 */
{
  {1, 0x02, 244},
  {14, 0x03, 244},
  {1, 0x02, 245},
  {14, 0x03, 245},
  {1, 0x02, 246},
  {14, 0x03, 246},
  {1, 0x02, 247},
  {14, 0x03, 247},
  {1, 0x02, 248},
  {14, 0x03, 248},
  {1, 0x02, 249},
  {14, 0x03, 249},
  {1, 0x02, 250},
  {14, 0x03, 250},
  {1, 0x02, 251},
  {14, 0x03, 251},
},
/* 177 */
{
  {2, 0x02, 244},
  {7, 0x02, 244},
  {15, 0x02, 244},
  {28, 0x03, 244},
  {2, 0x02, 245},
  {7, 0x02, 245},
  {15, 0x02, 245},
  {28, 0x03, 245},
  {2, 0x02, 246},
  {7, 0x02, 246},
  {15, 0x02, 246},
  {28, 0x03, 246},
  {2, 0x02, 247},
  {7, 0x02, 247},
  {15, 0x02, 247},
  {28, 0x03, 247},
},
/* 178 */
{
  {3, 0x02, 244},
  {4, 0x02, 244},
  {8, 0x02, 244},
  {11, 0x02, 244},
  {16, 0x02, 244},
  {21, 0x02, 244},
  {29, 0x02, 244},
  {37, 0x03, 244},
  {3, 0x02, 245},
  {4, 0x02, 245},
  {8, 0x02, 245},
  {11, 0x02, 245},
  {16, 0x02, 245},
  {21, 0x02, 245},
  {29, 0x02, 245},
  {37, 0x03, 245},
},
/* 179 */
{
  {3, 0x02, 246},
  {4, 0x02, 246},
  {8, 0x02, 246},
  {11, 0x02, 246},
  {16, 0x02, 246},
  {21, 0x02, 246},
  {29, 0x02, 246},
  {37, 0x03, 246},
  {3, 0x02, 247},
  {4, 0x02, 247},
  {8, 0x02, 247},
  {11, 0x02, 247},
  {16, 0x02, 247},
  {21, 0x02, 247},
  {29, 0x02, 247},
  {37, 0x03, 247},
},
/* 180 */
{
  {2, 0x02, 248},
  {7, 0x02, 248},
  {15, 0x02, 248},
  {28, 0x03, 248},
  {2, 0x02, 249},
  {7, 0x02, 249},
  {15, 0x02, 249},
  {28, 0x03, 249},
  {2, 0x02, 250},
  {7, 0x02, 250},
  {15, 0x02, 250},
  {28, 0x03, 250},
  {2, 0x02, 251},
  {7, 0x02, 251},
  {15, 0x02, 251},
  {28, 0x03, 251},
},
/* 181 */
{
  {3, 0x02, 248},
  {4, 0x02, 248},
  {8, 0x02, 248},
  {11, 0x02, 248},
  {16, 0x02, 248},
  {21, 0x02, 248},
  {29, 0x02, 248},
  {37, 0x03, 248},
  {3, 0x02, 249},
  {4, 0x02, 249},
  {8, 0x02, 249},
  {11, 0x02, 249},
  {16, 0x02, 249},
  {21, 0x02, 249},
  {29, 0x02, 249},
  {37, 0x03, 249},
},
/* 182 */
{
  {3, 0x02, 250},
  {4, 0x02, 250},
  {8, 0x02, 250},
  {11, 0x02, 250},
  {16, 0x02, 250},
  {21, 0x02, 250},
  {29, 0x02, 250},
  {37, 0x03, 250},
  {3, 0x02, 251},
  {4, 0x02, 251},
  {8, 0x02, 251},
  {11, 0x02, 251},
  {16, 0x02, 251},
  {21, 0x02, 251},
  {29, 0x02, 251},
  {37, 0x03, 251},
},
/* 183 */
{
  {1, 0x02, 252},
  {14, 0x03, 252},
  {1, 0x02, 253},
  {14, 0x03, 253},
  {1, 0x02, 254},
  {14, 0x03, 254},
  {1, 0x02, 255},
  {14, 0x03, 255},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {0, 0x03, 0},
  {0, 0x03, 1},
  {0, 0x03, 2},
  {0, 0x03, 3},
  {0, 0x03, 4},
  {0, 0x03, 5},
},
/* 184 */
{
  {2, 0x02, 252},
  {7, 0x02, 252},
  {15, 0x02, 252},
  {28, 0x03, 252},
  {2, 0x02, 253},
  {7, 0x02, 253},
  {15, 0x02, 253},
  {28, 0x03, 253},
  {2, 0x02, 254},
  {7, 0x02, 254},
  {15, 0x02, 254},
  {28, 0x03, 254},
  {2, 0x02, 255},
  {7, 0x02, 255},
  {15, 0x02, 255},
  {28, 0x03, 255},
},
/* 185 */
{
  {3, 0x02, 252},
  {4, 0x02, 252},
  {8, 0x02, 252},
  {11, 0x02, 252},
  {16, 0x02, 252},
  {21, 0x02, 252},
  {29, 0x02, 252},
  {37, 0x03, 252},
  {3, 0x02, 253},
  {4, 0x02, 253},
  {8, 0x02, 253},
  {11, 0x02, 253},
  {16, 0x02, 253},
  {21, 0x02, 253},
  {29, 0x02, 253},
  {37, 0x03, 253},
},
/* 186 */
{
  {3, 0x02, 254},
  {4, 0x02, 254},
  {8, 0x02, 254},
  {11, 0x02, 254},
  {16, 0x02, 254},
  {21, 0x02, 254},
  {29, 0x02, 254},
  {37, 0x03, 254},
  {3, 0x02, 255},
  {4, 0x02, 255},
  {8, 0x02, 255},
  {11, 0x02, 255},
  {16, 0x02, 255},
  {21, 0x02, 255},
  {29, 0x02, 255},
  {37, 0x03, 255},
},
/* 187 */
{
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {1, 0x02, 0},
  {14, 0x03, 0},
  {1, 0x02, 1},
  {14, 0x03, 1},
  {1, 0x02, 2},
  {14, 0x03, 2},
  {1, 0x02, 3},
  {14, 0x03, 3},
  {1, 0x02, 4},
  {14, 0x03, 4},
  {1, 0x02, 5},
  {14, 0x03, 5},
},
/* 188 */
{
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {2, 0x02, 0},
  {7, 0x02, 0},
  {15, 0x02, 0},
  {28, 0x03, 0},
  {2, 0x02, 1},
  {7, 0x02, 1},
  {15, 0x02, 1},
  {28, 0x03, 1},
},
/* 189 */
{
  {3, 0x02, 0},
  {4, 0x02, 0},
  {8, 0x02, 0},
  {11, 0x02, 0},
  {16, 0x02, 0},
  {21, 0x02, 0},
  {29, 0x02, 0},
  {37, 0x03, 0},
  {3, 0x02, 1},
  {4, 0x02, 1},
  {8, 0x02, 1},
  {11, 0x02, 1},
  {16, 0x02, 1},
  {21, 0x02, 1},
  {29, 0x02, 1},
  {37, 0x03, 1},
},
/* 190 */
{
  {2, 0x02, 2},
  {7, 0x02, 2},
  {15, 0x02, 2},
  {28, 0x03, 2},
  {2, 0x02, 3},
  {7, 0x02, 3},
  {15, 0x02, 3},
  {28, 0x03, 3},
  {2, 0x02, 4},
  {7, 0x02, 4},
  {15, 0x02, 4},
  {28, 0x03, 4},
  {2, 0x02, 5},
  {7, 0x02, 5},
  {15, 0x02, 5},
  {28, 0x03, 5},
},
/* 191 */
{
  {3, 0x02, 2},
  {4, 0x02, 2},
  {8, 0x02, 2},
  {11, 0x02, 2},
  {16, 0x02, 2},
  {21, 0x02, 2},
  {29, 0x02, 2},
  {37, 0x03, 2},
  {3, 0x02, 3},
  {4, 0x02, 3},
  {8, 0x02, 3},
  {11, 0x02, 3},
  {16, 0x02, 3},
  {21, 0x02, 3},
  {29, 0x02, 3},
  {37, 0x03, 3},
},
/* 192 */
{
  {3, 0x02, 4},
  {4, 0x02, 4},
  {8, 0x02, 4},
  {11, 0x02, 4},
  {16, 0x02, 4},
  {21, 0x02, 4},
  {29, 0x02, 4},
  {37, 0x03, 4},
  {3, 0x02, 5},
  {4, 0x02, 5},
  {8, 0x02, 5},
  {11, 0x02, 5},
  {16, 0x02, 5},
  {21, 0x02, 5},
  {29, 0x02, 5},
  {37, 0x03, 5},
},
/* 193 */
{
  {197, 0x00, 0},
  {200, 0x00, 0},
  {204, 0x00, 0},
  {207, 0x00, 0},
  {212, 0x00, 0},
  {215, 0x00, 0},
  {219, 0x00, 0},
  {222, 0x00, 0},
  {228, 0x00, 0},
  {231, 0x00, 0},
  {235, 0x00, 0},
  {238, 0x00, 0},
  {243, 0x00, 0},
  {246, 0x00, 0},
  {250, 0x00, 0},
  {253, 0x00, 0},
},
/* 194 */
{
  {198, 0x00, 0},
  {199, 0x00, 0},
  {201, 0x00, 0},
  {202, 0x00, 0},
  {205, 0x00, 0},
  {206, 0x00, 0},
  {208, 0x00, 0},
  {209, 0x00, 0},
  {213, 0x00, 0},
  {214, 0x00, 0},
  {216, 0x00, 0},
  {217, 0x00, 0},
  {220, 0x00, 0},
  {221, 0x00, 0},
  {223, 0x00, 0},
  {224, 0x00, 0},
},
/* 195 */
{
  {0, 0x03, 6},
  {0, 0x03, 7},
  {0, 0x03, 8},
  {0, 0x03, 9},
  {0, 0x03, 10},
  {0, 0x03, 11},
  {0, 0x03, 12},
  {0, 0x03, 13},
  {0, 0x03, 14},
  {0, 0x03, 15},
  {0, 0x03, 16},
  {0, 0x03, 17},
  {0, 0x03, 18},
  {0, 0x03, 19},
  {0, 0x03, 20},
  {0, 0x03, 21},
},
/* 196 */
{
  {1, 0x02, 6},
  {14, 0x03, 6},
  {1, 0x02, 7},
  {14, 0x03, 7},
  {1, 0x02, 8},
  {14, 0x03, 8},
  {1, 0x02, 9},
  {14, 0x03, 9},
  {1, 0x02, 10},
  {14, 0x03, 10},
  {1, 0x02, 11},
  {14, 0x03, 11},
  {1, 0x02, 12},
  {14, 0x03, 12},
  {1, 0x02, 13},
  {14, 0x03, 13},
},
/* 197 */
{
  {2, 0x02, 6},
  {7, 0x02, 6},
  {15, 0x02, 6},
  {28, 0x03, 6},
  {2, 0x02, 7},
  {7, 0x02, 7},
  {15, 0x02, 7},
  {28, 0x03, 7},
  {2, 0x02, 8},
  {7, 0x02, 8},
  {15, 0x02, 8},
  {28, 0x03, 8},
  {2, 0x02, 9},
  {7, 0x02, 9},
  {15, 0x02, 9},
  {28, 0x03, 9},
},
/* 198 */
{
  {3, 0x02, 6},
  {4, 0x02, 6},
  {8, 0x02, 6},
  {11, 0x02, 6},
  {16, 0x02, 6},
  {21, 0x02, 6},
  {29, 0x02, 6},
  {37, 0x03, 6},
  {3, 0x02, 7},
  {4, 0x02, 7},
  {8, 0x02, 7},
  {11, 0x02, 7},
  {16, 0x02, 7},
  {21, 0x02, 7},
  {29, 0x02, 7},
  {37, 0x03, 7},
},
/* 199 */
{
  {3, 0x02, 8},
  {4, 0x02, 8},
  {8, 0x02, 8},
  {11, 0x02, 8},
  {16, 0x02, 8},
  {21, 0x02, 8},
  {29, 0x02, 8},
  {37, 0x03, 8},
  {3, 0x02, 9},
  {4, 0x02, 9},
  {8, 0x02, 9},
  {11, 0x02, 9},
  {16, 0x02, 9},
  {21, 0x02, 9},
  {29, 0x02, 9},
  {37, 0x03, 9},
},
/* 200 */
{
  {2, 0x02, 10},
  {7, 0x02, 10},
  {15, 0x02, 10},
  {28, 0x03, 10},
  {2, 0x02, 11},
  {7, 0x02, 11},
  {15, 0x02, 11},
  {28, 0x03, 11},
  {2, 0x02, 12},
  {7, 0x02, 12},
  {15, 0x02, 12},
  {28, 0x03, 12},
  {2, 0x02, 13},
  {7, 0x02, 13},
  {15, 0x02, 13},
  {28, 0x03, 13},
},
/* 201 */
{
  {3, 0x02, 10},
  {4, 0x02, 10},
  {8, 0x02, 10},
  {11, 0x02, 10},
  {16, 0x02, 10},
  {21, 0x02, 10},
  {29, 0x02, 10},
  {37, 0x03, 10},
  {3, 0x02, 11},
  {4, 0x02, 11},
  {8, 0x02, 11},
  {11, 0x02, 11},
  {16, 0x02, 11},
  {21, 0x02, 11},
  {29, 0x02, 11},
  {37, 0x03, 11},
},
/* 202 */
{
  {3, 0x02, 12},
  {4, 0x02, 12},
  {8, 0x02, 12},
  {11, 0x02, 12},
  {16, 0x02, 12},
  {21, 0x02, 12},
  {29, 0x02, 12},
  {37, 0x03, 12},
  {3, 0x02, 13},
  {4, 0x02, 13},
  {8, 0x02, 13},
  {11, 0x02, 13},
  {16, 0x02, 13},
  {21, 0x02, 13},
  {29, 0x02, 13},
  {37, 0x03, 13},
},
/* 203 */
{
  {1, 0x02, 14},
  {14, 0x03, 14},
  {1, 0x02, 15},
  {14, 0x03, 15},
  {1, 0x02, 16},
  {14, 0x03, 16},
  {1, 0x02, 17},
  {14, 0x03, 17},
  {1, 0x02, 18},
  {14, 0x03, 18},
  {1, 0x02, 19},
  {14, 0x03, 19},
  {1, 0x02, 20},
  {14, 0x03, 20},
  {1, 0x02, 21},
  {14, 0x03, 21},
},
/* 204 */
{
  {2, 0x02, 14},
  {7, 0x02, 14},
  {15, 0x02, 14},
  {28, 0x03, 14},
  {2, 0x02, 15},
  {7, 0x02, 15},
  {15, 0x02, 15},
  {28, 0x03, 15},
  {2, 0x02, 16},
  {7, 0x02, 16},
  {15, 0x02, 16},
  {28, 0x03, 16},
  {2, 0x02, 17},
  {7, 0x02, 17},
  {15, 0x02, 17},
  {28, 0x03, 17},
},
/* 205 */
{
  {3, 0x02, 14},
  {4, 0x02, 14},
  {8, 0x02, 14},
  {11, 0x02, 14},
  {16, 0x02, 14},
  {21, 0x02, 14},
  {29, 0x02, 14},
  {37, 0x03, 14},
  {3, 0x02, 15},
  {4, 0x02, 15},
  {8, 0x02, 15},
  {11, 0x02, 15},
  {16, 0x02, 15},
  {21, 0x02, 15},
  {29, 0x02, 15},
  {37, 0x03, 15},
},
/* 206 */
{
  {3, 0x02, 16},
  {4, 0x02, 16},
  {8, 0x02, 16},
  {11, 0x02, 16},
  {16, 0x02, 16},
  {21, 0x02, 16},
  {29, 0x02, 16},
  {37, 0x03, 16},
  {3, 0x02, 17},
  {4, 0x02, 17},
  {8, 0x02, 17},
  {11, 0x02, 17},
  {16, 0x02, 17},
  {21, 0x02, 17},
  {29, 0x02, 17},
  {37, 0x03, 17},
},
/* 207 */
{
  {2, 0x02, 18},
  {7, 0x02, 18},
  {15, 0x02, 18},
  {28, 0x03, 18},
  {2, 0x02, 19},
  {7, 0x02, 19},
  {15, 0x02, 19},
  {28, 0x03, 19},
  {2, 0x02, 20},
  {7, 0x02, 20},
  {15, 0x02, 20},
  {28, 0x03, 20},
  {2, 0x02, 21},
  {7, 0x02, 21},
  {15, 0x02, 21},
  {28, 0x03, 21},
},
/* 208 */
{
  {3, 0x02, 18},
  {4, 0x02, 18},
  {8, 0x02, 18},
  {11, 0x02, 18},
  {16, 0x02, 18},
  {21, 0x02, 18},
  {29, 0x02, 18},
  {37, 0x03, 18},
  {3, 0x02, 19},
  {4, 0x02, 19},
  {8, 0x02, 19},
  {11, 0x02, 19},
  {16, 0x02, 19},
  {21, 0x02, 19},
  {29, 0x02, 19},
  {37, 0x03, 19},
},
/* 209 */
{
  {3, 0x02, 20},
  {4, 0x02, 20},
  {8, 0x02, 20},
  {11, 0x02, 20},
  {16, 0x02, 20},
  {21, 0x02, 20},
  {29, 0x02, 20},
  {37, 0x03, 20},
  {3, 0x02, 21},
  {4, 0x02, 21},
  {8, 0x02, 21},
  {11, 0x02, 21},
  {16, 0x02, 21},
  {21, 0x02, 21},
  {29, 0x02, 21},
  {37, 0x03, 21},
},
/* 210 */
{
  {0, 0x03, 22},
  {0, 0x03, 23},
  {0, 0x03, 24},
  {0, 0x03, 25},
  {0, 0x03, 26},
  {0, 0x03, 27},
  {0, 0x03, 28},
  {0, 0x03, 29},
  {0, 0x03, 30},
  {0, 0x03, 31},
  {0, 0x03, 92},
  {0, 0x03, 127},
  {0, 0x03, 128},
  {0, 0x03, 129},
  {0, 0x03, 130},
  {0, 0x03, 131},
},
/* 211 */
{
  {1, 0x02, 22},
  {14, 0x03, 22},
  {1, 0x02, 23},
  {14, 0x03, 23},
  {1, 0x02, 24},
  {14, 0x03, 24},
  {1, 0x02, 25},
  {14, 0x03, 25},
  {1, 0x02, 26},
  {14, 0x03, 26},
  {1, 0x02, 27},
  {14, 0x03, 27},
  {1, 0x02, 28},
  {14, 0x03, 28},
  {1, 0x02, 29},
  {14, 0x03, 29},
},
/* 212 */
{
  {2, 0x02, 22},
  {7, 0x02, 22},
  {15, 0x02, 22},
  {28, 0x03, 22},
  {2, 0x02, 23},
  {7, 0x02, 23},
  {15, 0x02, 23},
  {28, 0x03, 23},
  {2, 0x02, 24},
  {7, 0x02, 24},
  {15, 0x02, 24},
  {28, 0x03, 24},
  {2, 0x02, 25},
  {7, 0x02, 25},
  {15, 0x02, 25},
  {28, 0x03, 25},
},
/* 213 */
{
  {3, 0x02, 22},
  {4, 0x02, 22},
  {8, 0x02, 22},
  {11, 0x02, 22},
  {16, 0x02, 22},
  {21, 0x02, 22},
  {29, 0x02, 22},
  {37, 0x03, 22},
  {3, 0x02, 23},
  {4, 0x02, 23},
  {8, 0x02, 23},
  {11, 0x02, 23},
  {16, 0x02, 23},
  {21, 0x02, 23},
  {29, 0x02, 23},
  {37, 0x03, 23},
},
/* 214 */
{
  {3, 0x02, 24},
  {4, 0x02, 24},
  {8, 0x02, 24},
  {11, 0x02, 24},
  {16, 0x02, 24},
  {21, 0x02, 24},
  {29, 0x02, 24},
  {37, 0x03, 24},
  {3, 0x02, 25},
  {4, 0x02, 25},
  {8, 0x02, 25},
  {11, 0x02, 25},
  {16, 0x02, 25},
  {21, 0x02, 25},
  {29, 0x02, 25},
  {37, 0x03, 25},
},
/* 215 */
{
  {2, 0x02, 26},
  {7, 0x02, 26},
  {15, 0x02, 26},
  {28, 0x03, 26},
  {2, 0x02, 27},
  {7, 0x02, 27},
  {15, 0x02, 27},
  {28, 0x03, 27},
  {2, 0x02, 28},
  {7, 0x02, 28},
  {15, 0x02, 28},
  {28, 0x03, 28},
  {2, 0x02, 29},
  {7, 0x02, 29},
  {15, 0x02, 29},
  {28, 0x03, 29},
},
/* 216 */
{
  {3, 0x02, 26},
  {4, 0x02, 26},
  {8, 0x02, 26},
  {11, 0x02, 26},
  {16, 0x02, 26},
  {21, 0x02, 26},
  {29, 0x02, 26},
  {37, 0x03, 26},
  {3, 0x02, 27},
  {4, 0x02, 27},
  {8, 0x02, 27},
  {11, 0x02, 27},
  {16, 0x02, 27},
  {21, 0x02, 27},
  {29, 0x02, 27},
  {37, 0x03, 27},
},
/* 217 */
{
  {3, 0x02, 28},
  {4, 0x02, 28},
  {8, 0x02, 28},
  {11, 0x02, 28},
  {16, 0x02, 28},
  {21, 0x02, 28},
  {29, 0x02, 28},
  {37, 0x03, 28},
  {3, 0x02, 29},
  {4, 0x02, 29},
  {8, 0x02, 29},
  {11, 0x02, 29},
  {16, 0x02, 29},
  {21, 0x02, 29},
  {29, 0x02, 29},
  {37, 0x03, 29},
},
/* 218 */
{
  {1, 0x02, 30},
  {14, 0x03, 30},
  {1, 0x02, 31},
  {14, 0x03, 31},
  {1, 0x02, 92},
  {14, 0x03, 92},
  {1, 0x02, 127},
  {14, 0x03, 127},
  {1, 0x02, 128},
  {14, 0x03, 128},
  {1, 0x02, 129},
  {14, 0x03, 129},
  {1, 0x02, 130},
  {14, 0x03, 130},
  {1, 0x02, 131},
  {14, 0x03, 131},
},
/* 219 */
{
  {2, 0x02, 30},
  {7, 0x02, 30},
  {15, 0x02, 30},
  {28, 0x03, 30},
  {2, 0x02, 31},
  {7, 0x02, 31},
  {15, 0x02, 31},
  {28, 0x03, 31},
  {2, 0x02, 92},
  {7, 0x02, 92},
  {15, 0x02, 92},
  {28, 0x03, 92},
  {2, 0x02, 127},
  {7, 0x02, 127},
  {15, 0x02, 127},
  {28, 0x03, 127},
},
/* 220 */
{
  {3, 0x02, 30},
  {4, 0x02, 30},
  {8, 0x02, 30},
  {11, 0x02, 30},
  {16, 0x02, 30},
  {21, 0x02, 30},
  {29, 0x02, 30},
  {37, 0x03, 30},
  {3, 0x02, 31},
  {4, 0x02, 31},
  {8, 0x02, 31},
  {11, 0x02, 31},
  {16, 0x02, 31},
  {21, 0x02, 31},
  {29, 0x02, 31},
  {37, 0x03, 31},
},
/* 221 */
{
  {3, 0x02, 92},
  {4, 0x02, 92},
  {8, 0x02, 92},
  {11, 0x02, 92},
  {16, 0x02, 92},
  {21, 0x02, 92},
  {29, 0x02, 92},
  {37, 0x03, 92},
  {3, 0x02, 127},
  {4, 0x02, 127},
  {8, 0x02, 127},
  {11, 0x02, 127},
  {16, 0x02, 127},
  {21, 0x02, 127},
  {29, 0x02, 127},
  {37, 0x03, 127},
},
/* 222 */
{
  {2, 0x02, 128},
  {7, 0x02, 128},
  {15, 0x02, 128},
  {28, 0x03, 128},
  {2, 0x02, 129},
  {7, 0x02, 129},
  {15, 0x02, 129},
  {28, 0x03, 129},
  {2, 0x02, 130},
  {7, 0x02, 130},
  {15, 0x02, 130},
  {28, 0x03, 130},
  {2, 0x02, 131},
  {7, 0x02, 131},
  {15, 0x02, 131},
  {28, 0x03, 131},
},
/* 223 */
{
  {3, 0x02, 128},
  {4, 0x02, 128},
  {8, 0x02, 128},
  {11, 0x02, 128},
  {16, 0x02, 128},
  {21, 0x02, 128},
  {29, 0x02, 128},
  {37, 0x03, 128},
  {3, 0x02, 129},
  {4, 0x02, 129},
  {8, 0x02, 129},
  {11, 0x02, 129},
  {16, 0x02, 129},
  {21, 0x02, 129},
  {29, 0x02, 129},
  {37, 0x03, 129},
},
/* 224 */
{
  {3, 0x02, 130},
  {4, 0x02, 130},
  {8, 0x02, 130},
  {11, 0x02, 130},
  {16, 0x02, 130},
  {21, 0x02, 130},
  {29, 0x02, 130},
  {37, 0x03, 130},
  {3, 0x02, 131},
  {4, 0x02, 131},
  {8, 0x02, 131},
  {11, 0x02, 131},
  {16, 0x02, 131},
  {21, 0x02, 131},
  {29, 0x02, 131},
  {37, 0x03, 131},
},
/* 225 */
{
  {229, 0x00, 0},
  {230, 0x00, 0},
  {232, 0x00, 0},
  {233, 0x00, 0},
  {236, 0x00, 0},
  {237, 0x00, 0},
  {239, 0x00, 0},
  {240, 0x00, 0},
  {244, 0x00, 0},
  {245, 0x00, 0},
  {247, 0x00, 0},
  {248, 0x00, 0},
  {251, 0x00, 0},
  {252, 0x00, 0},
  {254, 0x00, 0},
  {255, 0x00, 0},
},
/* 226 */
{
  {0, 0x03, 132},
  {0, 0x03, 133},
  {0, 0x03, 134},
  {0, 0x03, 135},
  {0, 0x03, 136},
  {0, 0x03, 137},
  {0, 0x03, 138},
  {0, 0x03, 139},
  {0, 0x03, 140},
  {0, 0x03, 141},
  {0, 0x03, 142},
  {0, 0x03, 143},
  {0, 0x03, 144},
  {0, 0x03, 145},
  {0, 0x03, 146},
  {0, 0x03, 147},
},
/* 227 */
{
  {1, 0x02, 132},
  {14, 0x03, 132},
  {1, 0x02, 133},
  {14, 0x03, 133},
  {1, 0x02, 134},
  {14, 0x03, 134},
  {1, 0x02, 135},
  {14, 0x03, 135},
  {1, 0x02, 136},
  {14, 0x03, 136},
  {1, 0x02, 137},
  {14, 0x03, 137},
  {1, 0x02, 138},
  {14, 0x03, 138},
  {1, 0x02, 139},
  {14, 0x03, 139},
},
/* 228 */
{
  {2, 0x02, 132},
  {7, 0x02, 132},
  {15, 0x02, 132},
  {28, 0x03, 132},
  {2, 0x02, 133},
  {7, 0x02, 133},
  {15, 0x02, 133},
  {28, 0x03, 133},
  {2, 0x02, 134},
  {7, 0x02, 134},
  {15, 0x02, 134},
  {28, 0x03, 134},
  {2, 0x02, 135},
  {7, 0x02, 135},
  {15, 0x02, 135},
  {28, 0x03, 135},
},
/* 229 */
{
  {3, 0x02, 132},
  {4, 0x02, 132},
  {8, 0x02, 132},
  {11, 0x02, 132},
  {16, 0x02, 132},
  {21, 0x02, 132},
  {29, 0x02, 132},
  {37, 0x03, 132},
  {3, 0x02, 133},
  {4, 0x02, 133},
  {8, 0x02, 133},
  {11, 0x02, 133},
  {16, 0x02, 133},
  {21, 0x02, 133},
  {29, 0x02, 133},
  {37, 0x03, 133},
},
/* 230 */
{
  {3, 0x02, 134},
  {4, 0x02, 134},
  {8, 0x02, 134},
  {11, 0x02, 134},
  {16, 0x02, 134},
  {21, 0x02, 134},
  {29, 0x02, 134},
  {37, 0x03, 134},
  {3, 0x02, 135},
  {4, 0x02, 135},
  {8, 0x02, 135},
  {11, 0x02, 135},
  {16, 0x02, 135},
  {21, 0x02, 135},
  {29, 0x02, 135},
  {37, 0x03, 135},
},
/* 231 */
{
  {2, 0x02, 136},
  {7, 0x02, 136},
  {15, 0x02, 136},
  {28, 0x03, 136},
  {2, 0x02, 137},
  {7, 0x02, 137},
  {15, 0x02, 137},
  {28, 0x03, 137},
  {2, 0x02, 138},
  {7, 0x02, 138},
  {15, 0x02, 138},
  {28, 0x03, 138},
  {2, 0x02, 139},
  {7, 0x02, 139},
  {15, 0x02, 139},
  {28, 0x03, 139},
},
/* 232 */
{
  {3, 0x02, 136},
  {4, 0x02, 136},
  {8, 0x02, 136},
  {11, 0x02, 136},
  {16, 0x02, 136},
  {21, 0x02, 136},
  {29, 0x02, 136},
  {37, 0x03, 136},
  {3, 0x02, 137},
  {4, 0x02, 137},
  {8, 0x02, 137},
  {11, 0x02, 137},
  {16, 0x02, 137},
  {21, 0x02, 137},
  {29, 0x02, 137},
  {37, 0x03, 137},
},
/* 233 */
{
  {3, 0x02, 138},
  {4, 0x02, 138},
  {8, 0x02, 138},
  {11, 0x02, 138},
  {16, 0x02, 138},
  {21, 0x02, 138},
  {29, 0x02, 138},
  {37, 0x03, 138},
  {3, 0x02, 139},
  {4, 0x02, 139},
  {8, 0x02, 139},
  {11, 0x02, 139},
  {16, 0x02, 139},
  {21, 0x02, 139},
  {29, 0x02, 139},
  {37, 0x03, 139},
},
/* 234 */
{
  {1, 0x02, 140},
  {14, 0x03, 140},
  {1, 0x02, 141},
  {14, 0x03, 141},
  {1, 0x02, 142},
  {14, 0x03, 142},
  {1, 0x02, 143},
  {14, 0x03, 143},
  {1, 0x02, 144},
  {14, 0x03, 144},
  {1, 0x02, 145},
  {14, 0x03, 145},
  {1, 0x02, 146},
  {14, 0x03, 146},
  {1, 0x02, 147},
  {14, 0x03, 147},
},
/* 235 */
{
  {2, 0x02, 140},
  {7, 0x02, 140},
  {15, 0x02, 140},
  {28, 0x03, 140},
  {2, 0x02, 141},
  {7, 0x02, 141},
  {15, 0x02, 141},
  {28, 0x03, 141},
  {2, 0x02, 142},
  {7, 0x02, 142},
  {15, 0x02, 142},
  {28, 0x03, 142},
  {2, 0x02, 143},
  {7, 0x02, 143},
  {15, 0x02, 143},
  {28, 0x03, 143},
},
/* 236 */
{
  {3, 0x02, 140},
  {4, 0x02, 140},
  {8, 0x02, 140},
  {11, 0x02, 140},
  {16, 0x02, 140},
  {21, 0x02, 140},
  {29, 0x02, 140},
  {37, 0x03, 140},
  {3, 0x02, 141},
  {4, 0x02, 141},
  {8, 0x02, 141},
  {11, 0x02, 141},
  {16, 0x02, 141},
  {21, 0x02, 141},
  {29, 0x02, 141},
  {37, 0x03, 141},
},
/* 237 */
{
  {3, 0x02, 142},
  {4, 0x02, 142},
  {8, 0x02, 142},
  {11, 0x02, 142},
  {16, 0x02, 142},
  {21, 0x02, 142},
  {29, 0x02, 142},
  {37, 0x03, 142},
  {3, 0x02, 143},
  {4, 0x02, 143},
  {8, 0x02, 143},
  {11, 0x02, 143},
  {16, 0x02, 143},
  {21, 0x02, 143},
  {29, 0x02, 143},
  {37, 0x03, 143},
},
/* 238 */
{
  {2, 0x02, 144},
  {7, 0x02, 144},
  {15, 0x02, 144},
  {28, 0x03, 144},
  {2, 0x02, 145},
  {7, 0x02, 145},
  {15, 0x02, 145},
  {28, 0x03, 145},
  {2, 0x02, 146},
  {7, 0x02, 146},
  {15, 0x02, 146},
  {28, 0x03, 146},
  {2, 0x02, 147},
  {7, 0x02, 147},
  {15, 0x02, 147},
  {28, 0x03, 147},
},
/* 239 */
{
  {3, 0x02, 144},
  {4, 0x02, 144},
  {8, 0x02, 144},
  {11, 0x02, 144},
  {16, 0x02, 144},
  {21, 0x02, 144},
  {29, 0x02, 144},
  {37, 0x03, 144},
  {3, 0x02, 145},
  {4, 0x02, 145},
  {8, 0x02, 145},
  {11, 0x02, 145},
  {16, 0x02, 145},
  {21, 0x02, 145},
  {29, 0x02, 145},
  {37, 0x03, 145},
},
/* 240 */
{
  {3, 0x02, 146},
  {4, 0x02, 146},
  {8, 0x02, 146},
  {11, 0x02, 146},
  {16, 0x02, 146},
  {21, 0x02, 146},
  {29, 0x02, 146},
  {37, 0x03, 146},
  {3, 0x02, 147},
  {4, 0x02, 147},
  {8, 0x02, 147},
  {11, 0x02, 147},
  {16, 0x02, 147},
  {21, 0x02, 147},
  {29, 0x02, 147},
  {37, 0x03, 147},
},
/* 241 */
{
  {0, 0x03, 148},
  {0, 0x03, 149},
  {0, 0x03, 150},
  {0, 0x03, 151},
  {0, 0x03, 152},
  {0, 0x03, 153},
  {0, 0x03, 154},
  {0, 0x03, 155},
  {0, 0x03, 156},
  {0, 0x03, 157},
  {0, 0x03, 158},
  {0, 0x03, 159},
  {0, 0x03, 160},
  {0, 0x03, 161},
  {0, 0x03, 162},
  {0, 0x03, 163},
},
/* 242 */
{
  {1, 0x02, 148},
  {14, 0x03, 148},
  {1, 0x02, 149},
  {14, 0x03, 149},
  {1, 0x02, 150},
  {14, 0x03, 150},
  {1, 0x02, 151},
  {14, 0x03, 151},
  {1, 0x02, 152},
  {14, 0x03, 152},
  {1, 0x02, 153},
  {14, 0x03, 153},
  {1, 0x02, 154},
  {14, 0x03, 154},
  {1, 0x02, 155},
  {14, 0x03, 155},
},
/* 243 */
{
  {2, 0x02, 148},
  {7, 0x02, 148},
  {15, 0x02, 148},
  {28, 0x03, 148},
  {2, 0x02, 149},
  {7, 0x02, 149},
  {15, 0x02, 149},
  {28, 0x03, 149},
  {2, 0x02, 150},
  {7, 0x02, 150},
  {15, 0x02, 150},
  {28, 0x03, 150},
  {2, 0x02, 151},
  {7, 0x02, 151},
  {15, 0x02, 151},
  {28, 0x03, 151},
},
/* 244 */
{
  {3, 0x02, 148},
  {4, 0x02, 148},
  {8, 0x02, 148},
  {11, 0x02, 148},
  {16, 0x02, 148},
  {21, 0x02, 148},
  {29, 0x02, 148},
  {37, 0x03, 148},
  {3, 0x02, 149},
  {4, 0x02, 149},
  {8, 0x02, 149},
  {11, 0x02, 149},
  {16, 0x02, 149},
  {21, 0x02, 149},
  {29, 0x02, 149},
  {37, 0x03, 149},
},
/* 245 */
{
  {3, 0x02, 150},
  {4, 0x02, 150},
  {8, 0x02, 150},
  {11, 0x02, 150},
  {16, 0x02, 150},
  {21, 0x02, 150},
  {29, 0x02, 150},
  {37, 0x03, 150},
  {3, 0x02, 151},
  {4, 0x02, 151},
  {8, 0x02, 151},
  {11, 0x02, 151},
  {16, 0x02, 151},
  {21, 0x02, 151},
  {29, 0x02, 151},
  {37, 0x03, 151},
},
/* 246 */
{
  {2, 0x02, 152},
  {7, 0x02, 152},
  {15, 0x02, 152},
  {28, 0x03, 152},
  {2, 0x02, 153},
  {7, 0x02, 153},
  {15, 0x02, 153},
  {28, 0x03, 153},
  {2, 0x02, 154},
  {7, 0x02, 154},
  {15, 0x02, 154},
  {28, 0x03, 154},
  {2, 0x02, 155},
  {7, 0x02, 155},
  {15, 0x02, 155},
  {28, 0x03, 155},
},
/* 247 */
{
  {3, 0x02, 152},
  {4, 0x02, 152},
  {8, 0x02, 152},
  {11, 0x02, 152},
  {16, 0x02, 152},
  {21, 0x02, 152},
  {29, 0x02, 152},
  {37, 0x03, 152},
  {3, 0x02, 153},
  {4, 0x02, 153},
  {8, 0x02, 153},
  {11, 0x02, 153},
  {16, 0x02, 153},
  {21, 0x02, 153},
  {29, 0x02, 153},
  {37, 0x03, 153},
},
/* 248 */
{
  {3, 0x02, 154},
  {4, 0x02, 154},
  {8, 0x02, 154},
  {11, 0x02, 154},
  {16, 0x02, 154},
  {21, 0x02, 154},
  {29, 0x02, 154},
  {37, 0x03, 154},
  {3, 0x02, 155},
  {4, 0x02, 155},
  {8, 0x02, 155},
  {11, 0x02, 155},
  {16, 0x02, 155},
  {21, 0x02, 155},
  {29, 0x02, 155},
  {37, 0x03, 155},
},
/* 249 */
{
  {1, 0x02, 156},
  {14, 0x03, 156},
  {1, 0x02, 157},
  {14, 0x03, 157},
  {1, 0x02, 158},
  {14, 0x03, 158},
  {1, 0x02, 159},
  {14, 0x03, 159},
  {1, 0x02, 160},
  {14, 0x03, 160},
  {1, 0x02, 161},
  {14, 0x03, 161},
  {1, 0x02, 162},
  {14, 0x03, 162},
  {1, 0x02, 163},
  {14, 0x03, 163},
},
/* 250 */
{
  {2, 0x02, 156},
  {7, 0x02, 156},
  {15, 0x02, 156},
  {28, 0x03, 156},
  {2, 0x02, 157},
  {7, 0x02, 157},
  {15, 0x02, 157},
  {28, 0x03, 157},
  {2, 0x02, 158},
  {7, 0x02, 158},
  {15, 0x02, 158},
  {28, 0x03, 158},
  {2, 0x02, 159},
  {7, 0x02, 159},
  {15, 0x02, 159},
  {28, 0x03, 159},
},
/* 251 */
{
  {3, 0x02, 156},
  {4, 0x02, 156},
  {8, 0x02, 156},
  {11, 0x02, 156},
  {16, 0x02, 156},
  {21, 0x02, 156},
  {29, 0x02, 156},
  {37, 0x03, 156},
  {3, 0x02, 157},
  {4, 0x02, 157},
  {8, 0x02, 157},
  {11, 0x02, 157},
  {16, 0x02, 157},
  {21, 0x02, 157},
  {29, 0x02, 157},
  {37, 0x03, 157},
},
/* 252 */
{
  {3, 0x02, 158},
  {4, 0x02, 158},
  {8, 0x02, 158},
  {11, 0x02, 158},
  {16, 0x02, 158},
  {21, 0x02, 158},
  {29, 0x02, 158},
  {37, 0x03, 158},
  {3, 0x02, 159},
  {4, 0x02, 159},
  {8, 0x02, 159},
  {11, 0x02, 159},
  {16, 0x02, 159},
  {21, 0x02, 159},
  {29, 0x02, 159},
  {37, 0x03, 159},
},
/* 253 */
{
  {2, 0x02, 160},
  {7, 0x02, 160},
  {15, 0x02, 160},
  {28, 0x03, 160},
  {2, 0x02, 161},
  {7, 0x02, 161},
  {15, 0x02, 161},
  {28, 0x03, 161},
  {2, 0x02, 162},
  {7, 0x02, 162},
  {15, 0x02, 162},
  {28, 0x03, 162},
  {2, 0x02, 163},
  {7, 0x02, 163},
  {15, 0x02, 163},
  {28, 0x03, 163},
},
/* 254 */
{
  {3, 0x02, 160},
  {4, 0x02, 160},
  {8, 0x02, 160},
  {11, 0x02, 160},
  {16, 0x02, 160},
  {21, 0x02, 160},
  {29, 0x02, 160},
  {37, 0x03, 160},
  {3, 0x02, 161},
  {4, 0x02, 161},
  {8, 0x02, 161},
  {11, 0x02, 161},
  {16, 0x02, 161},
  {21, 0x02, 161},
  {29, 0x02, 161},
  {37, 0x03, 161},
},
/* 255 */
{
  {3, 0x02, 162},
  {4, 0x02, 162},
  {8, 0x02, 162},
  {11, 0x02, 162},
  {16, 0x02, 162},
  {21, 0x02, 162},
  {29, 0x02, 162},
  {37, 0x03, 162},
  {3, 0x02, 163},
  {4, 0x02, 163},
  {8, 0x02, 163},
  {11, 0x02, 163},
  {16, 0x02, 163},
  {21, 0x02, 163},
  {29, 0x02, 163},
  {37, 0x03, 163},
},
};

const nghttp2_huff_decode res_huff_decode_table[][16] = {
/* 0 */
{
  {0, 0x03, 32},
  {0, 0x03, 48},
  {0, 0x03, 49},
  {0, 0x03, 50},
  {7, 0x00, 0},
  {8, 0x00, 0},
  {10, 0x00, 0},
  {11, 0x00, 0},
  {15, 0x00, 0},
  {17, 0x00, 0},
  {21, 0x00, 0},
  {24, 0x00, 0},
  {29, 0x00, 0},
  {32, 0x00, 0},
  {39, 0x00, 0},
  {49, 0x01, 0},
},
/* 1 */
{
  {1, 0x02, 32},
  {12, 0x03, 32},
  {1, 0x02, 48},
  {12, 0x03, 48},
  {1, 0x02, 49},
  {12, 0x03, 49},
  {1, 0x02, 50},
  {12, 0x03, 50},
  {0, 0x03, 51},
  {0, 0x03, 52},
  {0, 0x03, 53},
  {0, 0x03, 56},
  {0, 0x03, 57},
  {0, 0x03, 58},
  {0, 0x03, 84},
  {0, 0x03, 97},
},
/* 2 */
{
  {2, 0x02, 32},
  {5, 0x02, 32},
  {13, 0x02, 32},
  {27, 0x03, 32},
  {2, 0x02, 48},
  {5, 0x02, 48},
  {13, 0x02, 48},
  {27, 0x03, 48},
  {2, 0x02, 49},
  {5, 0x02, 49},
  {13, 0x02, 49},
  {27, 0x03, 49},
  {2, 0x02, 50},
  {5, 0x02, 50},
  {13, 0x02, 50},
  {27, 0x03, 50},
},
/* 3 */
{
  {3, 0x02, 32},
  {4, 0x02, 32},
  {6, 0x02, 32},
  {9, 0x02, 32},
  {14, 0x02, 32},
  {20, 0x02, 32},
  {28, 0x02, 32},
  {38, 0x03, 32},
  {3, 0x02, 48},
  {4, 0x02, 48},
  {6, 0x02, 48},
  {9, 0x02, 48},
  {14, 0x02, 48},
  {20, 0x02, 48},
  {28, 0x02, 48},
  {38, 0x03, 48},
},
/* 4 */
{
  {3, 0x02, 49},
  {4, 0x02, 49},
  {6, 0x02, 49},
  {9, 0x02, 49},
  {14, 0x02, 49},
  {20, 0x02, 49},
  {28, 0x02, 49},
  {38, 0x03, 49},
  {3, 0x02, 50},
  {4, 0x02, 50},
  {6, 0x02, 50},
  {9, 0x02, 50},
  {14, 0x02, 50},
  {20, 0x02, 50},
  {28, 0x02, 50},
  {38, 0x03, 50},
},
/* 5 */
{
  {1, 0x02, 51},
  {12, 0x03, 51},
  {1, 0x02, 52},
  {12, 0x03, 52},
  {1, 0x02, 53},
  {12, 0x03, 53},
  {1, 0x02, 56},
  {12, 0x03, 56},
  {1, 0x02, 57},
  {12, 0x03, 57},
  {1, 0x02, 58},
  {12, 0x03, 58},
  {1, 0x02, 84},
  {12, 0x03, 84},
  {1, 0x02, 97},
  {12, 0x03, 97},
},
/* 6 */
{
  {2, 0x02, 51},
  {5, 0x02, 51},
  {13, 0x02, 51},
  {27, 0x03, 51},
  {2, 0x02, 52},
  {5, 0x02, 52},
  {13, 0x02, 52},
  {27, 0x03, 52},
  {2, 0x02, 53},
  {5, 0x02, 53},
  {13, 0x02, 53},
  {27, 0x03, 53},
  {2, 0x02, 56},
  {5, 0x02, 56},
  {13, 0x02, 56},
  {27, 0x03, 56},
},
/* 7 */
{
  {3, 0x02, 51},
  {4, 0x02, 51},
  {6, 0x02, 51},
  {9, 0x02, 51},
  {14, 0x02, 51},
  {20, 0x02, 51},
  {28, 0x02, 51},
  {38, 0x03, 51},
  {3, 0x02, 52},
  {4, 0x02, 52},
  {6, 0x02, 52},
  {9, 0x02, 52},
  {14, 0x02, 52},
  {20, 0x02, 52},
  {28, 0x02, 52},
  {38, 0x03, 52},
},
/* 8 */
{
  {3, 0x02, 53},
  {4, 0x02, 53},
  {6, 0x02, 53},
  {9, 0x02, 53},
  {14, 0x02, 53},
  {20, 0x02, 53},
  {28, 0x02, 53},
  {38, 0x03, 53},
  {3, 0x02, 56},
  {4, 0x02, 56},
  {6, 0x02, 56},
  {9, 0x02, 56},
  {14, 0x02, 56},
  {20, 0x02, 56},
  {28, 0x02, 56},
  {38, 0x03, 56},
},
/* 9 */
{
  {2, 0x02, 57},
  {5, 0x02, 57},
  {13, 0x02, 57},
  {27, 0x03, 57},
  {2, 0x02, 58},
  {5, 0x02, 58},
  {13, 0x02, 58},
  {27, 0x03, 58},
  {2, 0x02, 84},
  {5, 0x02, 84},
  {13, 0x02, 84},
  {27, 0x03, 84},
  {2, 0x02, 97},
  {5, 0x02, 97},
  {13, 0x02, 97},
  {27, 0x03, 97},
},
/* 10 */
{
  {3, 0x02, 57},
  {4, 0x02, 57},
  {6, 0x02, 57},
  {9, 0x02, 57},
  {14, 0x02, 57},
  {20, 0x02, 57},
  {28, 0x02, 57},
  {38, 0x03, 57},
  {3, 0x02, 58},
  {4, 0x02, 58},
  {6, 0x02, 58},
  {9, 0x02, 58},
  {14, 0x02, 58},
  {20, 0x02, 58},
  {28, 0x02, 58},
  {38, 0x03, 58},
},
/* 11 */
{
  {3, 0x02, 84},
  {4, 0x02, 84},
  {6, 0x02, 84},
  {9, 0x02, 84},
  {14, 0x02, 84},
  {20, 0x02, 84},
  {28, 0x02, 84},
  {38, 0x03, 84},
  {3, 0x02, 97},
  {4, 0x02, 97},
  {6, 0x02, 97},
  {9, 0x02, 97},
  {14, 0x02, 97},
  {20, 0x02, 97},
  {28, 0x02, 97},
  {38, 0x03, 97},
},
/* 12 */
{
  {0, 0x03, 101},
  {16, 0x00, 0},
  {18, 0x00, 0},
  {19, 0x00, 0},
  {22, 0x00, 0},
  {23, 0x00, 0},
  {25, 0x00, 0},
  {26, 0x00, 0},
  {30, 0x00, 0},
  {31, 0x00, 0},
  {33, 0x00, 0},
  {35, 0x00, 0},
  {40, 0x00, 0},
  {43, 0x00, 0},
  {50, 0x00, 0},
  {59, 0x01, 0},
},
/* 13 */
{
  {1, 0x02, 101},
  {12, 0x03, 101},
  {0, 0x03, 44},
  {0, 0x03, 45},
  {0, 0x03, 46},
  {0, 0x03, 54},
  {0, 0x03, 55},
  {0, 0x03, 71},
  {0, 0x03, 77},
  {0, 0x03, 83},
  {0, 0x03, 99},
  {0, 0x03, 100},
  {0, 0x03, 105},
  {0, 0x03, 109},
  {0, 0x03, 110},
  {0, 0x03, 111},
},
/* 14 */
{
  {2, 0x02, 101},
  {5, 0x02, 101},
  {13, 0x02, 101},
  {27, 0x03, 101},
  {1, 0x02, 44},
  {12, 0x03, 44},
  {1, 0x02, 45},
  {12, 0x03, 45},
  {1, 0x02, 46},
  {12, 0x03, 46},
  {1, 0x02, 54},
  {12, 0x03, 54},
  {1, 0x02, 55},
  {12, 0x03, 55},
  {1, 0x02, 71},
  {12, 0x03, 71},
},
/* 15 */
{
  {3, 0x02, 101},
  {4, 0x02, 101},
  {6, 0x02, 101},
  {9, 0x02, 101},
  {14, 0x02, 101},
  {20, 0x02, 101},
  {28, 0x02, 101},
  {38, 0x03, 101},
  {2, 0x02, 44},
  {5, 0x02, 44},
  {13, 0x02, 44},
  {27, 0x03, 44},
  {2, 0x02, 45},
  {5, 0x02, 45},
  {13, 0x02, 45},
  {27, 0x03, 45},
},
/* 16 */
{
  {3, 0x02, 44},
  {4, 0x02, 44},
  {6, 0x02, 44},
  {9, 0x02, 44},
  {14, 0x02, 44},
  {20, 0x02, 44},
  {28, 0x02, 44},
  {38, 0x03, 44},
  {3, 0x02, 45},
  {4, 0x02, 45},
  {6, 0x02, 45},
  {9, 0x02, 45},
  {14, 0x02, 45},
  {20, 0x02, 45},
  {28, 0x02, 45},
  {38, 0x03, 45},
},
/* 17 */
{
  {2, 0x02, 46},
  {5, 0x02, 46},
  {13, 0x02, 46},
  {27, 0x03, 46},
  {2, 0x02, 54},
  {5, 0x02, 54},
  {13, 0x02, 54},
  {27, 0x03, 54},
  {2, 0x02, 55},
  {5, 0x02, 55},
  {13, 0x02, 55},
  {27, 0x03, 55},
  {2, 0x02, 71},
  {5, 0x02, 71},
  {13, 0x02, 71},
  {27, 0x03, 71},
},
/* 18 */
{
  {3, 0x02, 46},
  {4, 0x02, 46},
  {6, 0x02, 46},
  {9, 0x02, 46},
  {14, 0x02, 46},
  {20, 0x02, 46},
  {28, 0x02, 46},
  {38, 0x03, 46},
  {3, 0x02, 54},
  {4, 0x02, 54},
  {6, 0x02, 54},
  {9, 0x02, 54},
  {14, 0x02, 54},
  {20, 0x02, 54},
  {28, 0x02, 54},
  {38, 0x03, 54},
},
/* 19 */
{
  {3, 0x02, 55},
  {4, 0x02, 55},
  {6, 0x02, 55},
  {9, 0x02, 55},
  {14, 0x02, 55},
  {20, 0x02, 55},
  {28, 0x02, 55},
  {38, 0x03, 55},
  {3, 0x02, 71},
  {4, 0x02, 71},
  {6, 0x02, 71},
  {9, 0x02, 71},
  {14, 0x02, 71},
  {20, 0x02, 71},
  {28, 0x02, 71},
  {38, 0x03, 71},
},
/* 20 */
{
  {1, 0x02, 77},
  {12, 0x03, 77},
  {1, 0x02, 83},
  {12, 0x03, 83},
  {1, 0x02, 99},
  {12, 0x03, 99},
  {1, 0x02, 100},
  {12, 0x03, 100},
  {1, 0x02, 105},
  {12, 0x03, 105},
  {1, 0x02, 109},
  {12, 0x03, 109},
  {1, 0x02, 110},
  {12, 0x03, 110},
  {1, 0x02, 111},
  {12, 0x03, 111},
},
/* 21 */
{
  {2, 0x02, 77},
  {5, 0x02, 77},
  {13, 0x02, 77},
  {27, 0x03, 77},
  {2, 0x02, 83},
  {5, 0x02, 83},
  {13, 0x02, 83},
  {27, 0x03, 83},
  {2, 0x02, 99},
  {5, 0x02, 99},
  {13, 0x02, 99},
  {27, 0x03, 99},
  {2, 0x02, 100},
  {5, 0x02, 100},
  {13, 0x02, 100},
  {27, 0x03, 100},
},
/* 22 */
{
  {3, 0x02, 77},
  {4, 0x02, 77},
  {6, 0x02, 77},
  {9, 0x02, 77},
  {14, 0x02, 77},
  {20, 0x02, 77},
  {28, 0x02, 77},
  {38, 0x03, 77},
  {3, 0x02, 83},
  {4, 0x02, 83},
  {6, 0x02, 83},
  {9, 0x02, 83},
  {14, 0x02, 83},
  {20, 0x02, 83},
  {28, 0x02, 83},
  {38, 0x03, 83},
},
/* 23 */
{
  {3, 0x02, 99},
  {4, 0x02, 99},
  {6, 0x02, 99},
  {9, 0x02, 99},
  {14, 0x02, 99},
  {20, 0x02, 99},
  {28, 0x02, 99},
  {38, 0x03, 99},
  {3, 0x02, 100},
  {4, 0x02, 100},
  {6, 0x02, 100},
  {9, 0x02, 100},
  {14, 0x02, 100},
  {20, 0x02, 100},
  {28, 0x02, 100},
  {38, 0x03, 100},
},
/* 24 */
{
  {2, 0x02, 105},
  {5, 0x02, 105},
  {13, 0x02, 105},
  {27, 0x03, 105},
  {2, 0x02, 109},
  {5, 0x02, 109},
  {13, 0x02, 109},
  {27, 0x03, 109},
  {2, 0x02, 110},
  {5, 0x02, 110},
  {13, 0x02, 110},
  {27, 0x03, 110},
  {2, 0x02, 111},
  {5, 0x02, 111},
  {13, 0x02, 111},
  {27, 0x03, 111},
},
/* 25 */
{
  {3, 0x02, 105},
  {4, 0x02, 105},
  {6, 0x02, 105},
  {9, 0x02, 105},
  {14, 0x02, 105},
  {20, 0x02, 105},
  {28, 0x02, 105},
  {38, 0x03, 105},
  {3, 0x02, 109},
  {4, 0x02, 109},
  {6, 0x02, 109},
  {9, 0x02, 109},
  {14, 0x02, 109},
  {20, 0x02, 109},
  {28, 0x02, 109},
  {38, 0x03, 109},
},
/* 26 */
{
  {3, 0x02, 110},
  {4, 0x02, 110},
  {6, 0x02, 110},
  {9, 0x02, 110},
  {14, 0x02, 110},
  {20, 0x02, 110},
  {28, 0x02, 110},
  {38, 0x03, 110},
  {3, 0x02, 111},
  {4, 0x02, 111},
  {6, 0x02, 111},
  {9, 0x02, 111},
  {14, 0x02, 111},
  {20, 0x02, 111},
  {28, 0x02, 111},
  {38, 0x03, 111},
},
/* 27 */
{
  {0, 0x03, 112},
  {0, 0x03, 114},
  {0, 0x03, 115},
  {0, 0x03, 116},
  {0, 0x03, 117},
  {34, 0x00, 0},
  {36, 0x00, 0},
  {37, 0x00, 0},
  {41, 0x00, 0},
  {42, 0x00, 0},
  {44, 0x00, 0},
  {46, 0x00, 0},
  {51, 0x00, 0},
  {54, 0x00, 0},
  {60, 0x00, 0},
  {67, 0x01, 0},
},
/* 28 */
{
  {1, 0x02, 112},
  {12, 0x03, 112},
  {1, 0x02, 114},
  {12, 0x03, 114},
  {1, 0x02, 115},
  {12, 0x03, 115},
  {1, 0x02, 116},
  {12, 0x03, 116},
  {1, 0x02, 117},
  {12, 0x03, 117},
  {0, 0x03, 34},
  {0, 0x03, 47},
  {0, 0x03, 61},
  {0, 0x03, 65},
  {0, 0x03, 98},
  {0, 0x03, 102},
},
/* 29 */
{
  {2, 0x02, 112},
  {5, 0x02, 112},
  {13, 0x02, 112},
  {27, 0x03, 112},
  {2, 0x02, 114},
  {5, 0x02, 114},
  {13, 0x02, 114},
  {27, 0x03, 114},
  {2, 0x02, 115},
  {5, 0x02, 115},
  {13, 0x02, 115},
  {27, 0x03, 115},
  {2, 0x02, 116},
  {5, 0x02, 116},
  {13, 0x02, 116},
  {27, 0x03, 116},
},
/* 30 */
{
  {3, 0x02, 112},
  {4, 0x02, 112},
  {6, 0x02, 112},
  {9, 0x02, 112},
  {14, 0x02, 112},
  {20, 0x02, 112},
  {28, 0x02, 112},
  {38, 0x03, 112},
  {3, 0x02, 114},
  {4, 0x02, 114},
  {6, 0x02, 114},
  {9, 0x02, 114},
  {14, 0x02, 114},
  {20, 0x02, 114},
  {28, 0x02, 114},
  {38, 0x03, 114},
},
/* 31 */
{
  {3, 0x02, 115},
  {4, 0x02, 115},
  {6, 0x02, 115},
  {9, 0x02, 115},
  {14, 0x02, 115},
  {20, 0x02, 115},
  {28, 0x02, 115},
  {38, 0x03, 115},
  {3, 0x02, 116},
  {4, 0x02, 116},
  {6, 0x02, 116},
  {9, 0x02, 116},
  {14, 0x02, 116},
  {20, 0x02, 116},
  {28, 0x02, 116},
  {38, 0x03, 116},
},
/* 32 */
{
  {2, 0x02, 117},
  {5, 0x02, 117},
  {13, 0x02, 117},
  {27, 0x03, 117},
  {1, 0x02, 34},
  {12, 0x03, 34},
  {1, 0x02, 47},
  {12, 0x03, 47},
  {1, 0x02, 61},
  {12, 0x03, 61},
  {1, 0x02, 65},
  {12, 0x03, 65},
  {1, 0x02, 98},
  {12, 0x03, 98},
  {1, 0x02, 102},
  {12, 0x03, 102},
},
/* 33 */
{
  {3, 0x02, 117},
  {4, 0x02, 117},
  {6, 0x02, 117},
  {9, 0x02, 117},
  {14, 0x02, 117},
  {20, 0x02, 117},
  {28, 0x02, 117},
  {38, 0x03, 117},
  {2, 0x02, 34},
  {5, 0x02, 34},
  {13, 0x02, 34},
  {27, 0x03, 34},
  {2, 0x02, 47},
  {5, 0x02, 47},
  {13, 0x02, 47},
  {27, 0x03, 47},
},
/* 34 */
{
  {3, 0x02, 34},
  {4, 0x02, 34},
  {6, 0x02, 34},
  {9, 0x02, 34},
  {14, 0x02, 34},
  {20, 0x02, 34},
  {28, 0x02, 34},
  {38, 0x03, 34},
  {3, 0x02, 47},
  {4, 0x02, 47},
  {6, 0x02, 47},
  {9, 0x02, 47},
  {14, 0x02, 47},
  {20, 0x02, 47},
  {28, 0x02, 47},
  {38, 0x03, 47},
},
/* 35 */
{
  {2, 0x02, 61},
  {5, 0x02, 61},
  {13, 0x02, 61},
  {27, 0x03, 61},
  {2, 0x02, 65},
  {5, 0x02, 65},
  {13, 0x02, 65},
  {27, 0x03, 65},
  {2, 0x02, 98},
  {5, 0x02, 98},
  {13, 0x02, 98},
  {27, 0x03, 98},
  {2, 0x02, 102},
  {5, 0x02, 102},
  {13, 0x02, 102},
  {27, 0x03, 102},
},
/* 36 */
{
  {3, 0x02, 61},
  {4, 0x02, 61},
  {6, 0x02, 61},
  {9, 0x02, 61},
  {14, 0x02, 61},
  {20, 0x02, 61},
  {28, 0x02, 61},
  {38, 0x03, 61},
  {3, 0x02, 65},
  {4, 0x02, 65},
  {6, 0x02, 65},
  {9, 0x02, 65},
  {14, 0x02, 65},
  {20, 0x02, 65},
  {28, 0x02, 65},
  {38, 0x03, 65},
},
/* 37 */
{
  {3, 0x02, 98},
  {4, 0x02, 98},
  {6, 0x02, 98},
  {9, 0x02, 98},
  {14, 0x02, 98},
  {20, 0x02, 98},
  {28, 0x02, 98},
  {38, 0x03, 98},
  {3, 0x02, 102},
  {4, 0x02, 102},
  {6, 0x02, 102},
  {9, 0x02, 102},
  {14, 0x02, 102},
  {20, 0x02, 102},
  {28, 0x02, 102},
  {38, 0x03, 102},
},
/* 38 */
{
  {0, 0x03, 103},
  {0, 0x03, 104},
  {0, 0x03, 108},
  {0, 0x03, 118},
  {0, 0x03, 120},
  {45, 0x00, 0},
  {47, 0x00, 0},
  {48, 0x00, 0},
  {52, 0x00, 0},
  {53, 0x00, 0},
  {55, 0x00, 0},
  {56, 0x00, 0},
  {61, 0x00, 0},
  {64, 0x00, 0},
  {68, 0x00, 0},
  {71, 0x01, 0},
},
/* 39 */
{
  {1, 0x02, 103},
  {12, 0x03, 103},
  {1, 0x02, 104},
  {12, 0x03, 104},
  {1, 0x02, 108},
  {12, 0x03, 108},
  {1, 0x02, 118},
  {12, 0x03, 118},
  {1, 0x02, 120},
  {12, 0x03, 120},
  {0, 0x03, 66},
  {0, 0x03, 67},
  {0, 0x03, 68},
  {0, 0x03, 69},
  {0, 0x03, 70},
  {0, 0x03, 73},
},
/* 40 */
{
  {2, 0x02, 103},
  {5, 0x02, 103},
  {13, 0x02, 103},
  {27, 0x03, 103},
  {2, 0x02, 104},
  {5, 0x02, 104},
  {13, 0x02, 104},
  {27, 0x03, 104},
  {2, 0x02, 108},
  {5, 0x02, 108},
  {13, 0x02, 108},
  {27, 0x03, 108},
  {2, 0x02, 118},
  {5, 0x02, 118},
  {13, 0x02, 118},
  {27, 0x03, 118},
},
/* 41 */
{
  {3, 0x02, 103},
  {4, 0x02, 103},
  {6, 0x02, 103},
  {9, 0x02, 103},
  {14, 0x02, 103},
  {20, 0x02, 103},
  {28, 0x02, 103},
  {38, 0x03, 103},
  {3, 0x02, 104},
  {4, 0x02, 104},
  {6, 0x02, 104},
  {9, 0x02, 104},
  {14, 0x02, 104},
  {20, 0x02, 104},
  {28, 0x02, 104},
  {38, 0x03, 104},
},
/* 42 */
{
  {3, 0x02, 108},
  {4, 0x02, 108},
  {6, 0x02, 108},
  {9, 0x02, 108},
  {14, 0x02, 108},
  {20, 0x02, 108},
  {28, 0x02, 108},
  {38, 0x03, 108},
  {3, 0x02, 118},
  {4, 0x02, 118},
  {6, 0x02, 118},
  {9, 0x02, 118},
  {14, 0x02, 118},
  {20, 0x02, 118},
  {28, 0x02, 118},
  {38, 0x03, 118},
},
/* 43 */
{
  {2, 0x02, 120},
  {5, 0x02, 120},
  {13, 0x02, 120},
  {27, 0x03, 120},
  {1, 0x02, 66},
  {12, 0x03, 66},
  {1, 0x02, 67},
  {12, 0x03, 67},
  {1, 0x02, 68},
  {12, 0x03, 68},
  {1, 0x02, 69},
  {12, 0x03, 69},
  {1, 0x02, 70},
  {12, 0x03, 70},
  {1, 0x02, 73},
  {12, 0x03, 73},
},
/* 44 */
{
  {3, 0x02, 120},
  {4, 0x02, 120},
  {6, 0x02, 120},
  {9, 0x02, 120},
  {14, 0x02, 120},
  {20, 0x02, 120},
  {28, 0x02, 120},
  {38, 0x03, 120},
  {2, 0x02, 66},
  {5, 0x02, 66},
  {13, 0x02, 66},
  {27, 0x03, 66},
  {2, 0x02, 67},
  {5, 0x02, 67},
  {13, 0x02, 67},
  {27, 0x03, 67},
},
/* 45 */
{
  {3, 0x02, 66},
  {4, 0x02, 66},
  {6, 0x02, 66},
  {9, 0x02, 66},
  {14, 0x02, 66},
  {20, 0x02, 66},
  {28, 0x02, 66},
  {38, 0x03, 66},
  {3, 0x02, 67},
  {4, 0x02, 67},
  {6, 0x02, 67},
  {9, 0x02, 67},
  {14, 0x02, 67},
  {20, 0x02, 67},
  {28, 0x02, 67},
  {38, 0x03, 67},
},
/* 46 */
{
  {2, 0x02, 68},
  {5, 0x02, 68},
  {13, 0x02, 68},
  {27, 0x03, 68},
  {2, 0x02, 69},
  {5, 0x02, 69},
  {13, 0x02, 69},
  {27, 0x03, 69},
  {2, 0x02, 70},
  {5, 0x02, 70},
  {13, 0x02, 70},
  {27, 0x03, 70},
  {2, 0x02, 73},
  {5, 0x02, 73},
  {13, 0x02, 73},
  {27, 0x03, 73},
},
/* 47 */
{
  {3, 0x02, 68},
  {4, 0x02, 68},
  {6, 0x02, 68},
  {9, 0x02, 68},
  {14, 0x02, 68},
  {20, 0x02, 68},
  {28, 0x02, 68},
  {38, 0x03, 68},
  {3, 0x02, 69},
  {4, 0x02, 69},
  {6, 0x02, 69},
  {9, 0x02, 69},
  {14, 0x02, 69},
  {20, 0x02, 69},
  {28, 0x02, 69},
  {38, 0x03, 69},
},
/* 48 */
{
  {3, 0x02, 70},
  {4, 0x02, 70},
  {6, 0x02, 70},
  {9, 0x02, 70},
  {14, 0x02, 70},
  {20, 0x02, 70},
  {28, 0x02, 70},
  {38, 0x03, 70},
  {3, 0x02, 73},
  {4, 0x02, 73},
  {6, 0x02, 73},
  {9, 0x02, 73},
  {14, 0x02, 73},
  {20, 0x02, 73},
  {28, 0x02, 73},
  {38, 0x03, 73},
},
/* 49 */
{
  {0, 0x03, 74},
  {0, 0x03, 78},
  {0, 0x03, 79},
  {0, 0x03, 87},
  {0, 0x03, 119},
  {0, 0x03, 121},
  {57, 0x00, 0},
  {58, 0x00, 0},
  {62, 0x00, 0},
  {63, 0x00, 0},
  {65, 0x00, 0},
  {66, 0x00, 0},
  {69, 0x00, 0},
  {70, 0x00, 0},
  {72, 0x00, 0},
  {75, 0x00, 0},
},
/* 50 */
{
  {1, 0x02, 74},
  {12, 0x03, 74},
  {1, 0x02, 78},
  {12, 0x03, 78},
  {1, 0x02, 79},
  {12, 0x03, 79},
  {1, 0x02, 87},
  {12, 0x03, 87},
  {1, 0x02, 119},
  {12, 0x03, 119},
  {1, 0x02, 121},
  {12, 0x03, 121},
  {0, 0x03, 37},
  {0, 0x03, 40},
  {0, 0x03, 41},
  {0, 0x03, 59},
},
/* 51 */
{
  {2, 0x02, 74},
  {5, 0x02, 74},
  {13, 0x02, 74},
  {27, 0x03, 74},
  {2, 0x02, 78},
  {5, 0x02, 78},
  {13, 0x02, 78},
  {27, 0x03, 78},
  {2, 0x02, 79},
  {5, 0x02, 79},
  {13, 0x02, 79},
  {27, 0x03, 79},
  {2, 0x02, 87},
  {5, 0x02, 87},
  {13, 0x02, 87},
  {27, 0x03, 87},
},
/* 52 */
{
  {3, 0x02, 74},
  {4, 0x02, 74},
  {6, 0x02, 74},
  {9, 0x02, 74},
  {14, 0x02, 74},
  {20, 0x02, 74},
  {28, 0x02, 74},
  {38, 0x03, 74},
  {3, 0x02, 78},
  {4, 0x02, 78},
  {6, 0x02, 78},
  {9, 0x02, 78},
  {14, 0x02, 78},
  {20, 0x02, 78},
  {28, 0x02, 78},
  {38, 0x03, 78},
},
/* 53 */
{
  {3, 0x02, 79},
  {4, 0x02, 79},
  {6, 0x02, 79},
  {9, 0x02, 79},
  {14, 0x02, 79},
  {20, 0x02, 79},
  {28, 0x02, 79},
  {38, 0x03, 79},
  {3, 0x02, 87},
  {4, 0x02, 87},
  {6, 0x02, 87},
  {9, 0x02, 87},
  {14, 0x02, 87},
  {20, 0x02, 87},
  {28, 0x02, 87},
  {38, 0x03, 87},
},
/* 54 */
{
  {2, 0x02, 119},
  {5, 0x02, 119},
  {13, 0x02, 119},
  {27, 0x03, 119},
  {2, 0x02, 121},
  {5, 0x02, 121},
  {13, 0x02, 121},
  {27, 0x03, 121},
  {1, 0x02, 37},
  {12, 0x03, 37},
  {1, 0x02, 40},
  {12, 0x03, 40},
  {1, 0x02, 41},
  {12, 0x03, 41},
  {1, 0x02, 59},
  {12, 0x03, 59},
},
/* 55 */
{
  {3, 0x02, 119},
  {4, 0x02, 119},
  {6, 0x02, 119},
  {9, 0x02, 119},
  {14, 0x02, 119},
  {20, 0x02, 119},
  {28, 0x02, 119},
  {38, 0x03, 119},
  {3, 0x02, 121},
  {4, 0x02, 121},
  {6, 0x02, 121},
  {9, 0x02, 121},
  {14, 0x02, 121},
  {20, 0x02, 121},
  {28, 0x02, 121},
  {38, 0x03, 121},
},
/* 56 */
{
  {2, 0x02, 37},
  {5, 0x02, 37},
  {13, 0x02, 37},
  {27, 0x03, 37},
  {2, 0x02, 40},
  {5, 0x02, 40},
  {13, 0x02, 40},
  {27, 0x03, 40},
  {2, 0x02, 41},
  {5, 0x02, 41},
  {13, 0x02, 41},
  {27, 0x03, 41},
  {2, 0x02, 59},
  {5, 0x02, 59},
  {13, 0x02, 59},
  {27, 0x03, 59},
},
/* 57 */
{
  {3, 0x02, 37},
  {4, 0x02, 37},
  {6, 0x02, 37},
  {9, 0x02, 37},
  {14, 0x02, 37},
  {20, 0x02, 37},
  {28, 0x02, 37},
  {38, 0x03, 37},
  {3, 0x02, 40},
  {4, 0x02, 40},
  {6, 0x02, 40},
  {9, 0x02, 40},
  {14, 0x02, 40},
  {20, 0x02, 40},
  {28, 0x02, 40},
  {38, 0x03, 40},
},
/* 58 */
{
  {3, 0x02, 41},
  {4, 0x02, 41},
  {6, 0x02, 41},
  {9, 0x02, 41},
  {14, 0x02, 41},
  {20, 0x02, 41},
  {28, 0x02, 41},
  {38, 0x03, 41},
  {3, 0x02, 59},
  {4, 0x02, 59},
  {6, 0x02, 59},
  {9, 0x02, 59},
  {14, 0x02, 59},
  {20, 0x02, 59},
  {28, 0x02, 59},
  {38, 0x03, 59},
},
/* 59 */
{
  {0, 0x03, 72},
  {0, 0x03, 76},
  {0, 0x03, 80},
  {0, 0x03, 82},
  {0, 0x03, 85},
  {0, 0x03, 86},
  {0, 0x03, 89},
  {0, 0x03, 95},
  {0, 0x03, 106},
  {0, 0x03, 107},
  {0, 0x03, 113},
  {0, 0x03, 122},
  {73, 0x00, 0},
  {74, 0x00, 0},
  {76, 0x00, 0},
  {78, 0x00, 0},
},
/* 60 */
{
  {1, 0x02, 72},
  {12, 0x03, 72},
  {1, 0x02, 76},
  {12, 0x03, 76},
  {1, 0x02, 80},
  {12, 0x03, 80},
  {1, 0x02, 82},
  {12, 0x03, 82},
  {1, 0x02, 85},
  {12, 0x03, 85},
  {1, 0x02, 86},
  {12, 0x03, 86},
  {1, 0x02, 89},
  {12, 0x03, 89},
  {1, 0x02, 95},
  {12, 0x03, 95},
},
/* 61 */
{
  {2, 0x02, 72},
  {5, 0x02, 72},
  {13, 0x02, 72},
  {27, 0x03, 72},
  {2, 0x02, 76},
  {5, 0x02, 76},
  {13, 0x02, 76},
  {27, 0x03, 76},
  {2, 0x02, 80},
  {5, 0x02, 80},
  {13, 0x02, 80},
  {27, 0x03, 80},
  {2, 0x02, 82},
  {5, 0x02, 82},
  {13, 0x02, 82},
  {27, 0x03, 82},
},
/* 62 */
{
  {3, 0x02, 72},
  {4, 0x02, 72},
  {6, 0x02, 72},
  {9, 0x02, 72},
  {14, 0x02, 72},
  {20, 0x02, 72},
  {28, 0x02, 72},
  {38, 0x03, 72},
  {3, 0x02, 76},
  {4, 0x02, 76},
  {6, 0x02, 76},
  {9, 0x02, 76},
  {14, 0x02, 76},
  {20, 0x02, 76},
  {28, 0x02, 76},
  {38, 0x03, 76},
},
/* 63 */
{
  {3, 0x02, 80},
  {4, 0x02, 80},
  {6, 0x02, 80},
  {9, 0x02, 80},
  {14, 0x02, 80},
  {20, 0x02, 80},
  {28, 0x02, 80},
  {38, 0x03, 80},
  {3, 0x02, 82},
  {4, 0x02, 82},
  {6, 0x02, 82},
  {9, 0x02, 82},
  {14, 0x02, 82},
  {20, 0x02, 82},
  {28, 0x02, 82},
  {38, 0x03, 82},
},
/* 64 */
{
  {2, 0x02, 85},
  {5, 0x02, 85},
  {13, 0x02, 85},
  {27, 0x03, 85},
  {2, 0x02, 86},
  {5, 0x02, 86},
  {13, 0x02, 86},
  {27, 0x03, 86},
  {2, 0x02, 89},
  {5, 0x02, 89},
  {13, 0x02, 89},
  {27, 0x03, 89},
  {2, 0x02, 95},
  {5, 0x02, 95},
  {13, 0x02, 95},
  {27, 0x03, 95},
},
/* 65 */
{
  {3, 0x02, 85},
  {4, 0x02, 85},
  {6, 0x02, 85},
  {9, 0x02, 85},
  {14, 0x02, 85},
  {20, 0x02, 85},
  {28, 0x02, 85},
  {38, 0x03, 85},
  {3, 0x02, 86},
  {4, 0x02, 86},
  {6, 0x02, 86},
  {9, 0x02, 86},
  {14, 0x02, 86},
  {20, 0x02, 86},
  {28, 0x02, 86},
  {38, 0x03, 86},
},
/* 66 */
{
  {3, 0x02, 89},
  {4, 0x02, 89},
  {6, 0x02, 89},
  {9, 0x02, 89},
  {14, 0x02, 89},
  {20, 0x02, 89},
  {28, 0x02, 89},
  {38, 0x03, 89},
  {3, 0x02, 95},
  {4, 0x02, 95},
  {6, 0x02, 95},
  {9, 0x02, 95},
  {14, 0x02, 95},
  {20, 0x02, 95},
  {28, 0x02, 95},
  {38, 0x03, 95},
},
/* 67 */
{
  {1, 0x02, 106},
  {12, 0x03, 106},
  {1, 0x02, 107},
  {12, 0x03, 107},
  {1, 0x02, 113},
  {12, 0x03, 113},
  {1, 0x02, 122},
  {12, 0x03, 122},
  {0, 0x03, 38},
  {0, 0x03, 75},
  {0, 0x03, 81},
  {0, 0x03, 88},
  {0, 0x03, 90},
  {77, 0x00, 0},
  {79, 0x00, 0},
  {81, 0x00, 0},
},
/* 68 */
{
  {2, 0x02, 106},
  {5, 0x02, 106},
  {13, 0x02, 106},
  {27, 0x03, 106},
  {2, 0x02, 107},
  {5, 0x02, 107},
  {13, 0x02, 107},
  {27, 0x03, 107},
  {2, 0x02, 113},
  {5, 0x02, 113},
  {13, 0x02, 113},
  {27, 0x03, 113},
  {2, 0x02, 122},
  {5, 0x02, 122},
  {13, 0x02, 122},
  {27, 0x03, 122},
},
/* 69 */
{
  {3, 0x02, 106},
  {4, 0x02, 106},
  {6, 0x02, 106},
  {9, 0x02, 106},
  {14, 0x02, 106},
  {20, 0x02, 106},
  {28, 0x02, 106},
  {38, 0x03, 106},
  {3, 0x02, 107},
  {4, 0x02, 107},
  {6, 0x02, 107},
  {9, 0x02, 107},
  {14, 0x02, 107},
  {20, 0x02, 107},
  {28, 0x02, 107},
  {38, 0x03, 107},
},
/* 70 */
{
  {3, 0x02, 113},
  {4, 0x02, 113},
  {6, 0x02, 113},
  {9, 0x02, 113},
  {14, 0x02, 113},
  {20, 0x02, 113},
  {28, 0x02, 113},
  {38, 0x03, 113},
  {3, 0x02, 122},
  {4, 0x02, 122},
  {6, 0x02, 122},
  {9, 0x02, 122},
  {14, 0x02, 122},
  {20, 0x02, 122},
  {28, 0x02, 122},
  {38, 0x03, 122},
},
/* 71 */
{
  {1, 0x02, 38},
  {12, 0x03, 38},
  {1, 0x02, 75},
  {12, 0x03, 75},
  {1, 0x02, 81},
  {12, 0x03, 81},
  {1, 0x02, 88},
  {12, 0x03, 88},
  {1, 0x02, 90},
  {12, 0x03, 90},
  {0, 0x03, 43},
  {0, 0x03, 91},
  {0, 0x03, 93},
  {80, 0x00, 0},
  {82, 0x00, 0},
  {84, 0x00, 0},
},
/* 72 */
{
  {2, 0x02, 38},
  {5, 0x02, 38},
  {13, 0x02, 38},
  {27, 0x03, 38},
  {2, 0x02, 75},
  {5, 0x02, 75},
  {13, 0x02, 75},
  {27, 0x03, 75},
  {2, 0x02, 81},
  {5, 0x02, 81},
  {13, 0x02, 81},
  {27, 0x03, 81},
  {2, 0x02, 88},
  {5, 0x02, 88},
  {13, 0x02, 88},
  {27, 0x03, 88},
},
/* 73 */
{
  {3, 0x02, 38},
  {4, 0x02, 38},
  {6, 0x02, 38},
  {9, 0x02, 38},
  {14, 0x02, 38},
  {20, 0x02, 38},
  {28, 0x02, 38},
  {38, 0x03, 38},
  {3, 0x02, 75},
  {4, 0x02, 75},
  {6, 0x02, 75},
  {9, 0x02, 75},
  {14, 0x02, 75},
  {20, 0x02, 75},
  {28, 0x02, 75},
  {38, 0x03, 75},
},
/* 74 */
{
  {3, 0x02, 81},
  {4, 0x02, 81},
  {6, 0x02, 81},
  {9, 0x02, 81},
  {14, 0x02, 81},
  {20, 0x02, 81},
  {28, 0x02, 81},
  {38, 0x03, 81},
  {3, 0x02, 88},
  {4, 0x02, 88},
  {6, 0x02, 88},
  {9, 0x02, 88},
  {14, 0x02, 88},
  {20, 0x02, 88},
  {28, 0x02, 88},
  {38, 0x03, 88},
},
/* 75 */
{
  {2, 0x02, 90},
  {5, 0x02, 90},
  {13, 0x02, 90},
  {27, 0x03, 90},
  {1, 0x02, 43},
  {12, 0x03, 43},
  {1, 0x02, 91},
  {12, 0x03, 91},
  {1, 0x02, 93},
  {12, 0x03, 93},
  {0, 0x03, 33},
  {0, 0x03, 42},
  {0, 0x03, 63},
  {83, 0x00, 0},
  {85, 0x00, 0},
  {86, 0x00, 0},
},
/* 76 */
{
  {3, 0x02, 90},
  {4, 0x02, 90},
  {6, 0x02, 90},
  {9, 0x02, 90},
  {14, 0x02, 90},
  {20, 0x02, 90},
  {28, 0x02, 90},
  {38, 0x03, 90},
  {2, 0x02, 43},
  {5, 0x02, 43},
  {13, 0x02, 43},
  {27, 0x03, 43},
  {2, 0x02, 91},
  {5, 0x02, 91},
  {13, 0x02, 91},
  {27, 0x03, 91},
},
/* 77 */
{
  {3, 0x02, 43},
  {4, 0x02, 43},
  {6, 0x02, 43},
  {9, 0x02, 43},
  {14, 0x02, 43},
  {20, 0x02, 43},
  {28, 0x02, 43},
  {38, 0x03, 43},
  {3, 0x02, 91},
  {4, 0x02, 91},
  {6, 0x02, 91},
  {9, 0x02, 91},
  {14, 0x02, 91},
  {20, 0x02, 91},
  {28, 0x02, 91},
  {38, 0x03, 91},
},
/* 78 */
{
  {2, 0x02, 93},
  {5, 0x02, 93},
  {13, 0x02, 93},
  {27, 0x03, 93},
  {1, 0x02, 33},
  {12, 0x03, 33},
  {1, 0x02, 42},
  {12, 0x03, 42},
  {1, 0x02, 63},
  {12, 0x03, 63},
  {0, 0x03, 35},
  {0, 0x03, 39},
  {0, 0x03, 62},
  {0, 0x03, 92},
  {87, 0x00, 0},
  {88, 0x00, 0},
},
/* 79 */
{
  {3, 0x02, 93},
  {4, 0x02, 93},
  {6, 0x02, 93},
  {9, 0x02, 93},
  {14, 0x02, 93},
  {20, 0x02, 93},
  {28, 0x02, 93},
  {38, 0x03, 93},
  {2, 0x02, 33},
  {5, 0x02, 33},
  {13, 0x02, 33},
  {27, 0x03, 33},
  {2, 0x02, 42},
  {5, 0x02, 42},
  {13, 0x02, 42},
  {27, 0x03, 42},
},
/* 80 */
{
  {3, 0x02, 33},
  {4, 0x02, 33},
  {6, 0x02, 33},
  {9, 0x02, 33},
  {14, 0x02, 33},
  {20, 0x02, 33},
  {28, 0x02, 33},
  {38, 0x03, 33},
  {3, 0x02, 42},
  {4, 0x02, 42},
  {6, 0x02, 42},
  {9, 0x02, 42},
  {14, 0x02, 42},
  {20, 0x02, 42},
  {28, 0x02, 42},
  {38, 0x03, 42},
},
/* 81 */
{
  {2, 0x02, 63},
  {5, 0x02, 63},
  {13, 0x02, 63},
  {27, 0x03, 63},
  {1, 0x02, 35},
  {12, 0x03, 35},
  {1, 0x02, 39},
  {12, 0x03, 39},
  {1, 0x02, 62},
  {12, 0x03, 62},
  {1, 0x02, 92},
  {12, 0x03, 92},
  {0, 0x03, 36},
  {0, 0x03, 124},
  {89, 0x00, 0},
  {91, 0x00, 0},
},
/* 82 */
{
  {3, 0x02, 63},
  {4, 0x02, 63},
  {6, 0x02, 63},
  {9, 0x02, 63},
  {14, 0x02, 63},
  {20, 0x02, 63},
  {28, 0x02, 63},
  {38, 0x03, 63},
  {2, 0x02, 35},
  {5, 0x02, 35},
  {13, 0x02, 35},
  {27, 0x03, 35},
  {2, 0x02, 39},
  {5, 0x02, 39},
  {13, 0x02, 39},
  {27, 0x03, 39},
},
/* 83 */
{
  {3, 0x02, 35},
  {4, 0x02, 35},
  {6, 0x02, 35},
  {9, 0x02, 35},
  {14, 0x02, 35},
  {20, 0x02, 35},
  {28, 0x02, 35},
  {38, 0x03, 35},
  {3, 0x02, 39},
  {4, 0x02, 39},
  {6, 0x02, 39},
  {9, 0x02, 39},
  {14, 0x02, 39},
  {20, 0x02, 39},
  {28, 0x02, 39},
  {38, 0x03, 39},
},
/* 84 */
{
  {2, 0x02, 62},
  {5, 0x02, 62},
  {13, 0x02, 62},
  {27, 0x03, 62},
  {2, 0x02, 92},
  {5, 0x02, 92},
  {13, 0x02, 92},
  {27, 0x03, 92},
  {1, 0x02, 36},
  {12, 0x03, 36},
  {1, 0x02, 124},
  {12, 0x03, 124},
  {0, 0x03, 94},
  {90, 0x00, 0},
  {92, 0x00, 0},
  {93, 0x00, 0},
},
/* 85 */
{
  {3, 0x02, 62},
  {4, 0x02, 62},
  {6, 0x02, 62},
  {9, 0x02, 62},
  {14, 0x02, 62},
  {20, 0x02, 62},
  {28, 0x02, 62},
  {38, 0x03, 62},
  {3, 0x02, 92},
  {4, 0x02, 92},
  {6, 0x02, 92},
  {9, 0x02, 92},
  {14, 0x02, 92},
  {20, 0x02, 92},
  {28, 0x02, 92},
  {38, 0x03, 92},
},
/* 86 */
{
  {2, 0x02, 36},
  {5, 0x02, 36},
  {13, 0x02, 36},
  {27, 0x03, 36},
  {2, 0x02, 124},
  {5, 0x02, 124},
  {13, 0x02, 124},
  {27, 0x03, 124},
  {1, 0x02, 94},
  {12, 0x03, 94},
  {0, 0x03, 60},
  {0, 0x03, 64},
  {0, 0x03, 123},
  {0, 0x03, 125},
  {0, 0x03, 126},
  {94, 0x00, 0},
},
/* 87 */
{
  {3, 0x02, 36},
  {4, 0x02, 36},
  {6, 0x02, 36},
  {9, 0x02, 36},
  {14, 0x02, 36},
  {20, 0x02, 36},
  {28, 0x02, 36},
  {38, 0x03, 36},
  {3, 0x02, 124},
  {4, 0x02, 124},
  {6, 0x02, 124},
  {9, 0x02, 124},
  {14, 0x02, 124},
  {20, 0x02, 124},
  {28, 0x02, 124},
  {38, 0x03, 124},
},
/* 88 */
{
  {2, 0x02, 94},
  {5, 0x02, 94},
  {13, 0x02, 94},
  {27, 0x03, 94},
  {1, 0x02, 60},
  {12, 0x03, 60},
  {1, 0x02, 64},
  {12, 0x03, 64},
  {1, 0x02, 123},
  {12, 0x03, 123},
  {1, 0x02, 125},
  {12, 0x03, 125},
  {1, 0x02, 126},
  {12, 0x03, 126},
  {0, 0x03, 96},
  {95, 0x00, 0},
},
/* 89 */
{
  {3, 0x02, 94},
  {4, 0x02, 94},
  {6, 0x02, 94},
  {9, 0x02, 94},
  {14, 0x02, 94},
  {20, 0x02, 94},
  {28, 0x02, 94},
  {38, 0x03, 94},
  {2, 0x02, 60},
  {5, 0x02, 60},
  {13, 0x02, 60},
  {27, 0x03, 60},
  {2, 0x02, 64},
  {5, 0x02, 64},
  {13, 0x02, 64},
  {27, 0x03, 64},
},
/* 90 */
{
  {3, 0x02, 60},
  {4, 0x02, 60},
  {6, 0x02, 60},
  {9, 0x02, 60},
  {14, 0x02, 60},
  {20, 0x02, 60},
  {28, 0x02, 60},
  {38, 0x03, 60},
  {3, 0x02, 64},
  {4, 0x02, 64},
  {6, 0x02, 64},
  {9, 0x02, 64},
  {14, 0x02, 64},
  {20, 0x02, 64},
  {28, 0x02, 64},
  {38, 0x03, 64},
},
/* 91 */
{
  {2, 0x02, 123},
  {5, 0x02, 123},
  {13, 0x02, 123},
  {27, 0x03, 123},
  {2, 0x02, 125},
  {5, 0x02, 125},
  {13, 0x02, 125},
  {27, 0x03, 125},
  {2, 0x02, 126},
  {5, 0x02, 126},
  {13, 0x02, 126},
  {27, 0x03, 126},
  {1, 0x02, 96},
  {12, 0x03, 96},
  {96, 0x00, 0},
  {159, 0x00, 0},
},
/* 92 */
{
  {3, 0x02, 123},
  {4, 0x02, 123},
  {6, 0x02, 123},
  {9, 0x02, 123},
  {14, 0x02, 123},
  {20, 0x02, 123},
  {28, 0x02, 123},
  {38, 0x03, 123},
  {3, 0x02, 125},
  {4, 0x02, 125},
  {6, 0x02, 125},
  {9, 0x02, 125},
  {14, 0x02, 125},
  {20, 0x02, 125},
  {28, 0x02, 125},
  {38, 0x03, 125},
},
/* 93 */
{
  {3, 0x02, 126},
  {4, 0x02, 126},
  {6, 0x02, 126},
  {9, 0x02, 126},
  {14, 0x02, 126},
  {20, 0x02, 126},
  {28, 0x02, 126},
  {38, 0x03, 126},
  {2, 0x02, 96},
  {5, 0x02, 96},
  {13, 0x02, 96},
  {27, 0x03, 96},
  {97, 0x00, 0},
  {128, 0x00, 0},
  {160, 0x00, 0},
  {193, 0x00, 0},
},
/* 94 */
{
  {3, 0x02, 96},
  {4, 0x02, 96},
  {6, 0x02, 96},
  {9, 0x02, 96},
  {14, 0x02, 96},
  {20, 0x02, 96},
  {28, 0x02, 96},
  {38, 0x03, 96},
  {98, 0x00, 0},
  {113, 0x00, 0},
  {129, 0x00, 0},
  {144, 0x00, 0},
  {161, 0x00, 0},
  {176, 0x00, 0},
  {194, 0x00, 0},
  {225, 0x00, 0},
},
/* 95 */
{
  {99, 0x00, 0},
  {106, 0x00, 0},
  {114, 0x00, 0},
  {121, 0x00, 0},
  {130, 0x00, 0},
  {137, 0x00, 0},
  {145, 0x00, 0},
  {152, 0x00, 0},
  {162, 0x00, 0},
  {169, 0x00, 0},
  {177, 0x00, 0},
  {184, 0x00, 0},
  {195, 0x00, 0},
  {210, 0x00, 0},
  {226, 0x00, 0},
  {241, 0x00, 0},
},
/* 96 */
{
  {100, 0x00, 0},
  {103, 0x00, 0},
  {107, 0x00, 0},
  {110, 0x00, 0},
  {115, 0x00, 0},
  {118, 0x00, 0},
  {122, 0x00, 0},
  {125, 0x00, 0},
  {131, 0x00, 0},
  {134, 0x00, 0},
  {138, 0x00, 0},
  {141, 0x00, 0},
  {146, 0x00, 0},
  {149, 0x00, 0},
  {153, 0x00, 0},
  {156, 0x00, 0},
},
/* 97 */
{
  {101, 0x00, 0},
  {102, 0x00, 0},
  {104, 0x00, 0},
  {105, 0x00, 0},
  {108, 0x00, 0},
  {109, 0x00, 0},
  {111, 0x00, 0},
  {112, 0x00, 0},
  {116, 0x00, 0},
  {117, 0x00, 0},
  {119, 0x00, 0},
  {120, 0x00, 0},
  {123, 0x00, 0},
  {124, 0x00, 0},
  {126, 0x00, 0},
  {127, 0x00, 0},
},
/* 98 */
{
  {0, 0x03, 163},
  {0, 0x03, 164},
  {0, 0x03, 165},
  {0, 0x03, 166},
  {0, 0x03, 167},
  {0, 0x03, 168},
  {0, 0x03, 169},
  {0, 0x03, 170},
  {0, 0x03, 171},
  {0, 0x03, 172},
  {0, 0x03, 173},
  {0, 0x03, 174},
  {0, 0x03, 175},
  {0, 0x03, 176},
  {0, 0x03, 177},
  {0, 0x03, 178},
},
/* 99 */
{
  {1, 0x02, 163},
  {12, 0x03, 163},
  {1, 0x02, 164},
  {12, 0x03, 164},
  {1, 0x02, 165},
  {12, 0x03, 165},
  {1, 0x02, 166},
  {12, 0x03, 166},
  {1, 0x02, 167},
  {12, 0x03, 167},
  {1, 0x02, 168},
  {12, 0x03, 168},
  {1, 0x02, 169},
  {12, 0x03, 169},
  {1, 0x02, 170},
  {12, 0x03, 170},
},
/* 100 */
{
  {2, 0x02, 163},
  {5, 0x02, 163},
  {13, 0x02, 163},
  {27, 0x03, 163},
  {2, 0x02, 164},
  {5, 0x02, 164},
  {13, 0x02, 164},
  {27, 0x03, 164},
  {2, 0x02, 165},
  {5, 0x02, 165},
  {13, 0x02, 165},
  {27, 0x03, 165},
  {2, 0x02, 166},
  {5, 0x02, 166},
  {13, 0x02, 166},
  {27, 0x03, 166},
},
/* 101 */
{
  {3, 0x02, 163},
  {4, 0x02, 163},
  {6, 0x02, 163},
  {9, 0x02, 163},
  {14, 0x02, 163},
  {20, 0x02, 163},
  {28, 0x02, 163},
  {38, 0x03, 163},
  {3, 0x02, 164},
  {4, 0x02, 164},
  {6, 0x02, 164},
  {9, 0x02, 164},
  {14, 0x02, 164},
  {20, 0x02, 164},
  {28, 0x02, 164},
  {38, 0x03, 164},
},
/* 102 */
{
  {3, 0x02, 165},
  {4, 0x02, 165},
  {6, 0x02, 165},
  {9, 0x02, 165},
  {14, 0x02, 165},
  {20, 0x02, 165},
  {28, 0x02, 165},
  {38, 0x03, 165},
  {3, 0x02, 166},
  {4, 0x02, 166},
  {6, 0x02, 166},
  {9, 0x02, 166},
  {14, 0x02, 166},
  {20, 0x02, 166},
  {28, 0x02, 166},
  {38, 0x03, 166},
},
/* 103 */
{
  {2, 0x02, 167},
  {5, 0x02, 167},
  {13, 0x02, 167},
  {27, 0x03, 167},
  {2, 0x02, 168},
  {5, 0x02, 168},
  {13, 0x02, 168},
  {27, 0x03, 168},
  {2, 0x02, 169},
  {5, 0x02, 169},
  {13, 0x02, 169},
  {27, 0x03, 169},
  {2, 0x02, 170},
  {5, 0x02, 170},
  {13, 0x02, 170},
  {27, 0x03, 170},
},
/* 104 */
{
  {3, 0x02, 167},
  {4, 0x02, 167},
  {6, 0x02, 167},
  {9, 0x02, 167},
  {14, 0x02, 167},
  {20, 0x02, 167},
  {28, 0x02, 167},
  {38, 0x03, 167},
  {3, 0x02, 168},
  {4, 0x02, 168},
  {6, 0x02, 168},
  {9, 0x02, 168},
  {14, 0x02, 168},
  {20, 0x02, 168},
  {28, 0x02, 168},
  {38, 0x03, 168},
},
/* 105 */
{
  {3, 0x02, 169},
  {4, 0x02, 169},
  {6, 0x02, 169},
  {9, 0x02, 169},
  {14, 0x02, 169},
  {20, 0x02, 169},
  {28, 0x02, 169},
  {38, 0x03, 169},
  {3, 0x02, 170},
  {4, 0x02, 170},
  {6, 0x02, 170},
  {9, 0x02, 170},
  {14, 0x02, 170},
  {20, 0x02, 170},
  {28, 0x02, 170},
  {38, 0x03, 170},
},
/* 106 */
{
  {1, 0x02, 171},
  {12, 0x03, 171},
  {1, 0x02, 172},
  {12, 0x03, 172},
  {1, 0x02, 173},
  {12, 0x03, 173},
  {1, 0x02, 174},
  {12, 0x03, 174},
  {1, 0x02, 175},
  {12, 0x03, 175},
  {1, 0x02, 176},
  {12, 0x03, 176},
  {1, 0x02, 177},
  {12, 0x03, 177},
  {1, 0x02, 178},
  {12, 0x03, 178},
},
/* 107 */
{
  {2, 0x02, 171},
  {5, 0x02, 171},
  {13, 0x02, 171},
  {27, 0x03, 171},
  {2, 0x02, 172},
  {5, 0x02, 172},
  {13, 0x02, 172},
  {27, 0x03, 172},
  {2, 0x02, 173},
  {5, 0x02, 173},
  {13, 0x02, 173},
  {27, 0x03, 173},
  {2, 0x02, 174},
  {5, 0x02, 174},
  {13, 0x02, 174},
  {27, 0x03, 174},
},
/* 108 */
{
  {3, 0x02, 171},
  {4, 0x02, 171},
  {6, 0x02, 171},
  {9, 0x02, 171},
  {14, 0x02, 171},
  {20, 0x02, 171},
  {28, 0x02, 171},
  {38, 0x03, 171},
  {3, 0x02, 172},
  {4, 0x02, 172},
  {6, 0x02, 172},
  {9, 0x02, 172},
  {14, 0x02, 172},
  {20, 0x02, 172},
  {28, 0x02, 172},
  {38, 0x03, 172},
},
/* 109 */
{
  {3, 0x02, 173},
  {4, 0x02, 173},
  {6, 0x02, 173},
  {9, 0x02, 173},
  {14, 0x02, 173},
  {20, 0x02, 173},
  {28, 0x02, 173},
  {38, 0x03, 173},
  {3, 0x02, 174},
  {4, 0x02, 174},
  {6, 0x02, 174},
  {9, 0x02, 174},
  {14, 0x02, 174},
  {20, 0x02, 174},
  {28, 0x02, 174},
  {38, 0x03, 174},
},
/* 110 */
{
  {2, 0x02, 175},
  {5, 0x02, 175},
  {13, 0x02, 175},
  {27, 0x03, 175},
  {2, 0x02, 176},
  {5, 0x02, 176},
  {13, 0x02, 176},
  {27, 0x03, 176},
  {2, 0x02, 177},
  {5, 0x02, 177},
  {13, 0x02, 177},
  {27, 0x03, 177},
  {2, 0x02, 178},
  {5, 0x02, 178},
  {13, 0x02, 178},
  {27, 0x03, 178},
},
/* 111 */
{
  {3, 0x02, 175},
  {4, 0x02, 175},
  {6, 0x02, 175},
  {9, 0x02, 175},
  {14, 0x02, 175},
  {20, 0x02, 175},
  {28, 0x02, 175},
  {38, 0x03, 175},
  {3, 0x02, 176},
  {4, 0x02, 176},
  {6, 0x02, 176},
  {9, 0x02, 176},
  {14, 0x02, 176},
  {20, 0x02, 176},
  {28, 0x02, 176},
  {38, 0x03, 176},
},
/* 112 */
{
  {3, 0x02, 177},
  {4, 0x02, 177},
  {6, 0x02, 177},
  {9, 0x02, 177},
  {14, 0x02, 177},
  {20, 0x02, 177},
  {28, 0x02, 177},
  {38, 0x03, 177},
  {3, 0x02, 178},
  {4, 0x02, 178},
  {6, 0x02, 178},
  {9, 0x02, 178},
  {14, 0x02, 178},
  {20, 0x02, 178},
  {28, 0x02, 178},
  {38, 0x03, 178},
},
/* 113 */
{
  {0, 0x03, 179},
  {0, 0x03, 180},
  {0, 0x03, 181},
  {0, 0x03, 182},
  {0, 0x03, 183},
  {0, 0x03, 184},
  {0, 0x03, 185},
  {0, 0x03, 186},
  {0, 0x03, 187},
  {0, 0x03, 188},
  {0, 0x03, 189},
  {0, 0x03, 190},
  {0, 0x03, 191},
  {0, 0x03, 192},
  {0, 0x03, 193},
  {0, 0x03, 194},
},
/* 114 */
{
  {1, 0x02, 179},
  {12, 0x03, 179},
  {1, 0x02, 180},
  {12, 0x03, 180},
  {1, 0x02, 181},
  {12, 0x03, 181},
  {1, 0x02, 182},
  {12, 0x03, 182},
  {1, 0x02, 183},
  {12, 0x03, 183},
  {1, 0x02, 184},
  {12, 0x03, 184},
  {1, 0x02, 185},
  {12, 0x03, 185},
  {1, 0x02, 186},
  {12, 0x03, 186},
},
/* 115 */
{
  {2, 0x02, 179},
  {5, 0x02, 179},
  {13, 0x02, 179},
  {27, 0x03, 179},
  {2, 0x02, 180},
  {5, 0x02, 180},
  {13, 0x02, 180},
  {27, 0x03, 180},
  {2, 0x02, 181},
  {5, 0x02, 181},
  {13, 0x02, 181},
  {27, 0x03, 181},
  {2, 0x02, 182},
  {5, 0x02, 182},
  {13, 0x02, 182},
  {27, 0x03, 182},
},
/* 116 */
{
  {3, 0x02, 179},
  {4, 0x02, 179},
  {6, 0x02, 179},
  {9, 0x02, 179},
  {14, 0x02, 179},
  {20, 0x02, 179},
  {28, 0x02, 179},
  {38, 0x03, 179},
  {3, 0x02, 180},
  {4, 0x02, 180},
  {6, 0x02, 180},
  {9, 0x02, 180},
  {14, 0x02, 180},
  {20, 0x02, 180},
  {28, 0x02, 180},
  {38, 0x03, 180},
},
/* 117 */
{
  {3, 0x02, 181},
  {4, 0x02, 181},
  {6, 0x02, 181},
  {9, 0x02, 181},
  {14, 0x02, 181},
  {20, 0x02, 181},
  {28, 0x02, 181},
  {38, 0x03, 181},
  {3, 0x02, 182},
  {4, 0x02, 182},
  {6, 0x02, 182},
  {9, 0x02, 182},
  {14, 0x02, 182},
  {20, 0x02, 182},
  {28, 0x02, 182},
  {38, 0x03, 182},
},
/* 118 */
{
  {2, 0x02, 183},
  {5, 0x02, 183},
  {13, 0x02, 183},
  {27, 0x03, 183},
  {2, 0x02, 184},
  {5, 0x02, 184},
  {13, 0x02, 184},
  {27, 0x03, 184},
  {2, 0x02, 185},
  {5, 0x02, 185},
  {13, 0x02, 185},
  {27, 0x03, 185},
  {2, 0x02, 186},
  {5, 0x02, 186},
  {13, 0x02, 186},
  {27, 0x03, 186},
},
/* 119 */
{
  {3, 0x02, 183},
  {4, 0x02, 183},
  {6, 0x02, 183},
  {9, 0x02, 183},
  {14, 0x02, 183},
  {20, 0x02, 183},
  {28, 0x02, 183},
  {38, 0x03, 183},
  {3, 0x02, 184},
  {4, 0x02, 184},
  {6, 0x02, 184},
  {9, 0x02, 184},
  {14, 0x02, 184},
  {20, 0x02, 184},
  {28, 0x02, 184},
  {38, 0x03, 184},
},
/* 120 */
{
  {3, 0x02, 185},
  {4, 0x02, 185},
  {6, 0x02, 185},
  {9, 0x02, 185},
  {14, 0x02, 185},
  {20, 0x02, 185},
  {28, 0x02, 185},
  {38, 0x03, 185},
  {3, 0x02, 186},
  {4, 0x02, 186},
  {6, 0x02, 186},
  {9, 0x02, 186},
  {14, 0x02, 186},
  {20, 0x02, 186},
  {28, 0x02, 186},
  {38, 0x03, 186},
},
/* 121 */
{
  {1, 0x02, 187},
  {12, 0x03, 187},
  {1, 0x02, 188},
  {12, 0x03, 188},
  {1, 0x02, 189},
  {12, 0x03, 189},
  {1, 0x02, 190},
  {12, 0x03, 190},
  {1, 0x02, 191},
  {12, 0x03, 191},
  {1, 0x02, 192},
  {12, 0x03, 192},
  {1, 0x02, 193},
  {12, 0x03, 193},
  {1, 0x02, 194},
  {12, 0x03, 194},
},
/* 122 */
{
  {2, 0x02, 187},
  {5, 0x02, 187},
  {13, 0x02, 187},
  {27, 0x03, 187},
  {2, 0x02, 188},
  {5, 0x02, 188},
  {13, 0x02, 188},
  {27, 0x03, 188},
  {2, 0x02, 189},
  {5, 0x02, 189},
  {13, 0x02, 189},
  {27, 0x03, 189},
  {2, 0x02, 190},
  {5, 0x02, 190},
  {13, 0x02, 190},
  {27, 0x03, 190},
},
/* 123 */
{
  {3, 0x02, 187},
  {4, 0x02, 187},
  {6, 0x02, 187},
  {9, 0x02, 187},
  {14, 0x02, 187},
  {20, 0x02, 187},
  {28, 0x02, 187},
  {38, 0x03, 187},
  {3, 0x02, 188},
  {4, 0x02, 188},
  {6, 0x02, 188},
  {9, 0x02, 188},
  {14, 0x02, 188},
  {20, 0x02, 188},
  {28, 0x02, 188},
  {38, 0x03, 188},
},
/* 124 */
{
  {3, 0x02, 189},
  {4, 0x02, 189},
  {6, 0x02, 189},
  {9, 0x02, 189},
  {14, 0x02, 189},
  {20, 0x02, 189},
  {28, 0x02, 189},
  {38, 0x03, 189},
  {3, 0x02, 190},
  {4, 0x02, 190},
  {6, 0x02, 190},
  {9, 0x02, 190},
  {14, 0x02, 190},
  {20, 0x02, 190},
  {28, 0x02, 190},
  {38, 0x03, 190},
},
/* 125 */
{
  {2, 0x02, 191},
  {5, 0x02, 191},
  {13, 0x02, 191},
  {27, 0x03, 191},
  {2, 0x02, 192},
  {5, 0x02, 192},
  {13, 0x02, 192},
  {27, 0x03, 192},
  {2, 0x02, 193},
  {5, 0x02, 193},
  {13, 0x02, 193},
  {27, 0x03, 193},
  {2, 0x02, 194},
  {5, 0x02, 194},
  {13, 0x02, 194},
  {27, 0x03, 194},
},
/* 126 */
{
  {3, 0x02, 191},
  {4, 0x02, 191},
  {6, 0x02, 191},
  {9, 0x02, 191},
  {14, 0x02, 191},
  {20, 0x02, 191},
  {28, 0x02, 191},
  {38, 0x03, 191},
  {3, 0x02, 192},
  {4, 0x02, 192},
  {6, 0x02, 192},
  {9, 0x02, 192},
  {14, 0x02, 192},
  {20, 0x02, 192},
  {28, 0x02, 192},
  {38, 0x03, 192},
},
/* 127 */
{
  {3, 0x02, 193},
  {4, 0x02, 193},
  {6, 0x02, 193},
  {9, 0x02, 193},
  {14, 0x02, 193},
  {20, 0x02, 193},
  {28, 0x02, 193},
  {38, 0x03, 193},
  {3, 0x02, 194},
  {4, 0x02, 194},
  {6, 0x02, 194},
  {9, 0x02, 194},
  {14, 0x02, 194},
  {20, 0x02, 194},
  {28, 0x02, 194},
  {38, 0x03, 194},
},
/* 128 */
{
  {132, 0x00, 0},
  {133, 0x00, 0},
  {135, 0x00, 0},
  {136, 0x00, 0},
  {139, 0x00, 0},
  {140, 0x00, 0},
  {142, 0x00, 0},
  {143, 0x00, 0},
  {147, 0x00, 0},
  {148, 0x00, 0},
  {150, 0x00, 0},
  {151, 0x00, 0},
  {154, 0x00, 0},
  {155, 0x00, 0},
  {157, 0x00, 0},
  {158, 0x00, 0},
},
/* 129 */
{
  {0, 0x03, 195},
  {0, 0x03, 196},
  {0, 0x03, 197},
  {0, 0x03, 198},
  {0, 0x03, 199},
  {0, 0x03, 200},
  {0, 0x03, 201},
  {0, 0x03, 202},
  {0, 0x03, 203},
  {0, 0x03, 204},
  {0, 0x03, 205},
  {0, 0x03, 206},
  {0, 0x03, 207},
  {0, 0x03, 208},
  {0, 0x03, 209},
  {0, 0x03, 210},
},
/* 130 */
{
  {1, 0x02, 195},
  {12, 0x03, 195},
  {1, 0x02, 196},
  {12, 0x03, 196},
  {1, 0x02, 197},
  {12, 0x03, 197},
  {1, 0x02, 198},
  {12, 0x03, 198},
  {1, 0x02, 199},
  {12, 0x03, 199},
  {1, 0x02, 200},
  {12, 0x03, 200},
  {1, 0x02, 201},
  {12, 0x03, 201},
  {1, 0x02, 202},
  {12, 0x03, 202},
},
/* 131 */
{
  {2, 0x02, 195},
  {5, 0x02, 195},
  {13, 0x02, 195},
  {27, 0x03, 195},
  {2, 0x02, 196},
  {5, 0x02, 196},
  {13, 0x02, 196},
  {27, 0x03, 196},
  {2, 0x02, 197},
  {5, 0x02, 197},
  {13, 0x02, 197},
  {27, 0x03, 197},
  {2, 0x02, 198},
  {5, 0x02, 198},
  {13, 0x02, 198},
  {27, 0x03, 198},
},
/* 132 */
{
  {3, 0x02, 195},
  {4, 0x02, 195},
  {6, 0x02, 195},
  {9, 0x02, 195},
  {14, 0x02, 195},
  {20, 0x02, 195},
  {28, 0x02, 195},
  {38, 0x03, 195},
  {3, 0x02, 196},
  {4, 0x02, 196},
  {6, 0x02, 196},
  {9, 0x02, 196},
  {14, 0x02, 196},
  {20, 0x02, 196},
  {28, 0x02, 196},
  {38, 0x03, 196},
},
/* 133 */
{
  {3, 0x02, 197},
  {4, 0x02, 197},
  {6, 0x02, 197},
  {9, 0x02, 197},
  {14, 0x02, 197},
  {20, 0x02, 197},
  {28, 0x02, 197},
  {38, 0x03, 197},
  {3, 0x02, 198},
  {4, 0x02, 198},
  {6, 0x02, 198},
  {9, 0x02, 198},
  {14, 0x02, 198},
  {20, 0x02, 198},
  {28, 0x02, 198},
  {38, 0x03, 198},
},
/* 134 */
{
  {2, 0x02, 199},
  {5, 0x02, 199},
  {13, 0x02, 199},
  {27, 0x03, 199},
  {2, 0x02, 200},
  {5, 0x02, 200},
  {13, 0x02, 200},
  {27, 0x03, 200},
  {2, 0x02, 201},
  {5, 0x02, 201},
  {13, 0x02, 201},
  {27, 0x03, 201},
  {2, 0x02, 202},
  {5, 0x02, 202},
  {13, 0x02, 202},
  {27, 0x03, 202},
},
/* 135 */
{
  {3, 0x02, 199},
  {4, 0x02, 199},
  {6, 0x02, 199},
  {9, 0x02, 199},
  {14, 0x02, 199},
  {20, 0x02, 199},
  {28, 0x02, 199},
  {38, 0x03, 199},
  {3, 0x02, 200},
  {4, 0x02, 200},
  {6, 0x02, 200},
  {9, 0x02, 200},
  {14, 0x02, 200},
  {20, 0x02, 200},
  {28, 0x02, 200},
  {38, 0x03, 200},
},
/* 136 */
{
  {3, 0x02, 201},
  {4, 0x02, 201},
  {6, 0x02, 201},
  {9, 0x02, 201},
  {14, 0x02, 201},
  {20, 0x02, 201},
  {28, 0x02, 201},
  {38, 0x03, 201},
  {3, 0x02, 202},
  {4, 0x02, 202},
  {6, 0x02, 202},
  {9, 0x02, 202},
  {14, 0x02, 202},
  {20, 0x02, 202},
  {28, 0x02, 202},
  {38, 0x03, 202},
},
/* 137 */
{
  {1, 0x02, 203},
  {12, 0x03, 203},
  {1, 0x02, 204},
  {12, 0x03, 204},
  {1, 0x02, 205},
  {12, 0x03, 205},
  {1, 0x02, 206},
  {12, 0x03, 206},
  {1, 0x02, 207},
  {12, 0x03, 207},
  {1, 0x02, 208},
  {12, 0x03, 208},
  {1, 0x02, 209},
  {12, 0x03, 209},
  {1, 0x02, 210},
  {12, 0x03, 210},
},
/* 138 */
{
  {2, 0x02, 203},
  {5, 0x02, 203},
  {13, 0x02, 203},
  {27, 0x03, 203},
  {2, 0x02, 204},
  {5, 0x02, 204},
  {13, 0x02, 204},
  {27, 0x03, 204},
  {2, 0x02, 205},
  {5, 0x02, 205},
  {13, 0x02, 205},
  {27, 0x03, 205},
  {2, 0x02, 206},
  {5, 0x02, 206},
  {13, 0x02, 206},
  {27, 0x03, 206},
},
/* 139 */
{
  {3, 0x02, 203},
  {4, 0x02, 203},
  {6, 0x02, 203},
  {9, 0x02, 203},
  {14, 0x02, 203},
  {20, 0x02, 203},
  {28, 0x02, 203},
  {38, 0x03, 203},
  {3, 0x02, 204},
  {4, 0x02, 204},
  {6, 0x02, 204},
  {9, 0x02, 204},
  {14, 0x02, 204},
  {20, 0x02, 204},
  {28, 0x02, 204},
  {38, 0x03, 204},
},
/* 140 */
{
  {3, 0x02, 205},
  {4, 0x02, 205},
  {6, 0x02, 205},
  {9, 0x02, 205},
  {14, 0x02, 205},
  {20, 0x02, 205},
  {28, 0x02, 205},
  {38, 0x03, 205},
  {3, 0x02, 206},
  {4, 0x02, 206},
  {6, 0x02, 206},
  {9, 0x02, 206},
  {14, 0x02, 206},
  {20, 0x02, 206},
  {28, 0x02, 206},
  {38, 0x03, 206},
},
/* 141 */
{
  {2, 0x02, 207},
  {5, 0x02, 207},
  {13, 0x02, 207},
  {27, 0x03, 207},
  {2, 0x02, 208},
  {5, 0x02, 208},
  {13, 0x02, 208},
  {27, 0x03, 208},
  {2, 0x02, 209},
  {5, 0x02, 209},
  {13, 0x02, 209},
  {27, 0x03, 209},
  {2, 0x02, 210},
  {5, 0x02, 210},
  {13, 0x02, 210},
  {27, 0x03, 210},
},
/* 142 */
{
  {3, 0x02, 207},
  {4, 0x02, 207},
  {6, 0x02, 207},
  {9, 0x02, 207},
  {14, 0x02, 207},
  {20, 0x02, 207},
  {28, 0x02, 207},
  {38, 0x03, 207},
  {3, 0x02, 208},
  {4, 0x02, 208},
  {6, 0x02, 208},
  {9, 0x02, 208},
  {14, 0x02, 208},
  {20, 0x02, 208},
  {28, 0x02, 208},
  {38, 0x03, 208},
},
/* 143 */
{
  {3, 0x02, 209},
  {4, 0x02, 209},
  {6, 0x02, 209},
  {9, 0x02, 209},
  {14, 0x02, 209},
  {20, 0x02, 209},
  {28, 0x02, 209},
  {38, 0x03, 209},
  {3, 0x02, 210},
  {4, 0x02, 210},
  {6, 0x02, 210},
  {9, 0x02, 210},
  {14, 0x02, 210},
  {20, 0x02, 210},
  {28, 0x02, 210},
  {38, 0x03, 210},
},
/* 144 */
{
  {0, 0x03, 211},
  {0, 0x03, 212},
  {0, 0x03, 213},
  {0, 0x03, 214},
  {0, 0x03, 215},
  {0, 0x03, 216},
  {0, 0x03, 217},
  {0, 0x03, 218},
  {0, 0x03, 219},
  {0, 0x03, 220},
  {0, 0x03, 221},
  {0, 0x03, 222},
  {0, 0x03, 223},
  {0, 0x03, 224},
  {0, 0x03, 225},
  {0, 0x03, 226},
},
/* 145 */
{
  {1, 0x02, 211},
  {12, 0x03, 211},
  {1, 0x02, 212},
  {12, 0x03, 212},
  {1, 0x02, 213},
  {12, 0x03, 213},
  {1, 0x02, 214},
  {12, 0x03, 214},
  {1, 0x02, 215},
  {12, 0x03, 215},
  {1, 0x02, 216},
  {12, 0x03, 216},
  {1, 0x02, 217},
  {12, 0x03, 217},
  {1, 0x02, 218},
  {12, 0x03, 218},
},
/* 146 */
{
  {2, 0x02, 211},
  {5, 0x02, 211},
  {13, 0x02, 211},
  {27, 0x03, 211},
  {2, 0x02, 212},
  {5, 0x02, 212},
  {13, 0x02, 212},
  {27, 0x03, 212},
  {2, 0x02, 213},
  {5, 0x02, 213},
  {13, 0x02, 213},
  {27, 0x03, 213},
  {2, 0x02, 214},
  {5, 0x02, 214},
  {13, 0x02, 214},
  {27, 0x03, 214},
},
/* 147 */
{
  {3, 0x02, 211},
  {4, 0x02, 211},
  {6, 0x02, 211},
  {9, 0x02, 211},
  {14, 0x02, 211},
  {20, 0x02, 211},
  {28, 0x02, 211},
  {38, 0x03, 211},
  {3, 0x02, 212},
  {4, 0x02, 212},
  {6, 0x02, 212},
  {9, 0x02, 212},
  {14, 0x02, 212},
  {20, 0x02, 212},
  {28, 0x02, 212},
  {38, 0x03, 212},
},
/* 148 */
{
  {3, 0x02, 213},
  {4, 0x02, 213},
  {6, 0x02, 213},
  {9, 0x02, 213},
  {14, 0x02, 213},
  {20, 0x02, 213},
  {28, 0x02, 213},
  {38, 0x03, 213},
  {3, 0x02, 214},
  {4, 0x02, 214},
  {6, 0x02, 214},
  {9, 0x02, 214},
  {14, 0x02, 214},
  {20, 0x02, 214},
  {28, 0x02, 214},
  {38, 0x03, 214},
},
/* 149 */
{
  {2, 0x02, 215},
  {5, 0x02, 215},
  {13, 0x02, 215},
  {27, 0x03, 215},
  {2, 0x02, 216},
  {5, 0x02, 216},
  {13, 0x02, 216},
  {27, 0x03, 216},
  {2, 0x02, 217},
  {5, 0x02, 217},
  {13, 0x02, 217},
  {27, 0x03, 217},
  {2, 0x02, 218},
  {5, 0x02, 218},
  {13, 0x02, 218},
  {27, 0x03, 218},
},
/* 150 */
{
  {3, 0x02, 215},
  {4, 0x02, 215},
  {6, 0x02, 215},
  {9, 0x02, 215},
  {14, 0x02, 215},
  {20, 0x02, 215},
  {28, 0x02, 215},
  {38, 0x03, 215},
  {3, 0x02, 216},
  {4, 0x02, 216},
  {6, 0x02, 216},
  {9, 0x02, 216},
  {14, 0x02, 216},
  {20, 0x02, 216},
  {28, 0x02, 216},
  {38, 0x03, 216},
},
/* 151 */
{
  {3, 0x02, 217},
  {4, 0x02, 217},
  {6, 0x02, 217},
  {9, 0x02, 217},
  {14, 0x02, 217},
  {20, 0x02, 217},
  {28, 0x02, 217},
  {38, 0x03, 217},
  {3, 0x02, 218},
  {4, 0x02, 218},
  {6, 0x02, 218},
  {9, 0x02, 218},
  {14, 0x02, 218},
  {20, 0x02, 218},
  {28, 0x02, 218},
  {38, 0x03, 218},
},
/* 152 */
{
  {1, 0x02, 219},
  {12, 0x03, 219},
  {1, 0x02, 220},
  {12, 0x03, 220},
  {1, 0x02, 221},
  {12, 0x03, 221},
  {1, 0x02, 222},
  {12, 0x03, 222},
  {1, 0x02, 223},
  {12, 0x03, 223},
  {1, 0x02, 224},
  {12, 0x03, 224},
  {1, 0x02, 225},
  {12, 0x03, 225},
  {1, 0x02, 226},
  {12, 0x03, 226},
},
/* 153 */
{
  {2, 0x02, 219},
  {5, 0x02, 219},
  {13, 0x02, 219},
  {27, 0x03, 219},
  {2, 0x02, 220},
  {5, 0x02, 220},
  {13, 0x02, 220},
  {27, 0x03, 220},
  {2, 0x02, 221},
  {5, 0x02, 221},
  {13, 0x02, 221},
  {27, 0x03, 221},
  {2, 0x02, 222},
  {5, 0x02, 222},
  {13, 0x02, 222},
  {27, 0x03, 222},
},
/* 154 */
{
  {3, 0x02, 219},
  {4, 0x02, 219},
  {6, 0x02, 219},
  {9, 0x02, 219},
  {14, 0x02, 219},
  {20, 0x02, 219},
  {28, 0x02, 219},
  {38, 0x03, 219},
  {3, 0x02, 220},
  {4, 0x02, 220},
  {6, 0x02, 220},
  {9, 0x02, 220},
  {14, 0x02, 220},
  {20, 0x02, 220},
  {28, 0x02, 220},
  {38, 0x03, 220},
},
/* 155 */
{
  {3, 0x02, 221},
  {4, 0x02, 221},
  {6, 0x02, 221},
  {9, 0x02, 221},
  {14, 0x02, 221},
  {20, 0x02, 221},
  {28, 0x02, 221},
  {38, 0x03, 221},
  {3, 0x02, 222},
  {4, 0x02, 222},
  {6, 0x02, 222},
  {9, 0x02, 222},
  {14, 0x02, 222},
  {20, 0x02, 222},
  {28, 0x02, 222},
  {38, 0x03, 222},
},
/* 156 */
{
  {2, 0x02, 223},
  {5, 0x02, 223},
  {13, 0x02, 223},
  {27, 0x03, 223},
  {2, 0x02, 224},
  {5, 0x02, 224},
  {13, 0x02, 224},
  {27, 0x03, 224},
  {2, 0x02, 225},
  {5, 0x02, 225},
  {13, 0x02, 225},
  {27, 0x03, 225},
  {2, 0x02, 226},
  {5, 0x02, 226},
  {13, 0x02, 226},
  {27, 0x03, 226},
},
/* 157 */
{
  {3, 0x02, 223},
  {4, 0x02, 223},
  {6, 0x02, 223},
  {9, 0x02, 223},
  {14, 0x02, 223},
  {20, 0x02, 223},
  {28, 0x02, 223},
  {38, 0x03, 223},
  {3, 0x02, 224},
  {4, 0x02, 224},
  {6, 0x02, 224},
  {9, 0x02, 224},
  {14, 0x02, 224},
  {20, 0x02, 224},
  {28, 0x02, 224},
  {38, 0x03, 224},
},
/* 158 */
{
  {3, 0x02, 225},
  {4, 0x02, 225},
  {6, 0x02, 225},
  {9, 0x02, 225},
  {14, 0x02, 225},
  {20, 0x02, 225},
  {28, 0x02, 225},
  {38, 0x03, 225},
  {3, 0x02, 226},
  {4, 0x02, 226},
  {6, 0x02, 226},
  {9, 0x02, 226},
  {14, 0x02, 226},
  {20, 0x02, 226},
  {28, 0x02, 226},
  {38, 0x03, 226},
},
/* 159 */
{
  {163, 0x00, 0},
  {166, 0x00, 0},
  {170, 0x00, 0},
  {173, 0x00, 0},
  {178, 0x00, 0},
  {181, 0x00, 0},
  {185, 0x00, 0},
  {188, 0x00, 0},
  {196, 0x00, 0},
  {203, 0x00, 0},
  {211, 0x00, 0},
  {218, 0x00, 0},
  {227, 0x00, 0},
  {234, 0x00, 0},
  {242, 0x00, 0},
  {249, 0x00, 0},
},
/* 160 */
{
  {164, 0x00, 0},
  {165, 0x00, 0},
  {167, 0x00, 0},
  {168, 0x00, 0},
  {171, 0x00, 0},
  {172, 0x00, 0},
  {174, 0x00, 0},
  {175, 0x00, 0},
  {179, 0x00, 0},
  {180, 0x00, 0},
  {182, 0x00, 0},
  {183, 0x00, 0},
  {186, 0x00, 0},
  {187, 0x00, 0},
  {189, 0x00, 0},
  {190, 0x00, 0},
},
/* 161 */
{
  {0, 0x03, 227},
  {0, 0x03, 228},
  {0, 0x03, 229},
  {0, 0x03, 230},
  {0, 0x03, 231},
  {0, 0x03, 232},
  {0, 0x03, 233},
  {0, 0x03, 234},
  {0, 0x03, 235},
  {0, 0x03, 236},
  {0, 0x03, 237},
  {0, 0x03, 238},
  {0, 0x03, 239},
  {0, 0x03, 240},
  {0, 0x03, 241},
  {0, 0x03, 242},
},
/* 162 */
{
  {1, 0x02, 227},
  {12, 0x03, 227},
  {1, 0x02, 228},
  {12, 0x03, 228},
  {1, 0x02, 229},
  {12, 0x03, 229},
  {1, 0x02, 230},
  {12, 0x03, 230},
  {1, 0x02, 231},
  {12, 0x03, 231},
  {1, 0x02, 232},
  {12, 0x03, 232},
  {1, 0x02, 233},
  {12, 0x03, 233},
  {1, 0x02, 234},
  {12, 0x03, 234},
},
/* 163 */
{
  {2, 0x02, 227},
  {5, 0x02, 227},
  {13, 0x02, 227},
  {27, 0x03, 227},
  {2, 0x02, 228},
  {5, 0x02, 228},
  {13, 0x02, 228},
  {27, 0x03, 228},
  {2, 0x02, 229},
  {5, 0x02, 229},
  {13, 0x02, 229},
  {27, 0x03, 229},
  {2, 0x02, 230},
  {5, 0x02, 230},
  {13, 0x02, 230},
  {27, 0x03, 230},
},
/* 164 */
{
  {3, 0x02, 227},
  {4, 0x02, 227},
  {6, 0x02, 227},
  {9, 0x02, 227},
  {14, 0x02, 227},
  {20, 0x02, 227},
  {28, 0x02, 227},
  {38, 0x03, 227},
  {3, 0x02, 228},
  {4, 0x02, 228},
  {6, 0x02, 228},
  {9, 0x02, 228},
  {14, 0x02, 228},
  {20, 0x02, 228},
  {28, 0x02, 228},
  {38, 0x03, 228},
},
/* 165 */
{
  {3, 0x02, 229},
  {4, 0x02, 229},
  {6, 0x02, 229},
  {9, 0x02, 229},
  {14, 0x02, 229},
  {20, 0x02, 229},
  {28, 0x02, 229},
  {38, 0x03, 229},
  {3, 0x02, 230},
  {4, 0x02, 230},
  {6, 0x02, 230},
  {9, 0x02, 230},
  {14, 0x02, 230},
  {20, 0x02, 230},
  {28, 0x02, 230},
  {38, 0x03, 230},
},
/* 166 */
{
  {2, 0x02, 231},
  {5, 0x02, 231},
  {13, 0x02, 231},
  {27, 0x03, 231},
  {2, 0x02, 232},
  {5, 0x02, 232},
  {13, 0x02, 232},
  {27, 0x03, 232},
  {2, 0x02, 233},
  {5, 0x02, 233},
  {13, 0x02, 233},
  {27, 0x03, 233},
  {2, 0x02, 234},
  {5, 0x02, 234},
  {13, 0x02, 234},
  {27, 0x03, 234},
},
/* 167 */
{
  {3, 0x02, 231},
  {4, 0x02, 231},
  {6, 0x02, 231},
  {9, 0x02, 231},
  {14, 0x02, 231},
  {20, 0x02, 231},
  {28, 0x02, 231},
  {38, 0x03, 231},
  {3, 0x02, 232},
  {4, 0x02, 232},
  {6, 0x02, 232},
  {9, 0x02, 232},
  {14, 0x02, 232},
  {20, 0x02, 232},
  {28, 0x02, 232},
  {38, 0x03, 232},
},
/* 168 */
{
  {3, 0x02, 233},
  {4, 0x02, 233},
  {6, 0x02, 233},
  {9, 0x02, 233},
  {14, 0x02, 233},
  {20, 0x02, 233},
  {28, 0x02, 233},
  {38, 0x03, 233},
  {3, 0x02, 234},
  {4, 0x02, 234},
  {6, 0x02, 234},
  {9, 0x02, 234},
  {14, 0x02, 234},
  {20, 0x02, 234},
  {28, 0x02, 234},
  {38, 0x03, 234},
},
/* 169 */
{
  {1, 0x02, 235},
  {12, 0x03, 235},
  {1, 0x02, 236},
  {12, 0x03, 236},
  {1, 0x02, 237},
  {12, 0x03, 237},
  {1, 0x02, 238},
  {12, 0x03, 238},
  {1, 0x02, 239},
  {12, 0x03, 239},
  {1, 0x02, 240},
  {12, 0x03, 240},
  {1, 0x02, 241},
  {12, 0x03, 241},
  {1, 0x02, 242},
  {12, 0x03, 242},
},
/* 170 */
{
  {2, 0x02, 235},
  {5, 0x02, 235},
  {13, 0x02, 235},
  {27, 0x03, 235},
  {2, 0x02, 236},
  {5, 0x02, 236},
  {13, 0x02, 236},
  {27, 0x03, 236},
  {2, 0x02, 237},
  {5, 0x02, 237},
  {13, 0x02, 237},
  {27, 0x03, 237},
  {2, 0x02, 238},
  {5, 0x02, 238},
  {13, 0x02, 238},
  {27, 0x03, 238},
},
/* 171 */
{
  {3, 0x02, 235},
  {4, 0x02, 235},
  {6, 0x02, 235},
  {9, 0x02, 235},
  {14, 0x02, 235},
  {20, 0x02, 235},
  {28, 0x02, 235},
  {38, 0x03, 235},
  {3, 0x02, 236},
  {4, 0x02, 236},
  {6, 0x02, 236},
  {9, 0x02, 236},
  {14, 0x02, 236},
  {20, 0x02, 236},
  {28, 0x02, 236},
  {38, 0x03, 236},
},
/* 172 */
{
  {3, 0x02, 237},
  {4, 0x02, 237},
  {6, 0x02, 237},
  {9, 0x02, 237},
  {14, 0x02, 237},
  {20, 0x02, 237},
  {28, 0x02, 237},
  {38, 0x03, 237},
  {3, 0x02, 238},
  {4, 0x02, 238},
  {6, 0x02, 238},
  {9, 0x02, 238},
  {14, 0x02, 238},
  {20, 0x02, 238},
  {28, 0x02, 238},
  {38, 0x03, 238},
},
/* 173 */
{
  {2, 0x02, 239},
  {5, 0x02, 239},
  {13, 0x02, 239},
  {27, 0x03, 239},
  {2, 0x02, 240},
  {5, 0x02, 240},
  {13, 0x02, 240},
  {27, 0x03, 240},
  {2, 0x02, 241},
  {5, 0x02, 241},
  {13, 0x02, 241},
  {27, 0x03, 241},
  {2, 0x02, 242},
  {5, 0x02, 242},
  {13, 0x02, 242},
  {27, 0x03, 242},
},
/* 174 */
{
  {3, 0x02, 239},
  {4, 0x02, 239},
  {6, 0x02, 239},
  {9, 0x02, 239},
  {14, 0x02, 239},
  {20, 0x02, 239},
  {28, 0x02, 239},
  {38, 0x03, 239},
  {3, 0x02, 240},
  {4, 0x02, 240},
  {6, 0x02, 240},
  {9, 0x02, 240},
  {14, 0x02, 240},
  {20, 0x02, 240},
  {28, 0x02, 240},
  {38, 0x03, 240},
},
/* 175 */
{
  {3, 0x02, 241},
  {4, 0x02, 241},
  {6, 0x02, 241},
  {9, 0x02, 241},
  {14, 0x02, 241},
  {20, 0x02, 241},
  {28, 0x02, 241},
  {38, 0x03, 241},
  {3, 0x02, 242},
  {4, 0x02, 242},
  {6, 0x02, 242},
  {9, 0x02, 242},
  {14, 0x02, 242},
  {20, 0x02, 242},
  {28, 0x02, 242},
  {38, 0x03, 242},
},
/* 176 */
{
  {0, 0x03, 243},
  {0, 0x03, 244},
  {0, 0x03, 245},
  {0, 0x03, 246},
  {0, 0x03, 247},
  {0, 0x03, 248},
  {0, 0x03, 249},
  {0, 0x03, 250},
  {0, 0x03, 251},
  {0, 0x03, 252},
  {0, 0x03, 253},
  {0, 0x03, 254},
  {0, 0x03, 255},
  {-1, 0x00, 0},
  {191, 0x00, 0},
  {192, 0x00, 0},
},
/* 177 */
{
  {1, 0x02, 243},
  {12, 0x03, 243},
  {1, 0x02, 244},
  {12, 0x03, 244},
  {1, 0x02, 245},
  {12, 0x03, 245},
  {1, 0x02, 246},
  {12, 0x03, 246},
  {1, 0x02, 247},
  {12, 0x03, 247},
  {1, 0x02, 248},
  {12, 0x03, 248},
  {1, 0x02, 249},
  {12, 0x03, 249},
  {1, 0x02, 250},
  {12, 0x03, 250},
},
/* 178 */
{
  {2, 0x02, 243},
  {5, 0x02, 243},
  {13, 0x02, 243},
  {27, 0x03, 243},
  {2, 0x02, 244},
  {5, 0x02, 244},
  {13, 0x02, 244},
  {27, 0x03, 244},
  {2, 0x02, 245},
  {5, 0x02, 245},
  {13, 0x02, 245},
  {27, 0x03, 245},
  {2, 0x02, 246},
  {5, 0x02, 246},
  {13, 0x02, 246},
  {27, 0x03, 246},
},
/* 179 */
{
  {3, 0x02, 243},
  {4, 0x02, 243},
  {6, 0x02, 243},
  {9, 0x02, 243},
  {14, 0x02, 243},
  {20, 0x02, 243},
  {28, 0x02, 243},
  {38, 0x03, 243},
  {3, 0x02, 244},
  {4, 0x02, 244},
  {6, 0x02, 244},
  {9, 0x02, 244},
  {14, 0x02, 244},
  {20, 0x02, 244},
  {28, 0x02, 244},
  {38, 0x03, 244},
},
/* 180 */
{
  {3, 0x02, 245},
  {4, 0x02, 245},
  {6, 0x02, 245},
  {9, 0x02, 245},
  {14, 0x02, 245},
  {20, 0x02, 245},
  {28, 0x02, 245},
  {38, 0x03, 245},
  {3, 0x02, 246},
  {4, 0x02, 246},
  {6, 0x02, 246},
  {9, 0x02, 246},
  {14, 0x02, 246},
  {20, 0x02, 246},
  {28, 0x02, 246},
  {38, 0x03, 246},
},
/* 181 */
{
  {2, 0x02, 247},
  {5, 0x02, 247},
  {13, 0x02, 247},
  {27, 0x03, 247},
  {2, 0x02, 248},
  {5, 0x02, 248},
  {13, 0x02, 248},
  {27, 0x03, 248},
  {2, 0x02, 249},
  {5, 0x02, 249},
  {13, 0x02, 249},
  {27, 0x03, 249},
  {2, 0x02, 250},
  {5, 0x02, 250},
  {13, 0x02, 250},
  {27, 0x03, 250},
},
/* 182 */
{
  {3, 0x02, 247},
  {4, 0x02, 247},
  {6, 0x02, 247},
  {9, 0x02, 247},
  {14, 0x02, 247},
  {20, 0x02, 247},
  {28, 0x02, 247},
  {38, 0x03, 247},
  {3, 0x02, 248},
  {4, 0x02, 248},
  {6, 0x02, 248},
  {9, 0x02, 248},
  {14, 0x02, 248},
  {20, 0x02, 248},
  {28, 0x02, 248},
  {38, 0x03, 248},
},
/* 183 */
{
  {3, 0x02, 249},
  {4, 0x02, 249},
  {6, 0x02, 249},
  {9, 0x02, 249},
  {14, 0x02, 249},
  {20, 0x02, 249},
  {28, 0x02, 249},
  {38, 0x03, 249},
  {3, 0x02, 250},
  {4, 0x02, 250},
  {6, 0x02, 250},
  {9, 0x02, 250},
  {14, 0x02, 250},
  {20, 0x02, 250},
  {28, 0x02, 250},
  {38, 0x03, 250},
},
/* 184 */
{
  {1, 0x02, 251},
  {12, 0x03, 251},
  {1, 0x02, 252},
  {12, 0x03, 252},
  {1, 0x02, 253},
  {12, 0x03, 253},
  {1, 0x02, 254},
  {12, 0x03, 254},
  {1, 0x02, 255},
  {12, 0x03, 255},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {0, 0x03, 0},
  {0, 0x03, 1},
  {0, 0x03, 2},
  {0, 0x03, 3},
},
/* 185 */
{
  {2, 0x02, 251},
  {5, 0x02, 251},
  {13, 0x02, 251},
  {27, 0x03, 251},
  {2, 0x02, 252},
  {5, 0x02, 252},
  {13, 0x02, 252},
  {27, 0x03, 252},
  {2, 0x02, 253},
  {5, 0x02, 253},
  {13, 0x02, 253},
  {27, 0x03, 253},
  {2, 0x02, 254},
  {5, 0x02, 254},
  {13, 0x02, 254},
  {27, 0x03, 254},
},
/* 186 */
{
  {3, 0x02, 251},
  {4, 0x02, 251},
  {6, 0x02, 251},
  {9, 0x02, 251},
  {14, 0x02, 251},
  {20, 0x02, 251},
  {28, 0x02, 251},
  {38, 0x03, 251},
  {3, 0x02, 252},
  {4, 0x02, 252},
  {6, 0x02, 252},
  {9, 0x02, 252},
  {14, 0x02, 252},
  {20, 0x02, 252},
  {28, 0x02, 252},
  {38, 0x03, 252},
},
/* 187 */
{
  {3, 0x02, 253},
  {4, 0x02, 253},
  {6, 0x02, 253},
  {9, 0x02, 253},
  {14, 0x02, 253},
  {20, 0x02, 253},
  {28, 0x02, 253},
  {38, 0x03, 253},
  {3, 0x02, 254},
  {4, 0x02, 254},
  {6, 0x02, 254},
  {9, 0x02, 254},
  {14, 0x02, 254},
  {20, 0x02, 254},
  {28, 0x02, 254},
  {38, 0x03, 254},
},
/* 188 */
{
  {2, 0x02, 255},
  {5, 0x02, 255},
  {13, 0x02, 255},
  {27, 0x03, 255},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {1, 0x02, 0},
  {12, 0x03, 0},
  {1, 0x02, 1},
  {12, 0x03, 1},
  {1, 0x02, 2},
  {12, 0x03, 2},
  {1, 0x02, 3},
  {12, 0x03, 3},
},
/* 189 */
{
  {3, 0x02, 255},
  {4, 0x02, 255},
  {6, 0x02, 255},
  {9, 0x02, 255},
  {14, 0x02, 255},
  {20, 0x02, 255},
  {28, 0x02, 255},
  {38, 0x03, 255},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
  {-1, 0x00, 0},
},
/* 190 */
{
  {2, 0x02, 0},
  {5, 0x02, 0},
  {13, 0x02, 0},
  {27, 0x03, 0},
  {2, 0x02, 1},
  {5, 0x02, 1},
  {13, 0x02, 1},
  {27, 0x03, 1},
  {2, 0x02, 2},
  {5, 0x02, 2},
  {13, 0x02, 2},
  {27, 0x03, 2},
  {2, 0x02, 3},
  {5, 0x02, 3},
  {13, 0x02, 3},
  {27, 0x03, 3},
},
/* 191 */
{
  {3, 0x02, 0},
  {4, 0x02, 0},
  {6, 0x02, 0},
  {9, 0x02, 0},
  {14, 0x02, 0},
  {20, 0x02, 0},
  {28, 0x02, 0},
  {38, 0x03, 0},
  {3, 0x02, 1},
  {4, 0x02, 1},
  {6, 0x02, 1},
  {9, 0x02, 1},
  {14, 0x02, 1},
  {20, 0x02, 1},
  {28, 0x02, 1},
  {38, 0x03, 1},
},
/* 192 */
{
  {3, 0x02, 2},
  {4, 0x02, 2},
  {6, 0x02, 2},
  {9, 0x02, 2},
  {14, 0x02, 2},
  {20, 0x02, 2},
  {28, 0x02, 2},
  {38, 0x03, 2},
  {3, 0x02, 3},
  {4, 0x02, 3},
  {6, 0x02, 3},
  {9, 0x02, 3},
  {14, 0x02, 3},
  {20, 0x02, 3},
  {28, 0x02, 3},
  {38, 0x03, 3},
},
/* 193 */
{
  {197, 0x00, 0},
  {200, 0x00, 0},
  {204, 0x00, 0},
  {207, 0x00, 0},
  {212, 0x00, 0},
  {215, 0x00, 0},
  {219, 0x00, 0},
  {222, 0x00, 0},
  {228, 0x00, 0},
  {231, 0x00, 0},
  {235, 0x00, 0},
  {238, 0x00, 0},
  {243, 0x00, 0},
  {246, 0x00, 0},
  {250, 0x00, 0},
  {253, 0x00, 0},
},
/* 194 */
{
  {198, 0x00, 0},
  {199, 0x00, 0},
  {201, 0x00, 0},
  {202, 0x00, 0},
  {205, 0x00, 0},
  {206, 0x00, 0},
  {208, 0x00, 0},
  {209, 0x00, 0},
  {213, 0x00, 0},
  {214, 0x00, 0},
  {216, 0x00, 0},
  {217, 0x00, 0},
  {220, 0x00, 0},
  {221, 0x00, 0},
  {223, 0x00, 0},
  {224, 0x00, 0},
},
/* 195 */
{
  {0, 0x03, 4},
  {0, 0x03, 5},
  {0, 0x03, 6},
  {0, 0x03, 7},
  {0, 0x03, 8},
  {0, 0x03, 9},
  {0, 0x03, 10},
  {0, 0x03, 11},
  {0, 0x03, 12},
  {0, 0x03, 13},
  {0, 0x03, 14},
  {0, 0x03, 15},
  {0, 0x03, 16},
  {0, 0x03, 17},
  {0, 0x03, 18},
  {0, 0x03, 19},
},
/* 196 */
{
  {1, 0x02, 4},
  {12, 0x03, 4},
  {1, 0x02, 5},
  {12, 0x03, 5},
  {1, 0x02, 6},
  {12, 0x03, 6},
  {1, 0x02, 7},
  {12, 0x03, 7},
  {1, 0x02, 8},
  {12, 0x03, 8},
  {1, 0x02, 9},
  {12, 0x03, 9},
  {1, 0x02, 10},
  {12, 0x03, 10},
  {1, 0x02, 11},
  {12, 0x03, 11},
},
/* 197 */
{
  {2, 0x02, 4},
  {5, 0x02, 4},
  {13, 0x02, 4},
  {27, 0x03, 4},
  {2, 0x02, 5},
  {5, 0x02, 5},
  {13, 0x02, 5},
  {27, 0x03, 5},
  {2, 0x02, 6},
  {5, 0x02, 6},
  {13, 0x02, 6},
  {27, 0x03, 6},
  {2, 0x02, 7},
  {5, 0x02, 7},
  {13, 0x02, 7},
  {27, 0x03, 7},
},
/* 198 */
{
  {3, 0x02, 4},
  {4, 0x02, 4},
  {6, 0x02, 4},
  {9, 0x02, 4},
  {14, 0x02, 4},
  {20, 0x02, 4},
  {28, 0x02, 4},
  {38, 0x03, 4},
  {3, 0x02, 5},
  {4, 0x02, 5},
  {6, 0x02, 5},
  {9, 0x02, 5},
  {14, 0x02, 5},
  {20, 0x02, 5},
  {28, 0x02, 5},
  {38, 0x03, 5},
},
/* 199 */
{
  {3, 0x02, 6},
  {4, 0x02, 6},
  {6, 0x02, 6},
  {9, 0x02, 6},
  {14, 0x02, 6},
  {20, 0x02, 6},
  {28, 0x02, 6},
  {38, 0x03, 6},
  {3, 0x02, 7},
  {4, 0x02, 7},
  {6, 0x02, 7},
  {9, 0x02, 7},
  {14, 0x02, 7},
  {20, 0x02, 7},
  {28, 0x02, 7},
  {38, 0x03, 7},
},
/* 200 */
{
  {2, 0x02, 8},
  {5, 0x02, 8},
  {13, 0x02, 8},
  {27, 0x03, 8},
  {2, 0x02, 9},
  {5, 0x02, 9},
  {13, 0x02, 9},
  {27, 0x03, 9},
  {2, 0x02, 10},
  {5, 0x02, 10},
  {13, 0x02, 10},
  {27, 0x03, 10},
  {2, 0x02, 11},
  {5, 0x02, 11},
  {13, 0x02, 11},
  {27, 0x03, 11},
},
/* 201 */
{
  {3, 0x02, 8},
  {4, 0x02, 8},
  {6, 0x02, 8},
  {9, 0x02, 8},
  {14, 0x02, 8},
  {20, 0x02, 8},
  {28, 0x02, 8},
  {38, 0x03, 8},
  {3, 0x02, 9},
  {4, 0x02, 9},
  {6, 0x02, 9},
  {9, 0x02, 9},
  {14, 0x02, 9},
  {20, 0x02, 9},
  {28, 0x02, 9},
  {38, 0x03, 9},
},
/* 202 */
{
  {3, 0x02, 10},
  {4, 0x02, 10},
  {6, 0x02, 10},
  {9, 0x02, 10},
  {14, 0x02, 10},
  {20, 0x02, 10},
  {28, 0x02, 10},
  {38, 0x03, 10},
  {3, 0x02, 11},
  {4, 0x02, 11},
  {6, 0x02, 11},
  {9, 0x02, 11},
  {14, 0x02, 11},
  {20, 0x02, 11},
  {28, 0x02, 11},
  {38, 0x03, 11},
},
/* 203 */
{
  {1, 0x02, 12},
  {12, 0x03, 12},
  {1, 0x02, 13},
  {12, 0x03, 13},
  {1, 0x02, 14},
  {12, 0x03, 14},
  {1, 0x02, 15},
  {12, 0x03, 15},
  {1, 0x02, 16},
  {12, 0x03, 16},
  {1, 0x02, 17},
  {12, 0x03, 17},
  {1, 0x02, 18},
  {12, 0x03, 18},
  {1, 0x02, 19},
  {12, 0x03, 19},
},
/* 204 */
{
  {2, 0x02, 12},
  {5, 0x02, 12},
  {13, 0x02, 12},
  {27, 0x03, 12},
  {2, 0x02, 13},
  {5, 0x02, 13},
  {13, 0x02, 13},
  {27, 0x03, 13},
  {2, 0x02, 14},
  {5, 0x02, 14},
  {13, 0x02, 14},
  {27, 0x03, 14},
  {2, 0x02, 15},
  {5, 0x02, 15},
  {13, 0x02, 15},
  {27, 0x03, 15},
},
/* 205 */
{
  {3, 0x02, 12},
  {4, 0x02, 12},
  {6, 0x02, 12},
  {9, 0x02, 12},
  {14, 0x02, 12},
  {20, 0x02, 12},
  {28, 0x02, 12},
  {38, 0x03, 12},
  {3, 0x02, 13},
  {4, 0x02, 13},
  {6, 0x02, 13},
  {9, 0x02, 13},
  {14, 0x02, 13},
  {20, 0x02, 13},
  {28, 0x02, 13},
  {38, 0x03, 13},
},
/* 206 */
{
  {3, 0x02, 14},
  {4, 0x02, 14},
  {6, 0x02, 14},
  {9, 0x02, 14},
  {14, 0x02, 14},
  {20, 0x02, 14},
  {28, 0x02, 14},
  {38, 0x03, 14},
  {3, 0x02, 15},
  {4, 0x02, 15},
  {6, 0x02, 15},
  {9, 0x02, 15},
  {14, 0x02, 15},
  {20, 0x02, 15},
  {28, 0x02, 15},
  {38, 0x03, 15},
},
/* 207 */
{
  {2, 0x02, 16},
  {5, 0x02, 16},
  {13, 0x02, 16},
  {27, 0x03, 16},
  {2, 0x02, 17},
  {5, 0x02, 17},
  {13, 0x02, 17},
  {27, 0x03, 17},
  {2, 0x02, 18},
  {5, 0x02, 18},
  {13, 0x02, 18},
  {27, 0x03, 18},
  {2, 0x02, 19},
  {5, 0x02, 19},
  {13, 0x02, 19},
  {27, 0x03, 19},
},
/* 208 */
{
  {3, 0x02, 16},
  {4, 0x02, 16},
  {6, 0x02, 16},
  {9, 0x02, 16},
  {14, 0x02, 16},
  {20, 0x02, 16},
  {28, 0x02, 16},
  {38, 0x03, 16},
  {3, 0x02, 17},
  {4, 0x02, 17},
  {6, 0x02, 17},
  {9, 0x02, 17},
  {14, 0x02, 17},
  {20, 0x02, 17},
  {28, 0x02, 17},
  {38, 0x03, 17},
},
/* 209 */
{
  {3, 0x02, 18},
  {4, 0x02, 18},
  {6, 0x02, 18},
  {9, 0x02, 18},
  {14, 0x02, 18},
  {20, 0x02, 18},
  {28, 0x02, 18},
  {38, 0x03, 18},
  {3, 0x02, 19},
  {4, 0x02, 19},
  {6, 0x02, 19},
  {9, 0x02, 19},
  {14, 0x02, 19},
  {20, 0x02, 19},
  {28, 0x02, 19},
  {38, 0x03, 19},
},
/* 210 */
{
  {0, 0x03, 20},
  {0, 0x03, 21},
  {0, 0x03, 22},
  {0, 0x03, 23},
  {0, 0x03, 24},
  {0, 0x03, 25},
  {0, 0x03, 26},
  {0, 0x03, 27},
  {0, 0x03, 28},
  {0, 0x03, 29},
  {0, 0x03, 30},
  {0, 0x03, 31},
  {0, 0x03, 127},
  {0, 0x03, 128},
  {0, 0x03, 129},
  {0, 0x03, 130},
},
/* 211 */
{
  {1, 0x02, 20},
  {12, 0x03, 20},
  {1, 0x02, 21},
  {12, 0x03, 21},
  {1, 0x02, 22},
  {12, 0x03, 22},
  {1, 0x02, 23},
  {12, 0x03, 23},
  {1, 0x02, 24},
  {12, 0x03, 24},
  {1, 0x02, 25},
  {12, 0x03, 25},
  {1, 0x02, 26},
  {12, 0x03, 26},
  {1, 0x02, 27},
  {12, 0x03, 27},
},
/* 212 */
{
  {2, 0x02, 20},
  {5, 0x02, 20},
  {13, 0x02, 20},
  {27, 0x03, 20},
  {2, 0x02, 21},
  {5, 0x02, 21},
  {13, 0x02, 21},
  {27, 0x03, 21},
  {2, 0x02, 22},
  {5, 0x02, 22},
  {13, 0x02, 22},
  {27, 0x03, 22},
  {2, 0x02, 23},
  {5, 0x02, 23},
  {13, 0x02, 23},
  {27, 0x03, 23},
},
/* 213 */
{
  {3, 0x02, 20},
  {4, 0x02, 20},
  {6, 0x02, 20},
  {9, 0x02, 20},
  {14, 0x02, 20},
  {20, 0x02, 20},
  {28, 0x02, 20},
  {38, 0x03, 20},
  {3, 0x02, 21},
  {4, 0x02, 21},
  {6, 0x02, 21},
  {9, 0x02, 21},
  {14, 0x02, 21},
  {20, 0x02, 21},
  {28, 0x02, 21},
  {38, 0x03, 21},
},
/* 214 */
{
  {3, 0x02, 22},
  {4, 0x02, 22},
  {6, 0x02, 22},
  {9, 0x02, 22},
  {14, 0x02, 22},
  {20, 0x02, 22},
  {28, 0x02, 22},
  {38, 0x03, 22},
  {3, 0x02, 23},
  {4, 0x02, 23},
  {6, 0x02, 23},
  {9, 0x02, 23},
  {14, 0x02, 23},
  {20, 0x02, 23},
  {28, 0x02, 23},
  {38, 0x03, 23},
},
/* 215 */
{
  {2, 0x02, 24},
  {5, 0x02, 24},
  {13, 0x02, 24},
  {27, 0x03, 24},
  {2, 0x02, 25},
  {5, 0x02, 25},
  {13, 0x02, 25},
  {27, 0x03, 25},
  {2, 0x02, 26},
  {5, 0x02, 26},
  {13, 0x02, 26},
  {27, 0x03, 26},
  {2, 0x02, 27},
  {5, 0x02, 27},
  {13, 0x02, 27},
  {27, 0x03, 27},
},
/* 216 */
{
  {3, 0x02, 24},
  {4, 0x02, 24},
  {6, 0x02, 24},
  {9, 0x02, 24},
  {14, 0x02, 24},
  {20, 0x02, 24},
  {28, 0x02, 24},
  {38, 0x03, 24},
  {3, 0x02, 25},
  {4, 0x02, 25},
  {6, 0x02, 25},
  {9, 0x02, 25},
  {14, 0x02, 25},
  {20, 0x02, 25},
  {28, 0x02, 25},
  {38, 0x03, 25},
},
/* 217 */
{
  {3, 0x02, 26},
  {4, 0x02, 26},
  {6, 0x02, 26},
  {9, 0x02, 26},
  {14, 0x02, 26},
  {20, 0x02, 26},
  {28, 0x02, 26},
  {38, 0x03, 26},
  {3, 0x02, 27},
  {4, 0x02, 27},
  {6, 0x02, 27},
  {9, 0x02, 27},
  {14, 0x02, 27},
  {20, 0x02, 27},
  {28, 0x02, 27},
  {38, 0x03, 27},
},
/* 218 */
{
  {1, 0x02, 28},
  {12, 0x03, 28},
  {1, 0x02, 29},
  {12, 0x03, 29},
  {1, 0x02, 30},
  {12, 0x03, 30},
  {1, 0x02, 31},
  {12, 0x03, 31},
  {1, 0x02, 127},
  {12, 0x03, 127},
  {1, 0x02, 128},
  {12, 0x03, 128},
  {1, 0x02, 129},
  {12, 0x03, 129},
  {1, 0x02, 130},
  {12, 0x03, 130},
},
/* 219 */
{
  {2, 0x02, 28},
  {5, 0x02, 28},
  {13, 0x02, 28},
  {27, 0x03, 28},
  {2, 0x02, 29},
  {5, 0x02, 29},
  {13, 0x02, 29},
  {27, 0x03, 29},
  {2, 0x02, 30},
  {5, 0x02, 30},
  {13, 0x02, 30},
  {27, 0x03, 30},
  {2, 0x02, 31},
  {5, 0x02, 31},
  {13, 0x02, 31},
  {27, 0x03, 31},
},
/* 220 */
{
  {3, 0x02, 28},
  {4, 0x02, 28},
  {6, 0x02, 28},
  {9, 0x02, 28},
  {14, 0x02, 28},
  {20, 0x02, 28},
  {28, 0x02, 28},
  {38, 0x03, 28},
  {3, 0x02, 29},
  {4, 0x02, 29},
  {6, 0x02, 29},
  {9, 0x02, 29},
  {14, 0x02, 29},
  {20, 0x02, 29},
  {28, 0x02, 29},
  {38, 0x03, 29},
},
/* 221 */
{
  {3, 0x02, 30},
  {4, 0x02, 30},
  {6, 0x02, 30},
  {9, 0x02, 30},
  {14, 0x02, 30},
  {20, 0x02, 30},
  {28, 0x02, 30},
  {38, 0x03, 30},
  {3, 0x02, 31},
  {4, 0x02, 31},
  {6, 0x02, 31},
  {9, 0x02, 31},
  {14, 0x02, 31},
  {20, 0x02, 31},
  {28, 0x02, 31},
  {38, 0x03, 31},
},
/* 222 */
{
  {2, 0x02, 127},
  {5, 0x02, 127},
  {13, 0x02, 127},
  {27, 0x03, 127},
  {2, 0x02, 128},
  {5, 0x02, 128},
  {13, 0x02, 128},
  {27, 0x03, 128},
  {2, 0x02, 129},
  {5, 0x02, 129},
  {13, 0x02, 129},
  {27, 0x03, 129},
  {2, 0x02, 130},
  {5, 0x02, 130},
  {13, 0x02, 130},
  {27, 0x03, 130},
},
/* 223 */
{
  {3, 0x02, 127},
  {4, 0x02, 127},
  {6, 0x02, 127},
  {9, 0x02, 127},
  {14, 0x02, 127},
  {20, 0x02, 127},
  {28, 0x02, 127},
  {38, 0x03, 127},
  {3, 0x02, 128},
  {4, 0x02, 128},
  {6, 0x02, 128},
  {9, 0x02, 128},
  {14, 0x02, 128},
  {20, 0x02, 128},
  {28, 0x02, 128},
  {38, 0x03, 128},
},
/* 224 */
{
  {3, 0x02, 129},
  {4, 0x02, 129},
  {6, 0x02, 129},
  {9, 0x02, 129},
  {14, 0x02, 129},
  {20, 0x02, 129},
  {28, 0x02, 129},
  {38, 0x03, 129},
  {3, 0x02, 130},
  {4, 0x02, 130},
  {6, 0x02, 130},
  {9, 0x02, 130},
  {14, 0x02, 130},
  {20, 0x02, 130},
  {28, 0x02, 130},
  {38, 0x03, 130},
},
/* 225 */
{
  {229, 0x00, 0},
  {230, 0x00, 0},
  {232, 0x00, 0},
  {233, 0x00, 0},
  {236, 0x00, 0},
  {237, 0x00, 0},
  {239, 0x00, 0},
  {240, 0x00, 0},
  {244, 0x00, 0},
  {245, 0x00, 0},
  {247, 0x00, 0},
  {248, 0x00, 0},
  {251, 0x00, 0},
  {252, 0x00, 0},
  {254, 0x00, 0},
  {255, 0x00, 0},
},
/* 226 */
{
  {0, 0x03, 131},
  {0, 0x03, 132},
  {0, 0x03, 133},
  {0, 0x03, 134},
  {0, 0x03, 135},
  {0, 0x03, 136},
  {0, 0x03, 137},
  {0, 0x03, 138},
  {0, 0x03, 139},
  {0, 0x03, 140},
  {0, 0x03, 141},
  {0, 0x03, 142},
  {0, 0x03, 143},
  {0, 0x03, 144},
  {0, 0x03, 145},
  {0, 0x03, 146},
},
/* 227 */
{
  {1, 0x02, 131},
  {12, 0x03, 131},
  {1, 0x02, 132},
  {12, 0x03, 132},
  {1, 0x02, 133},
  {12, 0x03, 133},
  {1, 0x02, 134},
  {12, 0x03, 134},
  {1, 0x02, 135},
  {12, 0x03, 135},
  {1, 0x02, 136},
  {12, 0x03, 136},
  {1, 0x02, 137},
  {12, 0x03, 137},
  {1, 0x02, 138},
  {12, 0x03, 138},
},
/* 228 */
{
  {2, 0x02, 131},
  {5, 0x02, 131},
  {13, 0x02, 131},
  {27, 0x03, 131},
  {2, 0x02, 132},
  {5, 0x02, 132},
  {13, 0x02, 132},
  {27, 0x03, 132},
  {2, 0x02, 133},
  {5, 0x02, 133},
  {13, 0x02, 133},
  {27, 0x03, 133},
  {2, 0x02, 134},
  {5, 0x02, 134},
  {13, 0x02, 134},
  {27, 0x03, 134},
},
/* 229 */
{
  {3, 0x02, 131},
  {4, 0x02, 131},
  {6, 0x02, 131},
  {9, 0x02, 131},
  {14, 0x02, 131},
  {20, 0x02, 131},
  {28, 0x02, 131},
  {38, 0x03, 131},
  {3, 0x02, 132},
  {4, 0x02, 132},
  {6, 0x02, 132},
  {9, 0x02, 132},
  {14, 0x02, 132},
  {20, 0x02, 132},
  {28, 0x02, 132},
  {38, 0x03, 132},
},
/* 230 */
{
  {3, 0x02, 133},
  {4, 0x02, 133},
  {6, 0x02, 133},
  {9, 0x02, 133},
  {14, 0x02, 133},
  {20, 0x02, 133},
  {28, 0x02, 133},
  {38, 0x03, 133},
  {3, 0x02, 134},
  {4, 0x02, 134},
  {6, 0x02, 134},
  {9, 0x02, 134},
  {14, 0x02, 134},
  {20, 0x02, 134},
  {28, 0x02, 134},
  {38, 0x03, 134},
},
/* 231 */
{
  {2, 0x02, 135},
  {5, 0x02, 135},
  {13, 0x02, 135},
  {27, 0x03, 135},
  {2, 0x02, 136},
  {5, 0x02, 136},
  {13, 0x02, 136},
  {27, 0x03, 136},
  {2, 0x02, 137},
  {5, 0x02, 137},
  {13, 0x02, 137},
  {27, 0x03, 137},
  {2, 0x02, 138},
  {5, 0x02, 138},
  {13, 0x02, 138},
  {27, 0x03, 138},
},
/* 232 */
{
  {3, 0x02, 135},
  {4, 0x02, 135},
  {6, 0x02, 135},
  {9, 0x02, 135},
  {14, 0x02, 135},
  {20, 0x02, 135},
  {28, 0x02, 135},
  {38, 0x03, 135},
  {3, 0x02, 136},
  {4, 0x02, 136},
  {6, 0x02, 136},
  {9, 0x02, 136},
  {14, 0x02, 136},
  {20, 0x02, 136},
  {28, 0x02, 136},
  {38, 0x03, 136},
},
/* 233 */
{
  {3, 0x02, 137},
  {4, 0x02, 137},
  {6, 0x02, 137},
  {9, 0x02, 137},
  {14, 0x02, 137},
  {20, 0x02, 137},
  {28, 0x02, 137},
  {38, 0x03, 137},
  {3, 0x02, 138},
  {4, 0x02, 138},
  {6, 0x02, 138},
  {9, 0x02, 138},
  {14, 0x02, 138},
  {20, 0x02, 138},
  {28, 0x02, 138},
  {38, 0x03, 138},
},
/* 234 */
{
  {1, 0x02, 139},
  {12, 0x03, 139},
  {1, 0x02, 140},
  {12, 0x03, 140},
  {1, 0x02, 141},
  {12, 0x03, 141},
  {1, 0x02, 142},
  {12, 0x03, 142},
  {1, 0x02, 143},
  {12, 0x03, 143},
  {1, 0x02, 144},
  {12, 0x03, 144},
  {1, 0x02, 145},
  {12, 0x03, 145},
  {1, 0x02, 146},
  {12, 0x03, 146},
},
/* 235 */
{
  {2, 0x02, 139},
  {5, 0x02, 139},
  {13, 0x02, 139},
  {27, 0x03, 139},
  {2, 0x02, 140},
  {5, 0x02, 140},
  {13, 0x02, 140},
  {27, 0x03, 140},
  {2, 0x02, 141},
  {5, 0x02, 141},
  {13, 0x02, 141},
  {27, 0x03, 141},
  {2, 0x02, 142},
  {5, 0x02, 142},
  {13, 0x02, 142},
  {27, 0x03, 142},
},
/* 236 */
{
  {3, 0x02, 139},
  {4, 0x02, 139},
  {6, 0x02, 139},
  {9, 0x02, 139},
  {14, 0x02, 139},
  {20, 0x02, 139},
  {28, 0x02, 139},
  {38, 0x03, 139},
  {3, 0x02, 140},
  {4, 0x02, 140},
  {6, 0x02, 140},
  {9, 0x02, 140},
  {14, 0x02, 140},
  {20, 0x02, 140},
  {28, 0x02, 140},
  {38, 0x03, 140},
},
/* 237 */
{
  {3, 0x02, 141},
  {4, 0x02, 141},
  {6, 0x02, 141},
  {9, 0x02, 141},
  {14, 0x02, 141},
  {20, 0x02, 141},
  {28, 0x02, 141},
  {38, 0x03, 141},
  {3, 0x02, 142},
  {4, 0x02, 142},
  {6, 0x02, 142},
  {9, 0x02, 142},
  {14, 0x02, 142},
  {20, 0x02, 142},
  {28, 0x02, 142},
  {38, 0x03, 142},
},
/* 238 */
{
  {2, 0x02, 143},
  {5, 0x02, 143},
  {13, 0x02, 143},
  {27, 0x03, 143},
  {2, 0x02, 144},
  {5, 0x02, 144},
  {13, 0x02, 144},
  {27, 0x03, 144},
  {2, 0x02, 145},
  {5, 0x02, 145},
  {13, 0x02, 145},
  {27, 0x03, 145},
  {2, 0x02, 146},
  {5, 0x02, 146},
  {13, 0x02, 146},
  {27, 0x03, 146},
},
/* 239 */
{
  {3, 0x02, 143},
  {4, 0x02, 143},
  {6, 0x02, 143},
  {9, 0x02, 143},
  {14, 0x02, 143},
  {20, 0x02, 143},
  {28, 0x02, 143},
  {38, 0x03, 143},
  {3, 0x02, 144},
  {4, 0x02, 144},
  {6, 0x02, 144},
  {9, 0x02, 144},
  {14, 0x02, 144},
  {20, 0x02, 144},
  {28, 0x02, 144},
  {38, 0x03, 144},
},
/* 240 */
{
  {3, 0x02, 145},
  {4, 0x02, 145},
  {6, 0x02, 145},
  {9, 0x02, 145},
  {14, 0x02, 145},
  {20, 0x02, 145},
  {28, 0x02, 145},
  {38, 0x03, 145},
  {3, 0x02, 146},
  {4, 0x02, 146},
  {6, 0x02, 146},
  {9, 0x02, 146},
  {14, 0x02, 146},
  {20, 0x02, 146},
  {28, 0x02, 146},
  {38, 0x03, 146},
},
/* 241 */
{
  {0, 0x03, 147},
  {0, 0x03, 148},
  {0, 0x03, 149},
  {0, 0x03, 150},
  {0, 0x03, 151},
  {0, 0x03, 152},
  {0, 0x03, 153},
  {0, 0x03, 154},
  {0, 0x03, 155},
  {0, 0x03, 156},
  {0, 0x03, 157},
  {0, 0x03, 158},
  {0, 0x03, 159},
  {0, 0x03, 160},
  {0, 0x03, 161},
  {0, 0x03, 162},
},
/* 242 */
{
  {1, 0x02, 147},
  {12, 0x03, 147},
  {1, 0x02, 148},
  {12, 0x03, 148},
  {1, 0x02, 149},
  {12, 0x03, 149},
  {1, 0x02, 150},
  {12, 0x03, 150},
  {1, 0x02, 151},
  {12, 0x03, 151},
  {1, 0x02, 152},
  {12, 0x03, 152},
  {1, 0x02, 153},
  {12, 0x03, 153},
  {1, 0x02, 154},
  {12, 0x03, 154},
},
/* 243 */
{
  {2, 0x02, 147},
  {5, 0x02, 147},
  {13, 0x02, 147},
  {27, 0x03, 147},
  {2, 0x02, 148},
  {5, 0x02, 148},
  {13, 0x02, 148},
  {27, 0x03, 148},
  {2, 0x02, 149},
  {5, 0x02, 149},
  {13, 0x02, 149},
  {27, 0x03, 149},
  {2, 0x02, 150},
  {5, 0x02, 150},
  {13, 0x02, 150},
  {27, 0x03, 150},
},
/* 244 */
{
  {3, 0x02, 147},
  {4, 0x02, 147},
  {6, 0x02, 147},
  {9, 0x02, 147},
  {14, 0x02, 147},
  {20, 0x02, 147},
  {28, 0x02, 147},
  {38, 0x03, 147},
  {3, 0x02, 148},
  {4, 0x02, 148},
  {6, 0x02, 148},
  {9, 0x02, 148},
  {14, 0x02, 148},
  {20, 0x02, 148},
  {28, 0x02, 148},
  {38, 0x03, 148},
},
/* 245 */
{
  {3, 0x02, 149},
  {4, 0x02, 149},
  {6, 0x02, 149},
  {9, 0x02, 149},
  {14, 0x02, 149},
  {20, 0x02, 149},
  {28, 0x02, 149},
  {38, 0x03, 149},
  {3, 0x02, 150},
  {4, 0x02, 150},
  {6, 0x02, 150},
  {9, 0x02, 150},
  {14, 0x02, 150},
  {20, 0x02, 150},
  {28, 0x02, 150},
  {38, 0x03, 150},
},
/* 246 */
{
  {2, 0x02, 151},
  {5, 0x02, 151},
  {13, 0x02, 151},
  {27, 0x03, 151},
  {2, 0x02, 152},
  {5, 0x02, 152},
  {13, 0x02, 152},
  {27, 0x03, 152},
  {2, 0x02, 153},
  {5, 0x02, 153},
  {13, 0x02, 153},
  {27, 0x03, 153},
  {2, 0x02, 154},
  {5, 0x02, 154},
  {13, 0x02, 154},
  {27, 0x03, 154},
},
/* 247 */
{
  {3, 0x02, 151},
  {4, 0x02, 151},
  {6, 0x02, 151},
  {9, 0x02, 151},
  {14, 0x02, 151},
  {20, 0x02, 151},
  {28, 0x02, 151},
  {38, 0x03, 151},
  {3, 0x02, 152},
  {4, 0x02, 152},
  {6, 0x02, 152},
  {9, 0x02, 152},
  {14, 0x02, 152},
  {20, 0x02, 152},
  {28, 0x02, 152},
  {38, 0x03, 152},
},
/* 248 */
{
  {3, 0x02, 153},
  {4, 0x02, 153},
  {6, 0x02, 153},
  {9, 0x02, 153},
  {14, 0x02, 153},
  {20, 0x02, 153},
  {28, 0x02, 153},
  {38, 0x03, 153},
  {3, 0x02, 154},
  {4, 0x02, 154},
  {6, 0x02, 154},
  {9, 0x02, 154},
  {14, 0x02, 154},
  {20, 0x02, 154},
  {28, 0x02, 154},
  {38, 0x03, 154},
},
/* 249 */
{
  {1, 0x02, 155},
  {12, 0x03, 155},
  {1, 0x02, 156},
  {12, 0x03, 156},
  {1, 0x02, 157},
  {12, 0x03, 157},
  {1, 0x02, 158},
  {12, 0x03, 158},
  {1, 0x02, 159},
  {12, 0x03, 159},
  {1, 0x02, 160},
  {12, 0x03, 160},
  {1, 0x02, 161},
  {12, 0x03, 161},
  {1, 0x02, 162},
  {12, 0x03, 162},
},
/* 250 */
{
  {2, 0x02, 155},
  {5, 0x02, 155},
  {13, 0x02, 155},
  {27, 0x03, 155},
  {2, 0x02, 156},
  {5, 0x02, 156},
  {13, 0x02, 156},
  {27, 0x03, 156},
  {2, 0x02, 157},
  {5, 0x02, 157},
  {13, 0x02, 157},
  {27, 0x03, 157},
  {2, 0x02, 158},
  {5, 0x02, 158},
  {13, 0x02, 158},
  {27, 0x03, 158},
},
/* 251 */
{
  {3, 0x02, 155},
  {4, 0x02, 155},
  {6, 0x02, 155},
  {9, 0x02, 155},
  {14, 0x02, 155},
  {20, 0x02, 155},
  {28, 0x02, 155},
  {38, 0x03, 155},
  {3, 0x02, 156},
  {4, 0x02, 156},
  {6, 0x02, 156},
  {9, 0x02, 156},
  {14, 0x02, 156},
  {20, 0x02, 156},
  {28, 0x02, 156},
  {38, 0x03, 156},
},
/* 252 */
{
  {3, 0x02, 157},
  {4, 0x02, 157},
  {6, 0x02, 157},
  {9, 0x02, 157},
  {14, 0x02, 157},
  {20, 0x02, 157},
  {28, 0x02, 157},
  {38, 0x03, 157},
  {3, 0x02, 158},
  {4, 0x02, 158},
  {6, 0x02, 158},
  {9, 0x02, 158},
  {14, 0x02, 158},
  {20, 0x02, 158},
  {28, 0x02, 158},
  {38, 0x03, 158},
},
/* 253 */
{
  {2, 0x02, 159},
  {5, 0x02, 159},
  {13, 0x02, 159},
  {27, 0x03, 159},
  {2, 0x02, 160},
  {5, 0x02, 160},
  {13, 0x02, 160},
  {27, 0x03, 160},
  {2, 0x02, 161},
  {5, 0x02, 161},
  {13, 0x02, 161},
  {27, 0x03, 161},
  {2, 0x02, 162},
  {5, 0x02, 162},
  {13, 0x02, 162},
  {27, 0x03, 162},
},
/* 254 */
{
  {3, 0x02, 159},
  {4, 0x02, 159},
  {6, 0x02, 159},
  {9, 0x02, 159},
  {14, 0x02, 159},
  {20, 0x02, 159},
  {28, 0x02, 159},
  {38, 0x03, 159},
  {3, 0x02, 160},
  {4, 0x02, 160},
  {6, 0x02, 160},
  {9, 0x02, 160},
  {14, 0x02, 160},
  {20, 0x02, 160},
  {28, 0x02, 160},
  {38, 0x03, 160},
},
/* 255 */
{
  {3, 0x02, 161},
  {4, 0x02, 161},
  {6, 0x02, 161},
  {9, 0x02, 161},
  {14, 0x02, 161},
  {20, 0x02, 161},
  {28, 0x02, 161},
  {38, 0x03, 161},
  {3, 0x02, 162},
  {4, 0x02, 162},
  {6, 0x02, 162},
  {9, 0x02, 162},
  {14, 0x02, 162},
  {20, 0x02, 162},
  {28, 0x02, 162},
  {38, 0x03, 162},
},
};
